@@ -26,8 +26,11 @@
 #include "Accessor.h"
 #include "KeyWords.h"
 #endif
-#include "ContractionState.h"
 #include "SVector.h"
+#include "SplitVector.h"
+#include "Partitioning.h"
+#include "RunStyles.h"
+#include "ContractionState.h"
 #include "CellBuffer.h"
 #include "CallTip.h"
 #include "KeyMap.h"
@@ -37,8 +40,10 @@
 #include "Style.h"
 #include "AutoComplete.h"
 #include "ViewStyle.h"
+#include "Decoration.h"
 #include "CharClassify.h"
 #include "Document.h"
+#include "PositionCache.h"
 #include "Editor.h"
 #include "SString.h"
 #include "ScintillaBase.h"
@@ -113,6 +118,7 @@ class ScintillaGTK : public ScintillaBase {
 	static GdkAtom atomUTF8;
 	static GdkAtom atomString;
 	static GdkAtom atomUriList;
+	static GdkAtom atomDROPFILES_DND;
 	GdkAtom atomSought;
 
 #if PLAT_GTK_WIN32
@@ -151,9 +157,11 @@ private:
 	virtual void Initialise();
 	virtual void Finalise();
 	virtual void DisplayCursor(Window::Cursor c);
+	virtual bool DragThreshold(Point ptStart, Point ptNow);
 	virtual void StartDrag();
 	int TargetAsUTF8(char *text);
 	int EncodedFromUTF8(char *utf8, char *encoded);
+	virtual bool ValidCodePage(int codePage) const;
 public: 	// Public for scintilla_send_message
 	virtual sptr_t WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 private:
@@ -176,7 +184,6 @@ private:
 	virtual void NotifyParent(SCNotification scn);
 	void NotifyKey(int key, int modifiers);
 	void NotifyURIDropped(const char *list);
-	bool UseInputMethod() const;
 	const char *CharacterSetID() const;
 	virtual int KeyDefault(int key, int modifiers);
 	virtual void CopyToClipboard(const SelectionText &selectedText);
@@ -191,6 +198,7 @@ private:
 	void ReceivedDrop(GtkSelectionData *selection_data);
 	static void GetSelection(GtkSelectionData *selection_data, guint info, SelectionText *selected);
 #ifdef USE_GTK_CLIPBOARD
+	void StoreOnClipboard(SelectionText *clipText);
 	static void ClipboardGetSelection(GtkClipboard* clip, GtkSelectionData *selection_data, guint info, void *data);
 	static void ClipboardClearSelection(GtkClipboard* clip, void *data);
 #endif
@@ -254,8 +262,9 @@ private:
 	static gint SelectionNotify(GtkWidget *widget, GdkEventSelection *selection_event);
 #endif
 	static void DragBegin(GtkWidget *widget, GdkDragContext *context);
+	gboolean DragMotionThis(GdkDragContext *context, gint x, gint y, guint dragtime);
 	static gboolean DragMotion(GtkWidget *widget, GdkDragContext *context,
-	                           gint x, gint y, guint time);
+	                           gint x, gint y, guint dragtime);
 	static void DragLeave(GtkWidget *widget, GdkDragContext *context,
 	                      guint time);
 	static void DragEnd(GtkWidget *widget, GdkDragContext *context);
@@ -302,13 +311,20 @@ GdkAtom ScintillaGTK::atomClipboard = 0;
 GdkAtom ScintillaGTK::atomUTF8 = 0;
 GdkAtom ScintillaGTK::atomString = 0;
 GdkAtom ScintillaGTK::atomUriList = 0;
+GdkAtom ScintillaGTK::atomDROPFILES_DND = 0;
 
-static const GtkTargetEntry clipboardTargets[] = {
+static const GtkTargetEntry clipboardCopyTargets[] = {
+	{ "UTF8_STRING", 0, TARGET_UTF8_STRING },
+	{ "STRING", 0, TARGET_STRING },
+};
+static const gint nClipboardCopyTargets = sizeof(clipboardCopyTargets) / sizeof(clipboardCopyTargets[0]);
+
+static const GtkTargetEntry clipboardPasteTargets[] = {
 	{ "text/uri-list", 0, TARGET_URI },
 	{ "UTF8_STRING", 0, TARGET_UTF8_STRING },
 	{ "STRING", 0, TARGET_STRING },
 };
-static const gint nClipboardTargets = sizeof(clipboardTargets) / sizeof(clipboardTargets[0]);
+static const gint nClipboardPasteTargets = sizeof(clipboardPasteTargets) / sizeof(clipboardPasteTargets[0]);
 
 static GtkWidget *PWidget(Window &w) {
 	return reinterpret_cast<GtkWidget *>(w.GetID());
@@ -756,17 +772,37 @@ void ScintillaGTK::Initialise() {
 	gtk_widget_grab_focus(PWidget(wMain));
 
 	gtk_selection_add_targets(GTK_WIDGET(PWidget(wMain)), GDK_SELECTION_PRIMARY,
-	                          clipboardTargets, nClipboardTargets);
+	                          clipboardCopyTargets, nClipboardCopyTargets);
 
 #ifndef USE_GTK_CLIPBOARD
 	gtk_selection_add_targets(GTK_WIDGET(PWidget(wMain)), atomClipboard,
-	                          clipboardTargets, nClipboardTargets);
+	                          clipboardPasteTargets, nClipboardPasteTargets);
 #endif
 
 	gtk_drag_dest_set(GTK_WIDGET(PWidget(wMain)),
-	                  GTK_DEST_DEFAULT_ALL, clipboardTargets, nClipboardTargets,
+	                  GTK_DEST_DEFAULT_ALL, clipboardPasteTargets, nClipboardPasteTargets,
 	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
 
+#if GLIB_MAJOR_VERSION >= 2
+	// Set caret period based on GTK settings
+	gboolean blinkOn = false;
+	if (g_object_class_find_property(G_OBJECT_GET_CLASS(
+			G_OBJECT(gtk_settings_get_default())), "gtk-cursor-blink")) {
+		g_object_get(G_OBJECT(
+			gtk_settings_get_default()), "gtk-cursor-blink", &blinkOn, NULL);
+	}
+	if (blinkOn &&
+		g_object_class_find_property(G_OBJECT_GET_CLASS(
+			G_OBJECT(gtk_settings_get_default())), "gtk-cursor-blink-time")) {
+		gint value;
+		g_object_get(G_OBJECT(
+			gtk_settings_get_default()), "gtk-cursor-blink-time", &value, NULL);
+		caret.period = gint(value / 1.75);
+	} else {
+		caret.period = 0;
+	}
+#endif
+
 	SetTicking(true);
 }
 
@@ -782,14 +818,19 @@ void ScintillaGTK::DisplayCursor(Window::Cursor c) {
 		wText.SetCursor(static_cast<Window::Cursor>(cursorMode));
 }
 
+bool ScintillaGTK::DragThreshold(Point ptStart, Point ptNow) {
+#if GTK_MAJOR_VERSION < 2
+	return Editor::DragThreshold(ptStart, ptNow);
+#else
+	return gtk_drag_check_threshold(GTK_WIDGET(PWidget(wMain)),
+		ptStart.x, ptStart.y, ptNow.x, ptNow.y);
+#endif
+}
+
 void ScintillaGTK::StartDrag() {
 	dragWasDropped = false;
-	static const GtkTargetEntry targets[] = {
-	    { "UTF8_STRING", 0, TARGET_UTF8_STRING },
-	    { "STRING", 0, TARGET_STRING },
-	};
-	static const gint n_targets = sizeof(targets) / sizeof(targets[0]);
-	GtkTargetList *tl = gtk_target_list_new(targets, n_targets);
+	inDragDrop = ddDragging;
+	GtkTargetList *tl = gtk_target_list_new(clipboardCopyTargets, nClipboardCopyTargets);
 	gtk_drag_begin(GTK_WIDGET(PWidget(wMain)),
 	               tl,
 	               static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE),
@@ -798,10 +839,11 @@ void ScintillaGTK::StartDrag() {
 }
 
 #ifdef USE_CONVERTER
-static char *ConvertText(int *lenResult, char *s, size_t len, const char *charSetDest, const char *charSetSource) {
+static char *ConvertText(int *lenResult, char *s, size_t len, const char *charSetDest,
+	const char *charSetSource, bool transliterations) {
 	*lenResult = 0;
 	char *destForm = 0;
-	Converter conv(charSetDest, charSetSource);
+	Converter conv(charSetDest, charSetSource, transliterations);
 	if (conv) {
 		destForm = new char[len*3+1];
 		char *pin = s;
@@ -849,7 +891,7 @@ int ScintillaGTK::TargetAsUTF8(char *text) {
 				pdoc->GetCharRange(s, targetStart, targetLength);
 //~ fprintf(stderr, "    \"%s\"\n", s);
 				if (text) {
-					char *tmputf = ConvertText(&targetLength, s, targetLength, "UTF-8", charSetBuffer);
+					char *tmputf = ConvertText(&targetLength, s, targetLength, "UTF-8", charSetBuffer, false);
 					memcpy(text, tmputf, targetLength);
 					delete []tmputf;
 //~ fprintf(stderr, "    \"%s\"\n", text);
@@ -886,7 +928,7 @@ int ScintillaGTK::EncodedFromUTF8(char *utf8, char *encoded) {
 		if (*charSetBuffer) {
 //~ fprintf(stderr, "Encode %s %d\n", charSetBuffer, inputLength);
 			int outLength = 0;
-			char *tmpEncoded = ConvertText(&outLength, utf8, inputLength, charSetBuffer, "UTF-8");
+			char *tmpEncoded = ConvertText(&outLength, utf8, inputLength, charSetBuffer, "UTF-8", true);
 			if (tmpEncoded) {
 //~ fprintf(stderr, "    \"%s\"\n", tmpEncoded);
 				if (encoded) {
@@ -907,6 +949,10 @@ int ScintillaGTK::EncodedFromUTF8(char *utf8, char *encoded) {
 	return 0;
 }
 
+bool ScintillaGTK::ValidCodePage(int codePage) const {
+	return codePage == 0 || codePage == SC_CP_UTF8 || codePage == SC_CP_DBCS;
+}
+
 sptr_t ScintillaGTK::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam) {
 	switch (iMessage) {
 
@@ -1143,10 +1189,16 @@ bool ScintillaGTK::ModifyScrollBars(int nMax, int nPage) {
 	if (horizEndPreferred < 0)
 		horizEndPreferred = 0;
 	unsigned int pageWidth = rcText.Width();
+	unsigned int pageIncrement = pageWidth / 3;
+	unsigned int charWidth = vs.styles[STYLE_DEFAULT].aveCharWidth;
 	if (GTK_ADJUSTMENT(adjustmenth)->upper != horizEndPreferred ||
-	        GTK_ADJUSTMENT(adjustmenth)->page_size != pageWidth) {
+	        GTK_ADJUSTMENT(adjustmenth)->page_size != pageWidth ||
+	        GTK_ADJUSTMENT(adjustmenth)->page_increment != pageIncrement ||
+	        GTK_ADJUSTMENT(adjustmenth)->step_increment != charWidth) {
 		GTK_ADJUSTMENT(adjustmenth)->upper = horizEndPreferred;
+		GTK_ADJUSTMENT(adjustmenth)->step_increment = charWidth;
 		GTK_ADJUSTMENT(adjustmenth)->page_size = pageWidth;
+		GTK_ADJUSTMENT(adjustmenth)->page_increment = pageIncrement;
 		gtk_adjustment_changed(GTK_ADJUSTMENT(adjustmenth));
 		modified = true;
 	}
@@ -1209,117 +1261,17 @@ void ScintillaGTK::NotifyURIDropped(const char *list) {
 	NotifyParent(scn);
 }
 
-bool ScintillaGTK::UseInputMethod() const {
-	switch (vs.styles[STYLE_DEFAULT].characterSet) {
-	case SC_CHARSET_CHINESEBIG5:
-	case SC_CHARSET_GB2312:
-	case SC_CHARSET_HANGUL:
-	case SC_CHARSET_SHIFTJIS:
-	case SC_CHARSET_JOHAB:
-	case SC_CHARSET_HEBREW:
-	case SC_CHARSET_ARABIC:
-	case SC_CHARSET_VIETNAMESE:
-	case SC_CHARSET_THAI:
-		return true;
-	default:
-		return false;
-	}
-}
-
 const char *CharacterSetID(int characterSet);
 
 const char *ScintillaGTK::CharacterSetID() const {
 	return ::CharacterSetID(vs.styles[STYLE_DEFAULT].characterSet);
 }
 
-#if GTK_MAJOR_VERSION >= 2
-#define IS_ACC(x) \
-	((x) >= 65103 && (x) <= 65111)
-#define IS_CHAR(x) \
-	((x) >= 0 && (x) <= 128)
-
-#define IS_ACC_OR_CHAR(x) \
-	(IS_CHAR(x)) || (IS_ACC(x))
-
-static int MakeAccent(int key, int acc) {
-	const char *conv[] = {
-		"aeiounc AEIOUNC",
-		"eiuc~EIUC",
-		"n'N",
-		"nc`NC",
-		"nc^NC",
-		"ncNC"
-	};
-	int idx;
-	for (idx = 0; idx < 15; ++idx) {
-		if (char(key) == conv[0][idx]) {
-			break;
-		}
-	}
-	if (idx == 15) {
-		return key;
-	}
-	if (acc == GDK_dead_tilde) { // ~
-		return int((unsigned char)(conv[1][idx]));
-	} else if (acc == GDK_dead_acute) { // '
-		return int((unsigned char)(conv[2][idx]));
-	} else if (acc == GDK_dead_grave) { // `
-		return int((unsigned char)(conv[3][idx]));
-	} else if (acc == GDK_dead_circumflex) { // ^
-		return int((unsigned char)(conv[4][idx]));
-	} else if (acc == GDK_dead_diaeresis) { // "
-		return int((unsigned char)(conv[5][idx]));
-	}
-	return key;
-}
-#endif
-
 int ScintillaGTK::KeyDefault(int key, int modifiers) {
 	if (!(modifiers & SCI_CTRL) && !(modifiers & SCI_ALT)) {
-#if GTK_MAJOR_VERSION >= 2
-		if (!UseInputMethod()) {
-			char utfVal[4]="\0\0\0";
-			wchar_t wcs[2];
-			if (IS_CHAR(key) && IS_ACC(lastKey)) {
-				lastKey = key = MakeAccent(key, lastKey);
-			}
-			if (IS_ACC_OR_CHAR(key)) {
-				lastKey = key;
-			}
-			wcs[0] = gdk_keyval_to_unicode(key);
-			wcs[1] = 0;
-			UTF8FromUCS2(wcs, 1, utfVal, 3);
-			if (key <= 0xFE00) {
-				if (IsUnicodeMode()) {
-					AddCharUTF(utfVal,strlen(utfVal));
-					return 1;
-				} else {
-					const char *source = CharacterSetID();
-					if (*source) {
-						Converter conv(source, "UTF-8");
-						if (conv) {
-							char localeVal[4]="\0\0\0";
-							char *pin = utfVal;
-							size_t inLeft = strlen(utfVal);
-							char *pout = localeVal;
-							size_t outLeft = sizeof(localeVal);
-							size_t conversions = conv.Convert(&pin, &inLeft, &pout, &outLeft);
-							if (conversions != ((size_t)(-1))) {
-								*pout = '\0';
-								for (int i=0; localeVal[i]; i++) {
-									AddChar(localeVal[i]);
-								}
-								return 1;
-							}
-						}
-					}
-				}
-			}
-		}
-#endif
 		if (key < 256) {
-			AddChar(key);
-			return 1;
+			NotifyKey(key, modifiers);
+			return 0;
 		} else {
 			// Pass up to container in case it is an accelerator
 			NotifyKey(key, modifiers);
@@ -1340,17 +1292,9 @@ void ScintillaGTK::CopyToClipboard(const SelectionText &selectedText) {
 				atomClipboard,
 				GDK_CURRENT_TIME);
 #else
-	GtkClipboard *clipBoard;
-	clipBoard = gtk_widget_get_clipboard(GTK_WIDGET(PWidget(wMain)), atomClipboard);
-	if (clipBoard == NULL) // Occurs if widget isn't in a toplevel
-		return;
-
 	SelectionText *clipText = new SelectionText();
 	clipText->Copy(selectedText);
-
-	gtk_clipboard_set_with_data(clipBoard, clipboardTargets, nClipboardTargets,
-				    ClipboardGetSelection, ClipboardClearSelection, clipText);
-
+	StoreOnClipboard(clipText);
 #endif
 }
 
@@ -1362,17 +1306,9 @@ void ScintillaGTK::Copy() {
 		                        atomClipboard,
 		                        GDK_CURRENT_TIME);
 #else
-		GtkClipboard *clipBoard;
-		clipBoard = gtk_widget_get_clipboard(GTK_WIDGET(PWidget(wMain)), atomClipboard);
-		if (clipBoard == NULL) // Occurs if widget isn't in a toplevel
-			return;
-
 		SelectionText *clipText = new SelectionText();
 		CopySelectionRange(clipText);
-
-		gtk_clipboard_set_with_data(clipBoard, clipboardTargets, nClipboardTargets,
-					    ClipboardGetSelection, ClipboardClearSelection, clipText);
-
+		StoreOnClipboard(clipText);
 #endif
 #if PLAT_GTK_WIN32
 		if (selType == selRectangle) {
@@ -1510,7 +1446,7 @@ void ScintillaGTK::GetGtkSelectionText(GtkSelectionData *selectionData, Selectio
 		if (!IsUnicodeMode() && *charSetBuffer) {
 //fprintf(stderr, "Convert to locale %s\n", CharacterSetID());
 				// Convert to locale
-				dest = ConvertText(&len, selText.s, selText.len, charSetBuffer, "UTF-8");
+				dest = ConvertText(&len, selText.s, selText.len, charSetBuffer, "UTF-8", true);
 				selText.Set(dest, len, pdoc->dbcsCodePage,
 					vs.styles[STYLE_DEFAULT].characterSet, selText.rectangular);
 		}
@@ -1553,7 +1489,7 @@ void ScintillaGTK::ReceivedSelection(GtkSelectionData *selection_data) {
 
 void ScintillaGTK::ReceivedDrop(GtkSelectionData *selection_data) {
 	dragWasDropped = true;
-	if (selection_data->type == atomUriList) {
+	if (selection_data->type == atomUriList || selection_data->type == atomDROPFILES_DND) {
 		char *ptr = new char[selection_data->length + 1];
 		ptr[selection_data->length] = '\0';
 		memcpy(ptr, selection_data->data, selection_data->length);
@@ -1595,7 +1531,7 @@ void ScintillaGTK::GetSelection(GtkSelectionData *selection_data, guint info, Se
 		const char *charSet = ::CharacterSetID(text->characterSet);
 		if (*charSet) {
 			int new_len;
-			char* tmputf = ConvertText(&new_len, text->s, text->len, "UTF-8", charSet);
+			char* tmputf = ConvertText(&new_len, text->s, text->len, "UTF-8", charSet, false);
 			converted = new SelectionText();
 			converted->Set(tmputf, new_len, SC_CP_UTF8, 0, text->rectangular);
 			text = converted;
@@ -1638,14 +1574,14 @@ void ScintillaGTK::GetSelection(GtkSelectionData *selection_data, guint info, Se
 			if (text->codePage != SC_CP_UTF8) {
 				// Convert to UTF-8
 	//fprintf(stderr, "Convert to UTF-8 from %s\n", charSetBuffer);
-				tmputf = ConvertText(&len, selBuffer, len, "UTF-8", charSetBuffer);
+				tmputf = ConvertText(&len, selBuffer, len, "UTF-8", charSetBuffer, false);
 				selBuffer = tmputf;
 			}
 		} else if (info == TARGET_STRING) {
 			if (text->codePage == SC_CP_UTF8) {
 	//fprintf(stderr, "Convert to locale %s\n", charSetBuffer);
 				// Convert to locale
-				tmputf = ConvertText(&len, selBuffer, len, charSetBuffer, "UTF-8");
+				tmputf = ConvertText(&len, selBuffer, len, charSetBuffer, "UTF-8", true);
 				selBuffer = tmputf;
 			}
 		}
@@ -1688,6 +1624,18 @@ void ScintillaGTK::GetSelection(GtkSelectionData *selection_data, guint info, Se
 }
 
 #ifdef USE_GTK_CLIPBOARD
+void ScintillaGTK::StoreOnClipboard(SelectionText *clipText) {
+	GtkClipboard *clipBoard =
+		gtk_widget_get_clipboard(GTK_WIDGET(PWidget(wMain)), atomClipboard);
+	if (clipBoard == NULL) // Occurs if widget isn't in a toplevel
+		return;
+
+	if (gtk_clipboard_set_with_data(clipBoard, clipboardCopyTargets, nClipboardCopyTargets,
+				    ClipboardGetSelection, ClipboardClearSelection, clipText)) {
+		gtk_clipboard_set_can_store(clipBoard, clipboardCopyTargets, nClipboardCopyTargets);
+	}
+}
+
 void ScintillaGTK::ClipboardGetSelection(GtkClipboard *, GtkSelectionData *selection_data, guint info, void *data) {
 	GetSelection(selection_data, info, static_cast<SelectionText*>(data));
 }
@@ -1835,13 +1783,13 @@ gint ScintillaGTK::PressThis(GdkEventButton *event) {
 			return FALSE;
 		}
 	} else if (event->button == 4) {
-		// Wheel scrolling up (only xwin gtk does it this way)
+		// Wheel scrolling up (only GTK 1.x does it this way)
 		if (ctrl)
 			SetAdjustmentValue(adjustmenth, (xOffset / 2) - 6);
 		else
 			SetAdjustmentValue(adjustmentv, topLine - 3);
 	} else if (event->button == 5) {
-		// Wheel scrolling down (only xwin gtk does it this way)
+		// Wheel scrolling down (only GTK 1.x does it this way)
 		if (ctrl)
 			SetAdjustmentValue(adjustmenth, (xOffset / 2) + 6);
 		else
@@ -1881,8 +1829,8 @@ gint ScintillaGTK::MouseRelease(GtkWidget *widget, GdkEventButton *event) {
 	return FALSE;
 }
 
-// win32gtk has a special wheel mouse event for whatever reason and doesn't
-// use the button4/5 trick used under X windows.
+// win32gtk and GTK >= 2 use SCROLL_* events instead of passing the
+// button4/5/6/7 events to the GTK app
 #if PLAT_GTK_WIN32 || (GTK_MAJOR_VERSION >= 2)
 gint ScintillaGTK::ScrollEvent(GtkWidget *widget,
                                GdkEventScroll *event) {
@@ -1894,7 +1842,15 @@ gint ScintillaGTK::ScrollEvent(GtkWidget *widget,
 	// Compute amount and direction to scroll (even tho on win32 there is
 	// intensity of scrolling info in the native message, gtk doesn't
 	// support this so we simulate similarly adaptive scrolling)
+	// Note that this is disabled on OS X (Darwin) where the X11 server already has
+	// and adaptive scrolling algorithm that fights with this one
 	int cLineScroll;
+#if defined(__MWERKS__) || defined(__APPLE_CPP__) || defined(__APPLE_CC__)
+	cLineScroll = sciThis->linesPerScroll;
+	if (cLineScroll == 0)
+		cLineScroll = 4;
+	sciThis->wheelMouseIntensity = cLineScroll;
+#else
 	int timeDelta = 1000000;
 	GTimeVal curTime;
 	g_get_current_time(&curTime);
@@ -1912,7 +1868,8 @@ gint ScintillaGTK::ScrollEvent(GtkWidget *widget,
 			cLineScroll = 4;
 		sciThis->wheelMouseIntensity = cLineScroll;
 	}
-	if (event->direction == GDK_SCROLL_UP) {
+#endif
+	if (event->direction == GDK_SCROLL_UP || event->direction == GDK_SCROLL_LEFT) {
 		cLineScroll *= -1;
 	}
 	g_get_current_time(&sciThis->lastWheelMouseTime);
@@ -1928,21 +1885,23 @@ gint ScintillaGTK::ScrollEvent(GtkWidget *widget,
 		return FALSE;
 	}
 
+	// Horizontal scrolling
+	if (event->direction == GDK_SCROLL_LEFT || event->direction == GDK_SCROLL_RIGHT) {
+		sciThis->HorizontalScrollTo(sciThis->xOffset + cLineScroll);
+
 	// Text font size zoom
-	if (event->state & GDK_CONTROL_MASK) {
+	} else if (event->state & GDK_CONTROL_MASK) {
 		if (cLineScroll < 0) {
 			sciThis->KeyCommand(SCI_ZOOMIN);
-			return TRUE;
 		} else {
 			sciThis->KeyCommand(SCI_ZOOMOUT);
-			return TRUE;
 		}
 
 	// Regular scrolling
 	} else {
 		sciThis->ScrollTo(sciThis->topLine + cLineScroll);
-		return TRUE;
 	}
+	return TRUE;
 }
 #endif
 
@@ -2030,19 +1989,23 @@ static int KeyTranslate(int keyIn) {
 		return SCK_SUBTRACT;
 	case GDK_KP_Divide:
 		return SCK_DIVIDE;
+	case GDK_Super_L:
+		return SCK_WIN;
+	case GDK_Super_R:
+		return SCK_RWIN;
+	case GDK_Menu:
+		return SCK_MENU;
 	default:
 		return keyIn;
 	}
 }
 
 gboolean ScintillaGTK::KeyThis(GdkEventKey *event) {
-	//Platform::DebugPrintf("SC-key: %d %x [%s]\n",
+	//fprintf(stderr, "SC-key: %d %x [%s]\n",
 	//	event->keyval, event->state, (event->length > 0) ? event->string : "empty");
 #if GTK_MAJOR_VERSION >= 2
-	if (UseInputMethod()) {
-		if (gtk_im_context_filter_keypress(im_context, event)) {
-			return 1;
-		}
+	if (gtk_im_context_filter_keypress(im_context, event)) {
+		return 1;
 	}
 #endif
 	if (!event->keyval) {
@@ -2061,10 +2024,8 @@ gboolean ScintillaGTK::KeyThis(GdkEventKey *event) {
 	// This will have to change for Unicode
 	else if (key >= 0xFE00)
 		key = KeyTranslate(key);
-	else if (IsUnicodeMode())
-		;	// No operation
 #if GTK_MAJOR_VERSION < 2
-	else if ((key >= 0x100) && (key < 0x1000))
+	else if (!IsUnicodeMode() && (key >= 0x100) && (key < 0x1000))
 		key &= 0xff;
 #endif
 
@@ -2072,10 +2033,10 @@ gboolean ScintillaGTK::KeyThis(GdkEventKey *event) {
 	bool added = KeyDown(key, shift, ctrl, alt, &consumed) != 0;
 	if (!consumed)
 		consumed = added;
-	//Platform::DebugPrintf("SK-key: %d %x %x\n",event->keyval, event->state, consumed);
+	//fprintf(stderr, "SK-key: %d %x %x\n",event->keyval, event->state, consumed);
 	if (event->keyval == 0xffffff && event->length > 0) {
 		ClearSelection();
-		if (pdoc->InsertString(CurrentPosition(), event->string)) {
+		if (pdoc->InsertCString(CurrentPosition(), event->string)) {
 			MovePositionTo(CurrentPosition() + event->length);
 		}
 	}
@@ -2131,8 +2092,32 @@ void ScintillaGTK::Commit(GtkIMContext *, char  *str, ScintillaGTK *sciThis) {
 	sciThis->CommitThis(str);
 }
 
-void ScintillaGTK::CommitThis(char *str) {
-	AddCharUTF(str, strlen(str));
+void ScintillaGTK::CommitThis(char *utfVal) {
+	//~ fprintf(stderr, "Commit '%s'\n", utfVal);
+	if (IsUnicodeMode()) {
+		AddCharUTF(utfVal,strlen(utfVal));
+	} else {
+		const char *source = CharacterSetID();
+		if (*source) {
+			Converter conv(source, "UTF-8", true);
+			if (conv) {
+				char localeVal[4]="\0\0\0";
+				char *pin = utfVal;
+				size_t inLeft = strlen(utfVal);
+				char *pout = localeVal;
+				size_t outLeft = sizeof(localeVal);
+				size_t conversions = conv.Convert(&pin, &inLeft, &pout, &outLeft);
+				if (conversions != ((size_t)(-1))) {
+					*pout = '\0';
+					for (int i=0; localeVal[i]; i++) {
+						AddChar(localeVal[i]);
+					}
+				} else {
+					fprintf(stderr, "Conversion failed '%s'\n", utfVal);
+				}
+			}
+		}
+	}
 }
 
 void ScintillaGTK::PreeditChanged(GtkIMContext *, ScintillaGTK *sciThis) {
@@ -2387,18 +2372,30 @@ void ScintillaGTK::DragBegin(GtkWidget *, GdkDragContext *) {
 	//Platform::DebugPrintf("DragBegin\n");
 }
 
-gboolean ScintillaGTK::DragMotion(GtkWidget *widget, GdkDragContext *context,
-                                  gint x, gint y, guint dragtime) {
-	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
-	//Platform::DebugPrintf("DragMotion %d %d %x %x %x\n", x, y,
-	//	context->actions, context->suggested_action, sciThis);
+gboolean ScintillaGTK::DragMotionThis(GdkDragContext *context,
+                                 gint x, gint y, guint dragtime) {
 	Point npt(x, y);
-	sciThis->inDragDrop = true;
-	sciThis->SetDragPosition(sciThis->PositionFromLocation(npt));
-	gdk_drag_status(context, context->suggested_action, dragtime);
+	SetDragPosition(PositionFromLocation(npt));
+	GdkDragAction preferredAction = context->suggested_action;
+	int pos = PositionFromLocation(npt);
+	if ((inDragDrop == ddDragging) && (0 == PositionInSelection(pos))) {
+		// Avoid dragging selection onto itself as that produces a move
+		// with no real effect but which creates undo actions.
+		preferredAction = static_cast<GdkDragAction>(0);
+	} else if (context->actions == static_cast<GdkDragAction>
+		(GDK_ACTION_COPY | GDK_ACTION_MOVE)) {
+		preferredAction = GDK_ACTION_MOVE;
+	}
+	gdk_drag_status(context, preferredAction, dragtime);
 	return FALSE;
 }
 
+gboolean ScintillaGTK::DragMotion(GtkWidget *widget, GdkDragContext *context,
+                                 gint x, gint y, guint dragtime) {
+	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
+	return sciThis->DragMotionThis(context, x, y, dragtime);
+}
+
 void ScintillaGTK::DragLeave(GtkWidget *widget, GdkDragContext * /*context*/, guint) {
 	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
 	sciThis->SetDragPosition(invalidPosition);
@@ -2412,6 +2409,7 @@ void ScintillaGTK::DragEnd(GtkWidget *widget, GdkDragContext * /*context*/) {
 		sciThis->SetEmptySelection(sciThis->posDrag);
 	sciThis->SetDragPosition(invalidPosition);
 	//Platform::DebugPrintf("DragEnd %x %d\n", sciThis, sciThis->dragWasDropped);
+	sciThis->inDragDrop = ddNone;
 }
 
 gboolean ScintillaGTK::Drop(GtkWidget *widget, GdkDragContext * /*context*/,
@@ -2578,6 +2576,7 @@ void ScintillaGTK::ClassInit(OBJECT_CLASS* object_class, GtkWidgetClass *widget_
 	atomUTF8 = gdk_atom_intern("UTF8_STRING", FALSE);
 	atomString = GDK_SELECTION_TYPE_STRING;
 	atomUriList = gdk_atom_intern("text/uri-list", FALSE);
+	atomDROPFILES_DND = gdk_atom_intern("DROPFILES_DND", FALSE);
 
 	// Define default signal handlers for the class:  Could move more
 	// of the signal handlers here (those that currently attached to wDraw
