@@ -42,11 +42,36 @@ static void *PointerFromWindow(HWND hWnd) {
 static void SetWindowPointer(HWND hWnd, void *ptr) {
 	::SetWindowLong(hWnd, 0, reinterpret_cast<LONG>(ptr));
 }
+
+#ifndef GWLP_USERDATA
+#define GWLP_USERDATA GWL_USERDATA
 #endif
 
+#ifndef GWLP_WNDPROC
+#define GWLP_WNDPROC GWL_WNDPROC
+#endif
+
+#ifndef LONG_PTR
+#define LONG_PTR LONG
+#endif
+
+static LONG_PTR SetWindowLongPtr(HWND hWnd, int nIndex, LONG_PTR dwNewLong) {
+	return ::SetWindowLong(hWnd, nIndex, dwNewLong);
+}
+
+static LONG_PTR GetWindowLongPtr(HWND hWnd, int nIndex) {
+	return ::GetWindowLong(hWnd, nIndex);
+}
+#endif
+
+typedef BOOL (WINAPI *AlphaBlendSig)(HDC, int, int, int, int, HDC, int, int, int, int, BLENDFUNCTION);
+
 static CRITICAL_SECTION crPlatformLock;
 static HINSTANCE hinstPlatformRes = 0;
 static bool onNT = false;
+static HMODULE hDLLImage = 0;
+static AlphaBlendSig AlphaBlendFn = 0;
+
 
 bool IsNT() {
 	return onNT;
@@ -65,10 +90,14 @@ Palette::Palette() {
 	used = 0;
 	allowRealization = false;
 	hpal = 0;
+	size = 100;
+	entries = new ColourPair[size];
 }
 
 Palette::~Palette() {
 	Release();
+	delete []entries;
+	entries = 0;
 }
 
 void Palette::Release() {
@@ -76,6 +105,9 @@ void Palette::Release() {
 	if (hpal)
 		::DeleteObject(hpal);
 	hpal = 0;
+	delete []entries;
+	size = 100;
+	entries = new ColourPair[size];
 }
 
 /**
@@ -90,11 +122,20 @@ void Palette::WantFind(ColourPair &cp, bool want) {
 				return;
 		}
 
-		if (used < numEntries) {
-			entries[used].desired = cp.desired;
-			entries[used].allocated.Set(cp.desired.AsLong());
-			used++;
+		if (used >= size) {
+			int sizeNew = size * 2;
+			ColourPair *entriesNew = new ColourPair[sizeNew];
+			for (int j=0; j<size; j++) {
+				entriesNew[j] = entries[j];
+			}
+			delete []entries;
+			entries = entriesNew;
+			size = sizeNew;
 		}
+
+		entries[used].desired = cp.desired;
+		entries[used].allocated.Set(cp.desired.AsLong());
+		used++;
 	} else {
 		for (int i=0; i < used; i++) {
 			if (entries[i].desired == cp.desired) {
@@ -134,7 +175,7 @@ void Palette::Allocate(Window &) {
 	}
 }
 
-static void SetLogFont(LOGFONT &lf, const char *faceName, int characterSet, int size, bool bold, bool italic) {
+static void SetLogFont(LOGFONTA &lf, const char *faceName, int characterSet, int size, bool bold, bool italic) {
 	memset(&lf, 0, sizeof(lf));
 	// The negative is to allow for leading
 	lf.lfHeight = -(abs(size));
@@ -161,7 +202,7 @@ static int HashFont(const char *faceName, int characterSet, int size, bool bold,
 class FontCached : Font {
 	FontCached *next;
 	int usage;
-	LOGFONT lf;
+	LOGFONTA lf;
 	int hash;
 	FontCached(const char *faceName_, int characterSet_, int size_, bool bold_, bool italic_);
 	~FontCached() {}
@@ -180,7 +221,7 @@ FontCached::FontCached(const char *faceName_, int characterSet_, int size_, bool
 	next(0), usage(0), hash(0) {
 	::SetLogFont(lf, faceName_, characterSet_, size_, bold_, italic_);
 	hash = HashFont(faceName_, characterSet_, size_, bold_, italic_);
-	id = ::CreateFontIndirect(&lf);
+	id = ::CreateFontIndirectA(&lf);
 	usage = 1;
 }
 
@@ -289,6 +330,10 @@ class SurfaceImpl : public Surface {
 	int maxWidthMeasure;
 	int maxLenText;
 
+	int codePage;
+	// If 9x OS and current code page is same as ANSI code page.
+	bool win9xACPSame;
+
 	void BrushColor(ColourAllocated back);
 	void SetFont(Font &font_);
 
@@ -315,9 +360,12 @@ public:
 	void FillRectangle(PRectangle rc, ColourAllocated back);
 	void FillRectangle(PRectangle rc, Surface &surfacePattern);
 	void RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+	void AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill, int alphaFill,
+		ColourAllocated outline, int alphaOutline, int flags);
 	void Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back);
 	void Copy(PRectangle rc, Point from, Surface &surfaceSource);
 
+	void DrawTextCommon(PRectangle rc, Font &font_, int ybase, const char *s, int len, UINT fuOptions);
 	void DrawTextNoClip(PRectangle rc, Font &font_, int ybase, const char *s, int len, ColourAllocated fore, ColourAllocated back);
 	void DrawTextClipped(PRectangle rc, Font &font_, int ybase, const char *s, int len, ColourAllocated fore, ColourAllocated back);
 	void DrawTextTransparent(PRectangle rc, Font &font_, int ybase, const char *s, int len, ColourAllocated fore);
@@ -336,7 +384,7 @@ public:
 	void FlushCachedState();
 
 	void SetUnicodeMode(bool unicodeMode_);
-	void SetDBCSMode(int codePage);
+	void SetDBCSMode(int codePage_);
 };
 
 SurfaceImpl::SurfaceImpl() :
@@ -352,6 +400,9 @@ SurfaceImpl::SurfaceImpl() :
 	// There appears to be a 16 bit string length limit in GDI on NT and a limit of
 	// 8192 characters on Windows 95.
 	maxLenText = IsNT() ? 65535 : 8192;
+
+	codePage = 0;
+	win9xACPSame = false;
 }
 
 SurfaceImpl::~SurfaceImpl() {
@@ -510,7 +561,82 @@ void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAl
 	::RoundRect(hdc,
 		rc.left + 1, rc.top,
 		rc.right - 1, rc.bottom,
-		8, 8 );
+		8, 8);
+}
+
+// Plot a point into a DWORD buffer symetrically to all 4 qudrants
+static void AllFour(DWORD *pixels, int width, int height, int x, int y, DWORD val) {
+	pixels[y*width+x] = val;
+	pixels[y*width+width-1-x] = val;
+	pixels[(height-1-y)*width+x] = val;
+	pixels[(height-1-y)*width+width-1-x] = val;
+}
+
+#ifndef AC_SRC_OVER
+#define AC_SRC_OVER                 0x00
+#endif
+#ifndef AC_SRC_ALPHA
+#define AC_SRC_ALPHA		0x01
+#endif
+
+void SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill, int alphaFill,
+		ColourAllocated outline, int alphaOutline, int /* flags*/ ) {
+	if (AlphaBlendFn && rc.Width() > 0) {
+		HDC hMemDC = ::CreateCompatibleDC(reinterpret_cast<HDC>(hdc));
+		int width = rc.Width();
+		int height = rc.Height();
+		// Ensure not distorted too much by corners when small
+		cornerSize = Platform::Minimum(cornerSize, (Platform::Minimum(width, height) / 2) - 2);
+		BITMAPINFO bpih = {sizeof(BITMAPINFOHEADER), width, height, 1, 32, BI_RGB, 0, 0, 0, 0, 0};
+		void *image = 0;
+		HBITMAP hbmMem = CreateDIBSection(reinterpret_cast<HDC>(hMemDC), &bpih,
+			DIB_RGB_COLORS, &image, NULL, 0);
+
+		HBITMAP hbmOld = SelectBitmap(hMemDC, hbmMem);
+
+		byte pixVal[4] = {0};
+		DWORD valEmpty = *(reinterpret_cast<DWORD *>(pixVal));
+		pixVal[0] = static_cast<byte>(GetBValue(fill.AsLong()) * alphaFill / 255);
+		pixVal[1] = static_cast<byte>(GetGValue(fill.AsLong()) * alphaFill / 255);
+		pixVal[2] = static_cast<byte>(GetRValue(fill.AsLong()) * alphaFill / 255);
+		pixVal[3] = static_cast<byte>(alphaFill);
+		DWORD valFill = *(reinterpret_cast<DWORD *>(pixVal));
+		pixVal[0] = static_cast<byte>(GetBValue(outline.AsLong()) * alphaOutline / 255);
+		pixVal[1] = static_cast<byte>(GetGValue(outline.AsLong()) * alphaOutline / 255);
+		pixVal[2] = static_cast<byte>(GetRValue(outline.AsLong()) * alphaOutline / 255);
+		pixVal[3] = static_cast<byte>(alphaOutline);
+		DWORD valOutline = *(reinterpret_cast<DWORD *>(pixVal));
+		DWORD *pixels = reinterpret_cast<DWORD *>(image);
+		for (int y=0; y<height; y++) {
+			for (int x=0; x<width; x++) {
+				if ((x==0) || (x==width-1) || (y == 0) || (y == height-1)) {
+					pixels[y*width+x] = valOutline;
+				} else {
+					pixels[y*width+x] = valFill;
+				}
+			}
+		}
+		for (int c=0;c<cornerSize; c++) {
+			for (int x=0;x<c+1; x++) {
+				AllFour(pixels, width, height, x, c-x, valEmpty);
+			}
+		}
+		for (int x=1;x<cornerSize; x++) {
+			AllFour(pixels, width, height, x, cornerSize-x, valOutline);
+		}
+
+		BLENDFUNCTION merge = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };
+
+		AlphaBlendFn(reinterpret_cast<HDC>(hdc), rc.left, rc.top, width, height, hMemDC, 0, 0, width, height, merge);
+
+		SelectBitmap(hMemDC, hbmOld);
+		::DeleteObject(hbmMem);
+		::DeleteDC(hMemDC);
+	} else {
+		BrushColor(outline);
+		RECT rcw = RectFromPRectangle(rc);
+		FrameRect(hdc, &rcw, brush);
+	}
 }
 
 void SurfaceImpl::Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back) {
@@ -525,40 +651,74 @@ void SurfaceImpl::Copy(PRectangle rc, Point from, Surface &surfaceSource) {
 		static_cast<SurfaceImpl &>(surfaceSource).hdc, from.x, from.y, SRCCOPY);
 }
 
-#define MAX_US_LEN 10000
+const int MAX_US_LEN = 10000;
+
+void SurfaceImpl::DrawTextCommon(PRectangle rc, Font &font_, int ybase, const char *s, int len, UINT fuOptions) {
+	SetFont(font_);
+	RECT rcw = RectFromPRectangle(rc);
+	SIZE sz={0,0};
+	int pos = 0;
+	int x = rc.left;
+
+	// Text drawing may fail if the text is too big.
+	// If it does fail, slice up into segments and draw each segment.
+	const int maxSegmentLength = 0x200;
+
+	if ((!unicodeMode) && (IsNT() || (codePage==0) || win9xACPSame)) {
+		// Use ANSI calls
+		int lenDraw = Platform::Minimum(len, maxLenText);
+		if (!::ExtTextOutA(hdc, x, ybase, fuOptions, &rcw, s, lenDraw, NULL)) {
+			while (lenDraw > pos) {
+				int seglen = Platform::Minimum(maxSegmentLength, lenDraw - pos);
+				if (!::ExtTextOutA(hdc, x, ybase, fuOptions, &rcw, s+pos, seglen, NULL)) {
+					PLATFORM_ASSERT(false);
+					return;
+				}
+				::GetTextExtentPoint32A(hdc, s+pos, seglen, &sz);
+				x += sz.cx;
+				pos += seglen;
+			}
+		}
+	} else {
+		// Use Unicode calls
+		wchar_t tbuf[MAX_US_LEN];
+		int tlen;
+		if (unicodeMode) {
+			tlen = UTF16FromUTF8(s, len, tbuf, MAX_US_LEN);
+		} else {
+			// Support Asian string display in 9x English
+			tlen = ::MultiByteToWideChar(codePage, 0, s, len, NULL, 0);
+			if (tlen > MAX_US_LEN)
+				tlen = MAX_US_LEN;
+			::MultiByteToWideChar(codePage, 0, s, len, tbuf, tlen);
+		}
+		if (!::ExtTextOutW(hdc, x, ybase, fuOptions, &rcw, tbuf, tlen, NULL)) {
+			while (tlen > pos) {
+				int seglen = Platform::Minimum(maxSegmentLength, tlen - pos);
+				if (!::ExtTextOutW(hdc, x, ybase, fuOptions, &rcw, tbuf+pos, seglen, NULL)) {
+					PLATFORM_ASSERT(false);
+					return;
+				}
+				::GetTextExtentPoint32W(hdc, tbuf+pos, seglen, &sz);
+				x += sz.cx;
+				pos += seglen;
+			}
+		}
+	}
+}
 
 void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, int ybase, const char *s, int len,
 	ColourAllocated fore, ColourAllocated back) {
-	SetFont(font_);
 	::SetTextColor(hdc, fore.AsLong());
 	::SetBkColor(hdc, back.AsLong());
-	RECT rcw = RectFromPRectangle(rc);
-	if (unicodeMode) {
-		wchar_t tbuf[MAX_US_LEN];
-		int tlen = UCS2FromUTF8(s, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
-		tbuf[tlen] = L'\0';
-		::ExtTextOutW(hdc, rc.left, ybase, ETO_OPAQUE, &rcw, tbuf, tlen, NULL);
-	} else {
-		::ExtTextOut(hdc, rc.left, ybase, ETO_OPAQUE, &rcw, s,
-			Platform::Minimum(len, maxLenText), NULL);
-	}
+	DrawTextCommon(rc, font_, ybase, s, len, ETO_OPAQUE);
 }
 
 void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, int ybase, const char *s, int len,
 	ColourAllocated fore, ColourAllocated back) {
-	SetFont(font_);
 	::SetTextColor(hdc, fore.AsLong());
 	::SetBkColor(hdc, back.AsLong());
-	RECT rcw = RectFromPRectangle(rc);
-	if (unicodeMode) {
-		wchar_t tbuf[MAX_US_LEN];
-		int tlen = UCS2FromUTF8(s, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
-		tbuf[tlen] = L'\0';
-		::ExtTextOutW(hdc, rc.left, ybase, ETO_OPAQUE | ETO_CLIPPED, &rcw, tbuf, tlen, NULL);
-	} else {
-		::ExtTextOut(hdc, rc.left, ybase, ETO_OPAQUE | ETO_CLIPPED, &rcw, s,
-			Platform::Minimum(len, maxLenText), NULL);
-	}
+	DrawTextCommon(rc, font_, ybase, s, len, ETO_OPAQUE | ETO_CLIPPED);
 }
 
 void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_, int ybase, const char *s, int len,
@@ -566,19 +726,9 @@ void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_, int ybase, con
 	// Avoid drawing spaces in transparent mode
 	for (int i=0;i<len;i++) {
 		if (s[i] != ' ') {
-			SetFont(font_);
 			::SetTextColor(hdc, fore.AsLong());
 			::SetBkMode(hdc, TRANSPARENT);
-			RECT rcw = RectFromPRectangle(rc);
-			if (unicodeMode) {
-				wchar_t tbuf[MAX_US_LEN];
-				int tlen = UCS2FromUTF8(s, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
-				tbuf[tlen] = L'\0';
-				::ExtTextOutW(hdc, rc.left, ybase, 0, &rcw, tbuf, tlen, NULL);
-			} else {
-				::ExtTextOut(hdc, rc.left, ybase, 0, &rcw, s,
-					Platform::Minimum(len,maxLenText), NULL);
-			}
+			DrawTextCommon(rc, font_, ybase, s, len, 0);
 			::SetBkMode(hdc, OPAQUE);
 			return;
 		}
@@ -590,11 +740,16 @@ int SurfaceImpl::WidthText(Font &font_, const char *s, int len) {
 	SIZE sz={0,0};
 	if (unicodeMode) {
 		wchar_t tbuf[MAX_US_LEN];
-		int tlen = UCS2FromUTF8(s, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
-		tbuf[tlen] = L'\0';
+		int tlen = UTF16FromUTF8(s, len, tbuf, MAX_US_LEN);
 		::GetTextExtentPoint32W(hdc, tbuf, tlen, &sz);
+	} else if (IsNT() || (codePage==0) || win9xACPSame) {
+		::GetTextExtentPoint32A(hdc, s, Platform::Minimum(len, maxLenText), &sz);
 	} else {
-		::GetTextExtentPoint32(hdc, s, Platform::Minimum(len, maxLenText), &sz);
+		// Support Asian string display in 9x English
+		wchar_t tbuf[MAX_US_LEN];
+		int tlen = ::MultiByteToWideChar(codePage, 0, s, len, NULL, 0);
+		::MultiByteToWideChar(codePage, 0, s, len, tbuf, tlen);
+		::GetTextExtentPoint32W(hdc, tbuf, tlen, &sz);
 	}
 	return sz.cx;
 }
@@ -605,8 +760,7 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positi
 	int fit = 0;
 	if (unicodeMode) {
 		wchar_t tbuf[MAX_US_LEN];
-		int tlen = UCS2FromUTF8(s, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
-		tbuf[tlen] = L'\0';
+		int tlen = UTF16FromUTF8(s, len, tbuf, MAX_US_LEN);
 		int poses[MAX_US_LEN];
 		fit = tlen;
 		if (!::GetTextExtentExPointW(hdc, tbuf, tlen, maxWidthMeasure, &fit, poses, &sz)) {
@@ -624,14 +778,17 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positi
 		int i=0;
 		while (ui<fit) {
 			unsigned char uch = us[i];
-			positions[i++] = poses[ui];
-			if (uch >= 0x80) {
-				if (uch < (0x80 + 0x40 + 0x20)) {
-					positions[i++] = poses[ui];
-				} else {
-					positions[i++] = poses[ui];
-					positions[i++] = poses[ui];
-				}
+			unsigned int lenChar = 1;
+			if (uch >= (0x80 + 0x40 + 0x20 + 0x10)) {
+				lenChar = 4;
+				ui++;
+			} else if (uch >= (0x80 + 0x40 + 0x20)) {
+				lenChar = 3;
+			} else if (uch >= (0x80)) {
+				lenChar = 2;
+			}
+			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
+				positions[i++] = poses[ui];
 			}
 			ui++;
 		}
@@ -641,8 +798,8 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positi
 		while (i<len) {
 			positions[i++] = lastPos;
 		}
-	} else {
-		if (!::GetTextExtentExPoint(hdc, s, Platform::Minimum(len, maxLenText),
+	} else if (IsNT() || (codePage==0) || win9xACPSame) {
+		if (!::GetTextExtentExPointA(hdc, s, Platform::Minimum(len, maxLenText),
 			maxWidthMeasure, &fit, positions, &sz)) {
 			// Eeek - a NULL DC or other foolishness could cause this.
 			// The least we can do is set the positions to zero!
@@ -653,13 +810,38 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positi
 			for (int i=fit;i<len;i++)
 				positions[i] = positions[fit-1];
 		}
+	} else {
+		// Support Asian string display in 9x English
+		wchar_t tbuf[MAX_US_LEN];
+		int tlen = ::MultiByteToWideChar(codePage, 0, s, len, NULL, 0);
+		::MultiByteToWideChar(codePage, 0, s, len, tbuf, tlen);
+
+		int poses[MAX_US_LEN];
+		for (int widthSS=0; widthSS<tlen; widthSS++) {
+			::GetTextExtentPoint32W(hdc, tbuf, widthSS+1, &sz);
+			poses[widthSS] = sz.cx;
+		}
+
+		int ui = 0;
+		for (int i=0;i<len;) {
+			if (::IsDBCSLeadByteEx(codePage, s[i])) {
+				positions[i] = poses[ui];
+				positions[i+1] = poses[ui];
+				i += 2;
+			} else {
+				positions[i] = poses[ui];
+				i++;
+			}
+
+			ui++;
+		}
 	}
 }
 
 int SurfaceImpl::WidthChar(Font &font_, char ch) {
 	SetFont(font_);
 	SIZE sz;
-	::GetTextExtentPoint32(hdc, &ch, 1, &sz);
+	::GetTextExtentPoint32A(hdc, &ch, 1, &sz);
 	return sz.cx;
 }
 
@@ -733,8 +915,10 @@ void SurfaceImpl::SetUnicodeMode(bool unicodeMode_) {
 	unicodeMode=unicodeMode_;
 }
 
-void SurfaceImpl::SetDBCSMode(int) {
+void SurfaceImpl::SetDBCSMode(int codePage_) {
 	// No action on window as automatically handled by system.
+	codePage = codePage_;
+	win9xACPSame = !IsNT() && ((unsigned int)codePage == ::GetACP());
 }
 
 Surface *Surface::Allocate() {
@@ -771,11 +955,21 @@ void Window::SetPositionRelative(PRectangle rc, Window w) {
 		RECT rcOther;
 		::GetWindowRect(reinterpret_cast<HWND>(w.GetID()), &rcOther);
 		rc.Move(rcOther.left, rcOther.top);
-		if (rc.left < 0) {
-			rc.Move(-rc.left,0);
+
+		// Retrieve desktop bounds and make sure window popup's origin isn't left-top of the screen.
+		RECT rcDesktop = {0, 0, 0, 0};
+#ifdef SM_XVIRTUALSCREEN
+		rcDesktop.left = ::GetSystemMetrics(SM_XVIRTUALSCREEN);
+		rcDesktop.top = ::GetSystemMetrics(SM_YVIRTUALSCREEN);
+		rcDesktop.right = rcDesktop.left + ::GetSystemMetrics(SM_CXVIRTUALSCREEN);
+		rcDesktop.bottom = rcDesktop.top + ::GetSystemMetrics(SM_CYVIRTUALSCREEN);
+#endif
+
+		if (rc.left < rcDesktop.left) {
+			rc.Move(rcDesktop.left - rc.left,0);
 		}
-		if (rc.top < 0) {
-			rc.Move(0,-rc.top);
+		if (rc.top < rcDesktop.top) {
+			rc.Move(0,rcDesktop.top - rc.top);
 		}
 	}
 	SetPosition(rc);
@@ -855,16 +1049,44 @@ void Window::SetCursor(Cursor curs) {
 }
 
 void Window::SetTitle(const char *s) {
-	::SetWindowText(reinterpret_cast<HWND>(id), s);
+	::SetWindowTextA(reinterpret_cast<HWND>(id), s);
 }
 
+/* Returns rectangle of monitor pt is on, both rect and pt are in Window's
+   coordinates */
+#ifdef MULTIPLE_MONITOR_SUPPORT
+PRectangle Window::GetMonitorRect(Point pt) {
+	// MonitorFromPoint and GetMonitorInfo are not available on Windows 95 so are not used.
+	// There could be conditional code and dynamic loading in a future version
+	// so this would work on those platforms where they are available.
+	PRectangle rcPosition = GetPosition();
+	POINT ptDesktop = {pt.x + rcPosition.left, pt.y + rcPosition.top};
+	HMONITOR hMonitor = ::MonitorFromPoint(ptDesktop, MONITOR_DEFAULTTONEAREST);
+	MONITORINFOEX mi;
+	memset(&mi, 0, sizeof(mi));
+	mi.cbSize = sizeof(mi);
+	if (::GetMonitorInfo(hMonitor, &mi)) {
+		PRectangle rcMonitor(
+			mi.rcWork.left - rcPosition.left,
+			mi.rcWork.top - rcPosition.top,
+			mi.rcWork.right - rcPosition.left,
+			mi.rcWork.bottom - rcPosition.top);
+		return rcMonitor;
+	}
+}
+#else
+PRectangle Window::GetMonitorRect(Point) {
+	return PRectangle();
+}
+#endif
+
 struct ListItemData {
 	const char *text;
 	int pixId;
 };
 
 #define _ROUND2(n,pow2) \
-        ( ( (n) + (pow2) - 1) & ~((pow2) - 1) )
+	( ( (n) + (pow2) - 1) & ~((pow2) - 1) )
 
 class LineToItem {
 	char *words;
@@ -959,7 +1181,7 @@ ListItemData *LineToItem::Append(const char *text, int imageIndex) {
 	return item;
 }
 
-const char ListBoxX_ClassName[] = "ListBoxX";
+const TCHAR ListBoxX_ClassName[] = TEXT("ListBoxX");
 
 ListBox::ListBox() {
 }
@@ -1005,7 +1227,7 @@ class ListBoxX : public ListBox {
 	void CentreItem(int);
 	void Paint(HDC);
 	void Erase(HDC);
-	static long PASCAL ControlWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam);
+	static LRESULT PASCAL ControlWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam);
 
 	static const Point ItemInset;	// Padding around whole item
 	static const Point TextInset;	// Padding around text
@@ -1045,8 +1267,8 @@ public:
 	}
 	virtual void SetList(const char *list, char separator, char typesep);
 	void Draw(DRAWITEMSTRUCT *pDrawItem);
-	long WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam);
-	static long PASCAL StaticWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam);
+	LRESULT WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam);
+	static LRESULT PASCAL StaticWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam);
 };
 
 const Point ListBoxX::ItemInset(0, 0);
@@ -1118,22 +1340,24 @@ PRectangle ListBoxX::GetDesiredRect() {
 	HDC hdc = ::GetDC(lb);
 	HFONT oldFont = SelectFont(hdc, fontCopy);
 	SIZE textSize = {0, 0};
-	int len = strlen(widestItem);
+	int len = widestItem ? strlen(widestItem) : 0;
 	if (unicodeMode) {
 		wchar_t tbuf[MAX_US_LEN];
-		int tlen = UCS2FromUTF8(widestItem, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
-		tbuf[tlen] = L'\0';
-		::GetTextExtentPoint32W(hdc, tbuf, tlen, &textSize);
+		len = UTF16FromUTF8(widestItem, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
+		tbuf[len] = L'\0';
+		::GetTextExtentPoint32W(hdc, tbuf, len, &textSize);
 	} else {
-		::GetTextExtentPoint32(hdc, widestItem, len, &textSize);
+		::GetTextExtentPoint32A(hdc, widestItem, len, &textSize);
 	}
 	TEXTMETRIC tm;
 	::GetTextMetrics(hdc, &tm);
 	maxCharWidth = tm.tmMaxCharWidth;
 	SelectFont(hdc, oldFont);
 	::ReleaseDC(lb, hdc);
-	if (textSize.cx > width)
-		width = textSize.cx;
+
+	int widthDesired = Platform::Maximum(textSize.cx, (len + 1) * tm.tmAveCharWidth);
+	if (width < widthDesired)
+		width = widthDesired;
 
 	rcDesired.right = rcDesired.left + TextOffset() + width + (TextInset.x * 2);
 	if (Length() > rows)
@@ -1238,11 +1462,11 @@ void ListBoxX::Draw(DRAWITEMSTRUCT *pDrawItem) {
 
 		if (unicodeMode) {
 			wchar_t tbuf[MAX_US_LEN];
-			int tlen = UCS2FromUTF8(text, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
+			int tlen = UTF16FromUTF8(text, len, tbuf, sizeof(tbuf)/sizeof(wchar_t)-1);
 			tbuf[tlen] = L'\0';
 			::DrawTextW(pDrawItem->hDC, tbuf, tlen, &rcText, DT_NOPREFIX|DT_END_ELLIPSIS|DT_SINGLELINE|DT_NOCLIP);
 		} else {
-			::DrawText(pDrawItem->hDC, text, len, &rcText, DT_NOPREFIX|DT_END_ELLIPSIS|DT_SINGLELINE|DT_NOCLIP);
+			::DrawTextA(pDrawItem->hDC, text, len, &rcText, DT_NOPREFIX|DT_END_ELLIPSIS|DT_SINGLELINE|DT_NOCLIP);
 		}
 		if (pDrawItem->itemState & ODS_SELECTED) {
 			::DrawFocusRect(pDrawItem->hDC, &rcBox);
@@ -1273,9 +1497,9 @@ void ListBoxX::AppendListItem(const char *startword, const char *numword) {
 	if (numword) {
 		int pixId = 0;
 		char ch;
-        while ( (ch = *++numword) != '\0' ) {
-            pixId = 10 * pixId + (ch - '0');
-        }
+		while ((ch = *++numword) != '\0') {
+			pixId = 10 * pixId + (ch - '0');
+		}
 		item->pixId = pixId;
 	} else {
 		item->pixId = -1;
@@ -1323,7 +1547,7 @@ void ListBoxX::SetList(const char *list, char separator, char typesep) {
 		int count = lti.Count();
 		::SendMessage(lb, LB_INITSTORAGE, count, 0);
 		for (int j=0; j<count; j++) {
-			::SendMessage(lb, LB_ADDSTRING, 0, 0);
+			::SendMessage(lb, LB_ADDSTRING, 0, j+1);
 		}
 	}
 	SetRedraw(true);
@@ -1353,7 +1577,10 @@ Point ListBoxX::MinTrackSize() const {
 }
 
 Point ListBoxX::MaxTrackSize() const {
-	PRectangle rc(0, 0, maxCharWidth * maxItemCharacters, ItemHeight() * lti.Count());
+	PRectangle rc(0, 0,
+		maxCharWidth * maxItemCharacters + TextInset.x * 2 +
+		 TextOffset() + ::GetSystemMetrics(SM_CXVSCROLL),
+		ItemHeight() * lti.Count());
 	AdjustWindowRect(&rc);
 	return Point(rc.Width(), rc.Height());
 }
@@ -1529,7 +1756,7 @@ void ListBoxX::Paint(HDC hDC) {
 	Point extent = GetClientExtent();
 	HBITMAP hBitmap = ::CreateCompatibleBitmap(hDC, extent.x, extent.y);
 	HDC bitmapDC = ::CreateCompatibleDC(hDC);
-	SelectBitmap(bitmapDC, hBitmap);
+	HBITMAP hBitmapOld = SelectBitmap(bitmapDC, hBitmap);
 	// The list background is mainly erased during painting, but can be a small
 	// unpainted area when at the end of a non-integrally sized list with a
 	// vertical scroll bar
@@ -1538,11 +1765,14 @@ void ListBoxX::Paint(HDC hDC) {
 	// Paint the entire client area and vertical scrollbar
 	::SendMessage(lb, WM_PRINT, reinterpret_cast<WPARAM>(bitmapDC), PRF_CLIENT|PRF_NONCLIENT);
 	::BitBlt(hDC, 0, 0, extent.x, extent.y, bitmapDC, 0, 0, SRCCOPY);
+	// Select a stock brush to prevent warnings from BoundsChecker
+	::SelectObject(bitmapDC, GetStockFont(WHITE_BRUSH));
+	SelectBitmap(bitmapDC, hBitmapOld);
 	::DeleteDC(bitmapDC);
 	::DeleteObject(hBitmap);
 }
 
-long PASCAL ListBoxX::ControlWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
+LRESULT PASCAL ListBoxX::ControlWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
 	switch (uMsg) {
 	case WM_ERASEBKGND:
 		return TRUE;
@@ -1584,7 +1814,7 @@ long PASCAL ListBoxX::ControlWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM
 		return 0;
 	}
 
-	WNDPROC prevWndProc = reinterpret_cast<WNDPROC>(GetWindowLong(hWnd, GWL_USERDATA));
+	WNDPROC prevWndProc = reinterpret_cast<WNDPROC>(GetWindowLongPtr(hWnd, GWLP_USERDATA));
 	if (prevWndProc) {
 		return ::CallWindowProc(prevWndProc, hWnd, uMsg, wParam, lParam);
 	} else {
@@ -1592,7 +1822,7 @@ long PASCAL ListBoxX::ControlWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM
 	}
 }
 
-long ListBoxX::WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
+LRESULT ListBoxX::WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	switch (iMessage) {
 	case WM_CREATE: {
 			HINSTANCE hinstanceParent = GetWindowInstance(reinterpret_cast<HWND>(parent->GetID()));
@@ -1606,8 +1836,8 @@ long ListBoxX::WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
 				reinterpret_cast<HMENU>(ctrlID),
 				hinstanceParent,
 				0);
-			WNDPROC prevWndProc = reinterpret_cast<WNDPROC>(::SetWindowLong(lb, GWL_WNDPROC, reinterpret_cast<LONG>(ControlWndProc)));
-			::SetWindowLong(lb, GWL_USERDATA, reinterpret_cast<LONG>(prevWndProc));
+			WNDPROC prevWndProc = reinterpret_cast<WNDPROC>(::SetWindowLongPtr(lb, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(ControlWndProc)));
+			::SetWindowLongPtr(lb, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(prevWndProc));
 		}
 		break;
 
@@ -1697,7 +1927,7 @@ long ListBoxX::WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	return 0;
 }
 
-long PASCAL ListBoxX::StaticWndProc(
+LRESULT PASCAL ListBoxX::StaticWndProc(
     HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	if (iMessage == WM_CREATE) {
 		CREATESTRUCT *pCreate = reinterpret_cast<CREATESTRUCT *>(lParam);
@@ -1810,7 +2040,7 @@ protected:
 	HMODULE h;
 public:
 	DynamicLibraryImpl(const char *modulePath) {
-		h = ::LoadLibrary(modulePath);
+		h = ::LoadLibraryA(modulePath);
 	}
 
 	virtual ~DynamicLibraryImpl() {
@@ -1833,7 +2063,7 @@ public:
 };
 
 DynamicLibrary *DynamicLibrary::Load(const char *modulePath) {
-	return static_cast<DynamicLibrary *>( new DynamicLibraryImpl(modulePath) );
+	return static_cast<DynamicLibrary *>(new DynamicLibraryImpl(modulePath));
 }
 
 ColourDesired Platform::Chrome() {
@@ -1861,7 +2091,7 @@ bool Platform::MouseButtonBounce() {
 }
 
 void Platform::DebugDisplay(const char *s) {
-	::OutputDebugString(s);
+	::OutputDebugStringA(s);
 }
 
 bool Platform::IsKeyDown(int key) {
@@ -1933,7 +2163,7 @@ void Platform::Assert(const char *c, const char *file, int line) {
 	char buffer[2000];
 	sprintf(buffer, "Assertion [%s] failed at %s %d", c, file, line);
 	if (assertionPopUps) {
-		int idButton = ::MessageBox(0, buffer, "Assertion failure",
+		int idButton = ::MessageBoxA(0, buffer, "Assertion failure",
 			MB_ABORTRETRYIGNORE|MB_ICONHAND|MB_SETFOREGROUND|MB_TASKMODAL);
 		if (idButton == IDRETRY) {
 			::DebugBreak();
@@ -1945,6 +2175,7 @@ void Platform::Assert(const char *c, const char *file, int line) {
 	} else {
 		strcat(buffer, "\r\n");
 		Platform::DebugDisplay(buffer);
+		::DebugBreak();
 		abort();
 	}
 }
@@ -1963,6 +2194,14 @@ void Platform_Initialise(void *hInstance) {
 	onNT = osv.dwPlatformId == VER_PLATFORM_WIN32_NT;
 	::InitializeCriticalSection(&crPlatformLock);
 	hinstPlatformRes = reinterpret_cast<HINSTANCE>(hInstance);
+	// This may be called from DllMain, in which case the call to LoadLibrary
+	// is bad because it can upset the DLL load order.
+	if (!hDLLImage) {
+		hDLLImage = ::LoadLibrary(TEXT("Msimg32"));
+	}
+	if (hDLLImage) {
+		AlphaBlendFn = (AlphaBlendSig)::GetProcAddress(hDLLImage, "AlphaBlend");
+	}
 	ListBoxX_Register();
 }
 
