@@ -28,8 +28,10 @@
 #include "Accessor.h"
 #include "KeyWords.h"
 #endif
+#include "SplitVector.h"
+#include "Partitioning.h"
+#include "RunStyles.h"
 #include "ContractionState.h"
-#include "SVector.h"
 #include "CellBuffer.h"
 #include "CallTip.h"
 #include "KeyMap.h"
@@ -40,7 +42,9 @@
 #include "AutoComplete.h"
 #include "ViewStyle.h"
 #include "CharClassify.h"
+#include "Decoration.h"
 #include "Document.h"
+#include "PositionCache.h"
 #include "Editor.h"
 #include "ScintillaBase.h"
 #include "UniConversion.h"
@@ -154,6 +158,7 @@ class ScintillaWin :
 
 	CLIPFORMAT cfColumnSelect;
 
+	HRESULT hrOle;
 	DropSource ds;
 	DataObject dob;
 	DropTarget dt;
@@ -178,9 +183,11 @@ class ScintillaWin :
 
 	enum { invalidTimerID, standardTimerID, idleTimerID };
 
+	virtual bool DragThreshold(Point ptStart, Point ptNow);
 	virtual void StartDrag();
 	sptr_t WndPaint(uptr_t wParam);
 	sptr_t HandleComposition(uptr_t wParam, sptr_t lParam);
+	virtual bool ValidCodePage(int codePage) const;
 	virtual sptr_t DefWndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 	virtual bool SetIdle(bool on);
 	virtual void SetTicking(bool on);
@@ -196,7 +203,7 @@ class ScintillaWin :
 	virtual void NotifyFocus(bool focus);
 	virtual int GetCtrlID();
 	virtual void NotifyParent(SCNotification scn);
-	virtual void NotifyDoubleClick(Point pt, bool shift);
+	virtual void NotifyDoubleClick(Point pt, bool shift, bool ctrl, bool alt);
 	virtual void Copy();
 	virtual bool CanPaste();
 	virtual void Paste();
@@ -218,6 +225,7 @@ class ScintillaWin :
 	void FullPaint();
 	void FullPaintDC(HDC dc);
 	bool IsCompatibleDC(HDC dc);
+	DWORD EffectFromState(DWORD grfKeyState);
 
 	virtual int SetScrollInfo(int nBar, LPCSCROLLINFO lpsi, BOOL bRedraw);
 	virtual bool GetScrollInfo(int nBar, LPSCROLLINFO lpsi);
@@ -290,6 +298,8 @@ ScintillaWin::ScintillaWin(HWND hwnd) {
 	cfColumnSelect = static_cast<CLIPFORMAT>(
 		::RegisterClipboardFormat(TEXT("MSDEVColumnSelect")));
 
+	hrOle = E_FAIL;
+
 	wMain = hwnd;
 
 	dob.sci = this;
@@ -309,7 +319,7 @@ void ScintillaWin::Initialise() {
 	// Initialize COM.  If the app has already done this it will have
 	// no effect.  If the app hasnt, we really shouldnt ask them to call
 	// it just so this internal feature works.
-	::OleInitialize(NULL);
+	hrOle = ::OleInitialize(NULL);
 }
 
 void ScintillaWin::Finalise() {
@@ -318,14 +328,24 @@ void ScintillaWin::Finalise() {
 	SetIdle(false);
 	DestroySystemCaret();
 	::RevokeDragDrop(MainHWND());
-	::OleUninitialize();
+	if (SUCCEEDED(hrOle)) {
+		::OleUninitialize();
+	}
 }
 
 HWND ScintillaWin::MainHWND() {
 	return reinterpret_cast<HWND>(wMain.GetID());
 }
 
+bool ScintillaWin::DragThreshold(Point ptStart, Point ptNow) {
+	int xMove = abs(ptStart.x - ptNow.x);
+	int yMove = abs(ptStart.y - ptNow.y);
+	return (xMove > ::GetSystemMetrics(SM_CXDRAG)) ||
+		(yMove > ::GetSystemMetrics(SM_CYDRAG));
+}
+
 void ScintillaWin::StartDrag() {
+	inDragDrop = ddDragging;
 	DWORD dwEffect = 0;
 	dropWentOutside = true;
 	IDataObject *pDataObject = reinterpret_cast<IDataObject *>(&dob);
@@ -342,7 +362,7 @@ void ScintillaWin::StartDrag() {
 			ClearSelection();
 		}
 	}
-	inDragDrop = false;
+	inDragDrop = ddNone;
 	SetDragPosition(invalidPosition);
 }
 
@@ -359,7 +379,7 @@ static int InputCodePage() {
 	HKL inputLocale = ::GetKeyboardLayout(0);
 	LANGID inputLang = LOWORD(inputLocale);
 	char sCodePage[10];
-	int res = ::GetLocaleInfo(MAKELCID(inputLang, SORT_DEFAULT),
+	int res = ::GetLocaleInfoA(MAKELCID(inputLang, SORT_DEFAULT),
 	  LOCALE_IDEFAULTANSICODEPAGE, sCodePage, sizeof(sCodePage));
 	if (!res)
 		return 0;
@@ -395,6 +415,9 @@ static int KeyTranslate(int keyIn) {
 		case VK_ADD:		return SCK_ADD;
 		case VK_SUBTRACT:	return SCK_SUBTRACT;
 		case VK_DIVIDE:		return SCK_DIVIDE;
+		case VK_LWIN:		return SCK_WIN;
+		case VK_RWIN:		return SCK_RWIN;
+		case VK_APPS:		return SCK_MENU;
 		case VK_OEM_2:		return '/';
 		case VK_OEM_3:		return '`';
 		case VK_OEM_4:		return '[';
@@ -415,7 +438,8 @@ LRESULT ScintillaWin::WndPaint(uptr_t wParam) {
 
 	bool IsOcxCtrl = (wParam != 0); // if wParam != 0, it contains
 								   // a PAINSTRUCT* from the OCX
-	PLATFORM_ASSERT(hRgnUpdate == NULL);
+	// Removed since this interferes with reporting other assertions as it occurs repeatedly
+	//PLATFORM_ASSERT(hRgnUpdate == NULL);
 	hRgnUpdate = ::CreateRectRgn(0, 0, 0, 0);
 	if (IsOcxCtrl) {
 		pps = reinterpret_cast<PAINTSTRUCT*>(wParam);
@@ -473,7 +497,7 @@ sptr_t ScintillaWin::HandleComposition(uptr_t wParam, sptr_t lParam) {
 			if (IsUnicodeMode()) {
 				char utfval[maxLenInputIME * 3];
 				unsigned int len = UTF8Length(wcs, wides);
-				UTF8FromUCS2(wcs, wides, utfval, len);
+				UTF8FromUTF16(wcs, wides, utfval, len);
 				utfval[len] = '\0';
 				AddCharUTF(utfval, len);
 			} else {
@@ -529,6 +553,24 @@ static unsigned int SciMessageFromEM(unsigned int iMessage) {
 	return iMessage;
 }
 
+static UINT CodePageFromCharSet(DWORD characterSet, UINT documentCodePage) {
+	CHARSETINFO ci = { 0, 0, { { 0, 0, 0, 0 }, { 0, 0 } } };
+	BOOL bci = ::TranslateCharsetInfo((DWORD*)characterSet,
+		&ci, TCI_SRCCHARSET);
+
+	UINT cp;
+	if (bci)
+		cp = ci.ciACP;
+	else
+		cp = documentCodePage;
+
+	CPINFO cpi;
+	if (!IsValidCodePage(cp) && !GetCPInfo(cp, &cpi))
+		cp = CP_ACP;
+
+	return cp;
+}
+
 sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam) {
 	//Platform::DebugPrintf("S M:%x WP:%x L:%x\n", iMessage, wParam, lParam);
 	iMessage = SciMessageFromEM(iMessage);
@@ -682,7 +724,7 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 
 	case WM_SETCURSOR:
 		if (LoWord(lParam) == HTCLIENT) {
-			if (inDragDrop) {
+			if (inDragDrop == ddDragging) {
 				DisplayCursor(Window::cursorUp);
 			} else {
 				// Display regular (drag) cursor over selection
@@ -705,32 +747,44 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 		}
 
 	case WM_CHAR:
-		if (!iscntrl(wParam&0xff) || !lastKeyDownConsumed) {
-			if (IsUnicodeMode()) {
-				// For a wide character version of the window:
-				//char utfval[4];
-				//wchar_t wcs[2] = {wParam, 0};
-				//unsigned int len = UTF8Length(wcs, 1);
-				//UTF8FromUCS2(wcs, 1, utfval, len);
-				//AddCharUTF(utfval, len);
-				AddCharBytes('\0', LOBYTE(wParam));
+		if (((wParam >= 128) || !iscntrl(wParam)) || !lastKeyDownConsumed) {
+			if (::IsWindowUnicode(MainHWND())) {
+				wchar_t wcs[2] = {wParam, 0};
+				if (IsUnicodeMode()) {
+					// For a wide character version of the window:
+					char utfval[4];
+					unsigned int len = UTF8Length(wcs, 1);
+					UTF8FromUTF16(wcs, 1, utfval, len);
+					AddCharUTF(utfval, len);
+				} else {
+					UINT cpDest = CodePageFromCharSet(
+						vs.styles[STYLE_DEFAULT].characterSet, pdoc->dbcsCodePage);
+					char inBufferCP[20];
+					int size = ::WideCharToMultiByte(cpDest,
+						0, wcs, 1, inBufferCP, sizeof(inBufferCP) - 1, 0, 0);
+					AddCharUTF(inBufferCP, size);
+				}
 			} else {
-				AddChar(LOBYTE(wParam));
+				if (IsUnicodeMode()) {
+					AddCharBytes('\0', LOBYTE(wParam));
+				} else {
+					AddChar(LOBYTE(wParam));
+				}
 			}
 		}
 		return 0;
 
 	case WM_UNICHAR:
 		if (wParam == UNICODE_NOCHAR) {
-			return 1;
+			return IsUnicodeMode() ? 1 : 0;
 		} else if (lastKeyDownConsumed) {
 			return 1;
 		} else {
 			if (IsUnicodeMode()) {
 				char utfval[4];
-				wchar_t wcs[2] = {wParam, 0};
+				wchar_t wcs[2] = {static_cast<wchar_t>(wParam), 0};
 				unsigned int len = UTF8Length(wcs, 1);
-				UTF8FromUCS2(wcs, 1, utfval, len);
+				UTF8FromUTF16(wcs, 1, utfval, len);
 				AddCharUTF(utfval, len);
 				return 1;
 			} else {
@@ -854,8 +908,8 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 		capturedMouse = false;
 		return 0;
 
-        // These are not handled in Scintilla and its faster to dispatch them here.
-        // Also moves time out to here so profile doesn't count lots of empty message calls.
+	// These are not handled in Scintilla and its faster to dispatch them here.
+	// Also moves time out to here so profile doesn't count lots of empty message calls.
 
 	case WM_MOVE:
 	case WM_MOUSEACTIVATE:
@@ -954,6 +1008,12 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 	return 0l;
 }
 
+bool ScintillaWin::ValidCodePage(int codePage) const {
+	return codePage == 0 || codePage == SC_CP_UTF8 ||
+	       codePage == 932 || codePage == 936 || codePage == 949 ||
+	       codePage == 950 || codePage == 1361;
+}
+
 sptr_t ScintillaWin::DefWndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam) {
 	return ::DefWindowProc(MainHWND(), iMessage, wParam, lParam);
 }
@@ -1008,7 +1068,7 @@ bool ScintillaWin::HaveMouseCapture() {
 
 bool ScintillaWin::PaintContains(PRectangle rc) {
 	bool contains = true;
-	if (paintState == painting) {
+	if ((paintState == painting) && (!rc.Empty())) {
 		if (!rcPaint.Contains(rc)) {
 			contains = false;
 		} else {
@@ -1155,9 +1215,9 @@ void ScintillaWin::NotifyParent(SCNotification scn) {
 	              GetCtrlID(), reinterpret_cast<LPARAM>(&scn));
 }
 
-void ScintillaWin::NotifyDoubleClick(Point pt, bool shift) {
+void ScintillaWin::NotifyDoubleClick(Point pt, bool shift, bool ctrl, bool alt) {
 	//Platform::DebugPrintf("ScintillaWin Double click 0\n");
-	ScintillaBase::NotifyDoubleClick(pt, shift);
+	ScintillaBase::NotifyDoubleClick(pt, shift, ctrl, alt);
 	// Send myself a WM_LBUTTONDBLCLK, so the container can handle it too.
 	::SendMessage(MainHWND(),
 			  WM_LBUTTONDBLCLK,
@@ -1184,24 +1244,6 @@ bool ScintillaWin::CanPaste() {
 	return false;
 }
 
-static UINT CodePageFromCharSet(DWORD characterSet, UINT documentCodePage) {
-	CHARSETINFO ci = { 0, 0, { { 0, 0, 0, 0 }, { 0, 0 } } };
-	BOOL bci = ::TranslateCharsetInfo((DWORD*)characterSet,
-		&ci, TCI_SRCCHARSET);
-
-	UINT cp;
-	if (bci)
-		cp = ci.ciACP;
-	else
-		cp = documentCodePage;
-
-	CPINFO cpi;
-	if (!IsValidCodePage(cp) && !GetCPInfo(cp, &cpi))
-		cp = CP_ACP;
-
-	return cp;
-}
-
 class GlobalMemory {
 	HGLOBAL hand;
 public:
@@ -1281,7 +1323,7 @@ void ScintillaWin::Paste() {
 				len = UTF8Length(uptr, bytes / 2);
 				putf = new char[len + 1];
 				if (putf) {
-					UTF8FromUCS2(uptr, bytes / 2, putf, len);
+					UTF8FromUTF16(uptr, bytes / 2, putf, len);
 				}
 			} else {
 				// CF_UNICODETEXT available, but not in Unicode mode
@@ -1326,8 +1368,8 @@ void ScintillaWin::Paste() {
 					unsigned int mlen = UTF8Length(uptr, ulen);
 					char *putf = new char[mlen + 1];
 					if (putf) {
-						// CP_UTF8 not available on Windows 95, so use UTF8FromUCS2()
-						UTF8FromUCS2(uptr, ulen, putf, mlen);
+						// CP_UTF8 not available on Windows 95, so use UTF8FromUTF16()
+						UTF8FromUTF16(uptr, ulen, putf, mlen);
 					}
 
 					delete []uptr;
@@ -1366,11 +1408,11 @@ void ScintillaWin::AddToPopUp(const char *label, int cmd, bool enabled) {
 #ifdef TOTAL_CONTROL
 	HMENU hmenuPopup = reinterpret_cast<HMENU>(popup.GetID());
 	if (!label[0])
-		::AppendMenu(hmenuPopup, MF_SEPARATOR, 0, "");
+		::AppendMenuA(hmenuPopup, MF_SEPARATOR, 0, "");
 	else if (enabled)
-		::AppendMenu(hmenuPopup, MF_STRING, cmd, label);
+		::AppendMenuA(hmenuPopup, MF_STRING, cmd, label);
 	else
-		::AppendMenu(hmenuPopup, MF_STRING | MF_DISABLED | MF_GRAYED, cmd, label);
+		::AppendMenuA(hmenuPopup, MF_STRING | MF_DISABLED | MF_GRAYED, cmd, label);
 #endif
 }
 
@@ -1705,7 +1747,7 @@ void ScintillaWin::ImeStartComposition() {
 			// Since the style creation code has been made platform independent,
 			// The logfont for the IME is recreated here.
 			int styleHere = (pdoc->StyleAt(currentPos)) & 31;
-			LOGFONT lf = {0,0,0,0,0,0,0,0,0,0,0,0,0,TEXT("")};
+			LOGFONTA lf = {0,0,0,0,0,0,0,0,0,0,0,0,0, ""};
 			int sizeZoomed = vs.styles[styleHere].size + vs.zoomLevel;
 			if (sizeZoomed <= 2)	// Hangs if sizeZoomed <= 1
 				sizeZoomed = 2;
@@ -1723,7 +1765,7 @@ void ScintillaWin::ImeStartComposition() {
 			if (vs.styles[styleHere].fontName)
 				strcpy(lf.lfFaceName, vs.styles[styleHere].fontName);
 
-			::ImmSetCompositionFont(hIMC, &lf);
+			::ImmSetCompositionFontA(hIMC, &lf);
 		}
 		::ImmReleaseContext(MainHWND(), hIMC);
 		// Caret is displayed in IME window. So, caret in Scintilla is useless.
@@ -1755,7 +1797,7 @@ void ScintillaWin::AddCharBytes(char b0, char b1) {
 			::MultiByteToWideChar(inputCodePage, 0, ansiChars, 1, wcs, 1);
 		}
 		unsigned int len = UTF8Length(wcs, 1);
-		UTF8FromUCS2(wcs, 1, utfval, len);
+		UTF8FromUTF16(wcs, 1, utfval, len);
 		utfval[len] = '\0';
 		AddCharUTF(utfval, len ? len : 1);
 	} else if (b0) {
@@ -1783,10 +1825,10 @@ void ScintillaWin::CopyToClipboard(const SelectionText &selectedText) {
 
 	// Default Scintilla behaviour in Unicode mode
 	if (IsUnicodeMode()) {
-		int uchars = UCS2Length(selectedText.s, selectedText.len);
+		int uchars = UTF16Length(selectedText.s, selectedText.len);
 		uniText.Allocate(2 * uchars);
 		if (uniText) {
-			UCS2FromUTF8(selectedText.s, selectedText.len, static_cast<wchar_t *>(uniText.ptr), uchars);
+			UTF16FromUTF8(selectedText.s, selectedText.len, static_cast<wchar_t *>(uniText.ptr), uchars);
 		}
 	} else {
 		// Not Unicode mode
@@ -1958,6 +2000,20 @@ bool ScintillaWin::IsCompatibleDC(HDC hOtherDC) {
 	return isCompatible;
 }
 
+DWORD ScintillaWin::EffectFromState(DWORD grfKeyState) {
+	// These are the Wordpad semantics.
+	DWORD dwEffect;
+	if (inDragDrop == ddDragging)	// Internal defaults to move
+		dwEffect = DROPEFFECT_MOVE;
+	else
+		dwEffect = DROPEFFECT_COPY;
+	if (grfKeyState & MK_ALT)
+		dwEffect = DROPEFFECT_MOVE;
+	if (grfKeyState & MK_CONTROL)
+		dwEffect = DROPEFFECT_COPY;
+	return dwEffect;
+}
+
 /// Implement IUnknown
 STDMETHODIMP ScintillaWin::QueryInterface(REFIID riid, PVOID *ppv) {
 	*ppv = NULL;
@@ -1987,11 +2043,9 @@ STDMETHODIMP ScintillaWin::DragEnter(LPDATAOBJECT pIDataSource, DWORD grfKeyStat
                                      POINTL, PDWORD pdwEffect) {
 	if (pIDataSource == NULL)
 		return E_POINTER;
-	if (IsUnicodeMode()) {
-		FORMATETC fmtu = {CF_UNICODETEXT, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
-		HRESULT hrHasUText = pIDataSource->QueryGetData(&fmtu);
-		hasOKText = (hrHasUText == S_OK);
-	}
+	FORMATETC fmtu = {CF_UNICODETEXT, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
+	HRESULT hrHasUText = pIDataSource->QueryGetData(&fmtu);
+	hasOKText = (hrHasUText == S_OK);
 	if (!hasOKText) {
 		FORMATETC fmte = {CF_TEXT, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
 		HRESULT hrHasText = pIDataSource->QueryGetData(&fmte);
@@ -2002,14 +2056,7 @@ STDMETHODIMP ScintillaWin::DragEnter(LPDATAOBJECT pIDataSource, DWORD grfKeyStat
 		return S_OK;
 	}
 
-	if (inDragDrop)	// Internal defaults to move
-		*pdwEffect = DROPEFFECT_MOVE;
-	else
-		*pdwEffect = DROPEFFECT_COPY;
-	if (grfKeyState & MK_ALT)
-		*pdwEffect = DROPEFFECT_MOVE;
-	if (grfKeyState & MK_CONTROL)
-		*pdwEffect = DROPEFFECT_COPY;
+	*pdwEffect = EffectFromState(grfKeyState);
 	return S_OK;
 }
 
@@ -2019,15 +2066,8 @@ STDMETHODIMP ScintillaWin::DragOver(DWORD grfKeyState, POINTL pt, PDWORD pdwEffe
 		return S_OK;
 	}
 
-	// These are the Wordpad semantics.
-	if (inDragDrop)	// Internal defaults to move
-		*pdwEffect = DROPEFFECT_MOVE;
-	else
-		*pdwEffect = DROPEFFECT_COPY;
-	if (grfKeyState & MK_ALT)
-		*pdwEffect = DROPEFFECT_MOVE;
-	if (grfKeyState & MK_CONTROL)
-		*pdwEffect = DROPEFFECT_COPY;
+	*pdwEffect = EffectFromState(grfKeyState);
+
 	// Update the cursor.
 	POINT rpt = {pt.x, pt.y};
 	::ScreenToClient(MainHWND(), &rpt);
@@ -2043,14 +2083,7 @@ STDMETHODIMP ScintillaWin::DragLeave() {
 
 STDMETHODIMP ScintillaWin::Drop(LPDATAOBJECT pIDataSource, DWORD grfKeyState,
                                 POINTL pt, PDWORD pdwEffect) {
-	if (inDragDrop)	// Internal defaults to move
-		*pdwEffect = DROPEFFECT_MOVE;
-	else
-		*pdwEffect = DROPEFFECT_COPY;
-	if (grfKeyState & MK_ALT)
-		*pdwEffect = DROPEFFECT_MOVE;
-	if (grfKeyState & MK_CONTROL)
-		*pdwEffect = DROPEFFECT_COPY;
+	*pdwEffect = EffectFromState(grfKeyState);
 
 	if (pIDataSource == NULL)
 		return E_POINTER;
@@ -2058,22 +2091,38 @@ STDMETHODIMP ScintillaWin::Drop(LPDATAOBJECT pIDataSource, DWORD grfKeyState,
 	SetDragPosition(invalidPosition);
 
 	STGMEDIUM medium={0,{0},0};
-	HRESULT hr = S_OK;
 
 	char *data = 0;
 	bool dataAllocated = false;
 
-	if (IsUnicodeMode()) {
-		FORMATETC fmtu = {CF_UNICODETEXT, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
-		hr = pIDataSource->GetData(&fmtu, &medium);
-		if (SUCCEEDED(hr) && medium.hGlobal) {
-			wchar_t *udata = static_cast<wchar_t *>(::GlobalLock(medium.hGlobal));
+	FORMATETC fmtu = {CF_UNICODETEXT, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
+	HRESULT hr = pIDataSource->GetData(&fmtu, &medium);
+	if (SUCCEEDED(hr) && medium.hGlobal) {
+		wchar_t *udata = static_cast<wchar_t *>(::GlobalLock(medium.hGlobal));
+		if (IsUnicodeMode()) {
 			int tlen = ::GlobalSize(medium.hGlobal);
-			// Convert UCS-2 to UTF-8
+			// Convert UTF-16 to UTF-8
 			int dataLen = UTF8Length(udata, tlen/2);
 			data = new char[dataLen+1];
 			if (data) {
-				UTF8FromUCS2(udata, tlen/2, data, dataLen);
+				UTF8FromUTF16(udata, tlen/2, data, dataLen);
+				dataAllocated = true;
+			}
+		} else {
+			// Convert UTF-16 to ANSI
+			//
+			// Default Scintilla behavior in Unicode mode
+			// CF_UNICODETEXT available, but not in Unicode mode
+			// Convert from Unicode to current Scintilla code page
+			UINT cpDest = CodePageFromCharSet(
+				vs.styles[STYLE_DEFAULT].characterSet, pdoc->dbcsCodePage);
+			int tlen = ::WideCharToMultiByte(cpDest, 0, udata, -1,
+				NULL, 0, NULL, NULL) - 1; // subtract 0 terminator
+			data = new char[tlen + 1];
+			if (data) {
+				memset(data, 0, (tlen+1));
+				::WideCharToMultiByte(cpDest, 0, udata, -1,
+						data, tlen + 1, NULL, NULL);
 				dataAllocated = true;
 			}
 		}
@@ -2133,10 +2182,10 @@ STDMETHODIMP ScintillaWin::GetData(FORMATETC *pFEIn, STGMEDIUM *pSTM) {
 
 	GlobalMemory text;
 	if (pFEIn->cfFormat == CF_UNICODETEXT) {
-		int uchars = UCS2Length(drag.s, drag.len);
+		int uchars = UTF16Length(drag.s, drag.len);
 		text.Allocate(2 * uchars);
 		if (text) {
-			UCS2FromUTF8(drag.s, drag.len, static_cast<wchar_t *>(text.ptr), uchars);
+			UTF16FromUTF8(drag.s, drag.len, static_cast<wchar_t *>(text.ptr), uchars);
 		}
 	} else {
 		text.Allocate(drag.len);
@@ -2153,10 +2202,10 @@ bool ScintillaWin::Register(HINSTANCE hInstance_) {
 
 	hInstance = hInstance_;
 	bool result;
-#if 0
+
 	// Register the Scintilla class
 	if (IsNT()) {
-	//if (0) {
+
 		// Register Scintilla as a wide character window
 		WNDCLASSEXW wndclass;
 		wndclass.cbSize = sizeof(wndclass);
@@ -2171,9 +2220,9 @@ bool ScintillaWin::Register(HINSTANCE hInstance_) {
 		wndclass.lpszMenuName = NULL;
 		wndclass.lpszClassName = L"Scintilla";
 		wndclass.hIconSm = 0;
-		result = ::RegisterClassExW(&wndclass);
+		result = ::RegisterClassExW(&wndclass) != 0;
 	} else {
-#endif
+
 		// Register Scintilla as a normal character window
 		WNDCLASSEX wndclass;
 		wndclass.cbSize = sizeof(wndclass);
@@ -2189,7 +2238,7 @@ bool ScintillaWin::Register(HINSTANCE hInstance_) {
 		wndclass.lpszClassName = scintillaClassName;
 		wndclass.hIconSm = 0;
 		result = ::RegisterClassEx(&wndclass) != 0;
-	//}
+	}
 
 	if (result) {
 		// Register the CallTip class
