@@ -26,9 +26,13 @@
 #include "Extender.h"
 #include "DirectorExtension.h"
 #include "SciTE.h"
+#include "FilePath.h"
+#include "StringList.h"
+#include "PropSetFile.h"
+#include "Mutex.h"
+#include "JobQueue.h"
 #include "SciTEBase.h"
 
-
 static ExtensionAPI *host = 0;
 static int fdDirector = 0;
 static int fdCorrespondent = 0;
@@ -50,15 +54,16 @@ static bool SendPipeCommand(const char *pipeCommand, int fdOutPipe) {
 	return false;
 }
 
-static void RecieverPipeSignal(void *data, gint fd, GdkInputCondition condition){
+static void ReceiverPipeSignal(void *data, gint fd, GdkInputCondition condition){
 	int readLength;
 	char pipeData[8192];
 	PropSetFile pipeProps;
 	DirectorExtension *ext = reinterpret_cast<DirectorExtension *>(data);
-	
+
 	if (condition == GDK_INPUT_READ) {
 		SString pipeString;
-		while ((readLength = read(fd, pipeData, sizeof(pipeData))) > 0) {
+		while ((readLength = read(fd, pipeData, sizeof(pipeData) - 1)) > 0) {
+			pipeData[readLength] = '\0';
 			pipeString.append(pipeData);
 		}
 		ext->HandleStringMessage(pipeString.c_str());
@@ -73,7 +78,7 @@ static void SendDirector(const char *verb, const char *arg = 0) {
 			fdDestination = fdDirector;
 		else
 			fdDestination = fdCorrespondent;
-		
+
 		SString addressedMessage;
 		addressedMessage += verb;
 		addressedMessage += ":";
@@ -147,7 +152,7 @@ bool DirectorExtension::OnSwitchFile(const char *path) {
 		::SendDirector("switched", path);
 	}
 	return false;
-};
+}
 
 bool DirectorExtension::OnSave(const char *path) {
 	CheckEnvironment(host);
@@ -210,7 +215,7 @@ void DirectorExtension::HandleStringMessage(const char *message) {
 	// Message may contain multiple commands separated by '\n'
 	// Reentrance trouble - if this function is reentered, the fdCorrespondent may
 	// be set to zero before time.
-	WordList wlMessage(true);
+	StringList wlMessage(true);
 	wlMessage.Set(message);
 	//fprintf(fdDebug, "HandleStringMessage: got %s\n", message);
 	for (int i = 0; i < wlMessage.len; i++) {
@@ -242,7 +247,7 @@ bool DirectorExtension::CreatePipe(bool forceNew) {
 	//check we have been given a specific pipe name
 	if (pipeFilename.size() > 0)
 	{
-		printf("CreatePipe: if (pipeFilename.size() > 0): %s\n", pipeFilename.c_str());
+		//fprintf(fdDebug, "CreatePipe: if (pipeFilename.size() > 0): %s\n", pipeFilename.c_str());
 		//snprintf(pipeName, CHAR_MAX - 1, "%s", pipeFilename.c_str());
 		pipeName = g_strdup(pipeFilename.c_str());
 		fdReceiver = open(pipeName, O_RDWR | O_NONBLOCK);
@@ -262,7 +267,7 @@ bool DirectorExtension::CreatePipe(bool forceNew) {
 #endif
 //WB--
 			fdReceiver = open(pipeName, O_RDWR | O_NONBLOCK);
-			
+
 			fdReceiverString = fdReceiver;
 			host->SetProperty("ipc.scite.fdpipe",fdReceiverString.c_str());
 			tryStandardPipeCreation = false;
@@ -284,20 +289,20 @@ bool DirectorExtension::CreatePipe(bool forceNew) {
 	{
 		tryStandardPipeCreation = true;
 	}
-	
+
 	if( tryStandardPipeCreation )
 	{
 		//possible bug here (eventually), can't have more than a 1000 SciTE's open - ajkc 20001112
 		for (int i = 0; i < 1000; i++) {
-	
+
 			//create the pipe name - we use a number as well just incase multiple people have pipes open
 			//or we are forceing a new instance of scite (even if there is already one)
 			sprintf(pipeName, "/tmp/.SciTE.%d.ipc", i);
-	
+
 			//fprintf(fdDebug, "Trying pipe %s\n", pipeName);
 			//check to see if there is already one
 			fdReceiver = open(pipeName, O_RDWR | O_NONBLOCK);
-	
+
 			//there is one but it isn't ours
 			if (fdReceiver == -1 && errno == EACCES) {
 				//fprintf(fdDebug, "No access\n");
@@ -323,7 +328,7 @@ bool DirectorExtension::CreatePipe(bool forceNew) {
 			//there is so just open it (and we don't want out own)
 			else if (forceNew == false) {
 				//fprintf(fdDebug, "Another one there - opening\n");
-				
+
 				//there is already another pipe so set it to true for the return value
 				anotherPipe = true;
 				//I don;t think it is a good idea to be able to listen to our own pipes (yet) so just return
@@ -335,9 +340,9 @@ bool DirectorExtension::CreatePipe(bool forceNew) {
 	}
 
 	if (fdReceiver != -1) {
-		
+
 		//store the inputwatcher so we can remove it.
-		inputWatcher = gdk_input_add(fdReceiver, GDK_INPUT_READ, RecieverPipeSignal, this);
+		inputWatcher = gdk_input_add(fdReceiver, GDK_INPUT_READ, ReceiverPipeSignal, this);
 		return anotherPipe;
 	}
 
