@@ -88,8 +88,7 @@ const char fileRead[] = "rb";
 const char fileWrite[] = "wb";
 #endif
 
-FilePath::FilePath(const char *fileName_) : fileName(fileName_) {
-}
+FilePath::FilePath(const char *fileName_) : fileName(fileName_) {}
 
 FilePath::FilePath(FilePath const &directory, FilePath const &name) {
 	Set(directory, name);
@@ -112,14 +111,14 @@ void FilePath::Set(FilePath const &directory, FilePath const &name) {
 		fileName = name.fileName;
 	} else {
 		fileName = directory.fileName;
-		fileName.appendwithseparator(name.fileName.c_str(), pathSepChar);
+		fileName.appendwithseparator(name.fileName.c_str(),
+			fileName.endswith(pathSepString) ? '\0' : pathSepChar);
 	}
 }
 
 void FilePath::SetDirectory(FilePath directory) {
-	SString curName(fileName);
-	fileName = directory.fileName;
-	fileName.appendwithseparator(curName.c_str(), pathSepChar);
+	FilePath curName(*this);
+	Set(directory, curName);
 }
 
 void FilePath::Init() {
@@ -138,8 +137,8 @@ bool FilePath::SameNameAs(const FilePath &other) const {
 	return SameNameAs(other.fileName.c_str());
 }
 
-bool FilePath::IsSet() const { 
-	return fileName.length() > 0; 
+bool FilePath::IsSet() const {
+	return fileName.length() > 0;
 }
 
 bool FilePath::IsUntitled() const {
@@ -159,7 +158,7 @@ bool FilePath::IsAbsolute() const {
 		return true;
 #endif
 #ifdef WIN32
-	if (fileName[0] == '\\' || fileName[1] == ':')	// UNC path or drive separator
+	if (fileName[0] == pathSepChar || fileName[1] == ':')	// UNC path or drive separator
 		return true;
 #endif
 
@@ -168,7 +167,9 @@ bool FilePath::IsAbsolute() const {
 
 bool FilePath::IsRoot() const {
 #ifdef WIN32
-	return (fileName.length() == 3) && (fileName[1] == ':') && (fileName[2] == '\\');
+    if ((fileName[0] == pathSepChar) && (fileName[1] == pathSepChar) && (fileName.search(pathSepString, 2) < 0))
+        return true; // UNC path like \\server
+	return (fileName.length() == 3) && (fileName[1] == ':') && (fileName[2] == pathSepChar);
 #else
 	return fileName == "/";
 #endif
@@ -197,7 +198,7 @@ FilePath FilePath::Name() const {
 FilePath FilePath::BaseName() const {
 	const char *dirEnd = strrchr(fileName.c_str(), pathSepChar);
 	const char *extStart = strrchr(fileName.c_str(), '.');
-	if (dirEnd) { 
+	if (dirEnd) {
 		if (extStart > dirEnd) {
 			return FilePath(SString(dirEnd + 1, 0, extStart - dirEnd - 1).c_str());
 		} else {
@@ -387,8 +388,7 @@ void FilePath::SetWorkingDirectory() const {
 	chdir(AsFileSystem());
 }
 
-void FilePath::FixCase() {
-}
+void FilePath::FixCase() {}
 
 void FilePath::List(FilePathSet &directories, FilePathSet &files) {
 #ifdef WIN32
@@ -437,16 +437,27 @@ void FilePath::List(FilePathSet &directories, FilePathSet &files) {
 }
 
 FILE *FilePath::Open(const char *mode) const {
-	return fopen(fileName.c_str(), mode);
+	if (IsSet()) {
+		return fopen(fileName.c_str(), mode);
+	} else {
+		return NULL;
+	}
 }
 
 void FilePath::Remove() const {
 	unlink(AsFileSystem());
 }
 
+#ifndef R_OK
+// Neither Borland nor Microsoft define the constants used to call access
+#define R_OK 4
+#endif
+
 time_t FilePath::ModifiedTime() const {
 	if (IsUntitled())
 		return 0;
+	if (access(AsFileSystem(), R_OK) == -1)
+		return 0;
 	struct stat statusFile;
 	if (stat(AsFileSystem(), &statusFile) != -1)
 		return statusFile.st_mtime;
@@ -495,13 +506,13 @@ bool FilePath::IsDirectory() const {
 bool FilePath::Matches(const char *pattern) const {
 	SString pat(pattern);
 	pat.substitute(' ', '\0');
-	SString nameCopy(fileName);
+	SString nameCopy(Name().fileName);
 	nameCopy.lowercase();
 	size_t start = 0;
 	while (start < pat.length()) {
 		const char *patElement = pat.c_str() + start;
 		if (patElement[0] == '*') {
-			if (nameCopy.endswith(patElement+1)) {
+			if (nameCopy.endswith(patElement + 1)) {
 				return true;
 			}
 		} else {
@@ -653,7 +664,7 @@ FilePathSet::FilePathSet(const FilePathSet &other) {
 	size = other.size;
 	lengthBody = other.lengthBody;
 	body = new FilePath[size];
-	for (size_t i=0; i<lengthBody; i++) {
+	for (size_t i = 0; i < lengthBody; i++) {
 		body[i] = other.body[i];
 	}
 }
@@ -673,7 +684,7 @@ void FilePathSet::Append(FilePath fp) {
 	if (lengthBody >= size) {
 		size *= 2;
 		FilePath *bodyNew = new FilePath[size];
-		for (size_t i=0; i<lengthBody; i++) {
+		for (size_t i = 0; i < lengthBody; i++) {
 			bodyNew[i] = body[i];
 		}
 		delete []body;
@@ -685,3 +696,4 @@ void FilePathSet::Append(FilePath fp) {
 size_t FilePathSet::Length() const {
 	return lengthBody;
 }
+
