@@ -25,13 +25,17 @@ extern const char menuAccessIndicator[];
 #endif
 #endif
 
+#define ELEMENTS(a) (sizeof(a) / sizeof(a[0]))
+
 /**
  * The order of menus on Windows - the Buffers menu may not be present
  * and there is a Help menu at the end.
  */
-enum { menuFile = 0, menuEdit = 1, menuSearch = 2, menuView = 3,
-       menuTools = 4, menuOptions = 5, menuLanguage = 6, menuBuffers = 7,
-       menuHelp = 8};
+enum {
+    menuFile = 0, menuEdit = 1, menuSearch = 2, menuView = 3,
+    menuTools = 4, menuOptions = 5, menuLanguage = 6, menuBuffers = 7,
+    menuHelp = 8
+};
 
 /**
  * This is a fixed length list of strings suitable for display in combo boxes
@@ -67,7 +71,7 @@ public:
 			}
 		}
 	}
-	void AppendList(const SString &s, char sep='|') {
+	void AppendList(const SString &s, char sep = '|') {
 		int start = 0;
 		int end = 0;
 		while (s[end] != '\0') {
@@ -109,36 +113,28 @@ public:
 
 // Related to Utf8_16::encodingType but with additional values at end
 enum UniMode {
-	uni8Bit=0, uni16BE=1, uni16LE=2, uniUTF8=3,
-	uniCookie=4
+    uni8Bit = 0, uni16BE = 1, uni16LE = 2, uniUTF8 = 3,
+    uniCookie = 4
 };
 
-// State of folding in a given document, remembers line/state pairs,
-// restore aborts when these pairs mismatch
+// State of folding in a given document, remembers lines folded.
 class FoldState {
 private:
 	int *lines;
-	bool *folded;
 	int size;
 	int fill;
-	int readout;
 
 	void CopyFrom(const FoldState& b) {
 		Alloc(b.size);
 		memcpy(lines, b.lines, size*sizeof(int));
-		memcpy(folded, b.folded, size*sizeof(bool));
 		fill = b.fill;
-
-		readout = 0;
 	}
 
 public:
 	FoldState() {
 		lines = 0;
-		folded = 0;
 		size = 0;
 		fill = 0;
-		readout = -1;
 	}
 
 	FoldState &operator=(const FoldState &b) {
@@ -150,8 +146,6 @@ public:
 
 	FoldState(const FoldState &b) {
 		lines = 0;
-		folded = 0;
-		readout = -1;
 		size = 0;
 		fill = 0;
 
@@ -164,60 +158,40 @@ public:
 
 		delete []lines;
 		lines = new int[s];
-		delete []folded;
-		folded = new bool[s];
 		size = s;
 		fill = 0;
 
-		//assert(lines && folded && size>0);
+		//assert(lines && size>0);
 	}
 
 	void Clear() {
 
 		delete []lines;
 		lines = 0;
-		delete []folded;
-		folded = 0;
 
 		size = 0;
 		fill = 0;
-		readout = -1;
 	}
 
 	virtual ~FoldState() {
 		Clear();
 	}
 
-	void PushState(int line, bool folded_) {
+	void Append(int line) {
 		//assert(fill<size);
 		lines[fill] = line;
-		folded[fill] = folded_;
 		fill++;
 	}
 
-	// returns maximum numbers a readout can succeed
-	int BeginIteration() {
-		// strong assertion
-		//assert(fill==size);
-
-		readout = 0;
-		return fill;
+	int Folds() const {
+		return size;
 	}
 
-	// returns false at end of iteration
-	// no results are written then
-	bool GetState(int *pline, bool *pfold) {
-		//assert(readout>0);
-		if (readout >= fill) {
-			readout = -1;
-			return false;
+	int Line(int fold) const {
+		if (fold >= size) {
+			return 0;
 		} else {
-			if (pline && pfold) {
-				*pline = lines[readout];
-				*pfold = folded[readout];
-			}
-			readout++;
-			return true;
+			return lines[fold];
 		}
 	}
 };
@@ -234,9 +208,8 @@ public:
 	SString overrideExtension;	///< User has chosen to use a particular language
 	FoldState foldState;
 	Buffer() :
-		RecentFile(), doc(0), isDirty(false), useMonoFont(false),
-		unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), findMarks(fmNone), foldState() {
-	}
+			RecentFile(), doc(0), isDirty(false), useMonoFont(false),
+			unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), findMarks(fmNone), foldState() {}
 
 	void Init() {
 		RecentFile::Init();
@@ -279,30 +252,11 @@ public:
 	int StackPrev();
 	void CommitStackSelection();
 	void MoveToStackTop(int index);
+	void ShiftTo(int indexFrom, int indexTo);
 private:
 	void PopStack();
 };
 
-enum JobSubsystem {
-    jobCLI = 0, jobGUI = 1, jobShell = 2, jobExtension = 3, jobHelp = 4, jobOtherHelp = 5, jobGrep = 6};
-
-enum JobFlags {
-    jobForceQueue=1, jobHasInput=2, jobQuiet=4, // 8 reserved for jobVeryQuiet
-    jobRepSelMask=48, jobRepSelYes=16, jobRepSelAuto=32,
-	jobGroupUndo=64};
-
-class Job {
-public:
-	SString command;
-	FilePath directory;
-	SString input;
-	JobSubsystem jobType;
-	int flags;
-
-	Job();
-	void Clear();
-};
-
 // class to hold user defined keyboard short cuts
 class ShortcutItem {
 public:
@@ -317,35 +271,31 @@ public:
 	SString extension;
 };
 
-/// Find the character following a name which is made up of character from
-/// the set [a-zA-Z.]
-char AfterName(const char *s);
-
 typedef EntryMemory < 10 > ComboMemory;
 
 enum {
-	heightTools = 24,
-	heightTab = 24,
-	heightStatus = 20,
-	statusPosWidth = 256
+    heightTools = 24,
+    heightTab = 24,
+    heightStatus = 20,
+    statusPosWidth = 256
 };
 
 /// Warning IDs.
 enum {
-	warnFindWrapped = 1,
-	warnNotFound,
-	warnNoOtherBookmark,
-	warnWrongFile,
-	warnExecuteOK,
-	warnExecuteKO
+    warnFindWrapped = 1,
+    warnNotFound,
+    warnNoOtherBookmark,
+    warnWrongFile,
+    warnExecuteOK,
+    warnExecuteKO
 };
 
 /// Codes representing the effect a line has on indentation.
 enum IndentationStatus {
-	isNone,		// no effect on indentation
-	isBlockStart,	// indentation block begin such as "{" or VB "function"
-	isBlockEnd,	// indentation end indicator such as "}" or VB "end"
-	isKeyWordStart	// Keywords that cause indentation
+    isNone,		// no effect on indentation
+    isBlockStart,	// indentation block begin such as "{" or VB "function"
+    isBlockEnd,	// indentation end indicator such as "}" or VB "end"
+    isKeyWordStart	// Keywords that cause indentation
 };
 
 int IntFromHexDigit(int ch);
@@ -365,8 +315,8 @@ public:
 	bool visible;
 	bool changeable;
 	enum flags { sdNone = 0, sdFont = 0x1, sdSize = 0x2, sdFore = 0x4, sdBack = 0x8,
-	             sdBold = 0x10, sdItalics = 0x20, sdEOLFilled = 0x40, sdUnderlined = 0x80,
-	              sdCaseForce = 0x100, sdVisible = 0x200, sdChangeable = 0x400} specified;
+	        sdBold = 0x10, sdItalics = 0x20, sdEOLFilled = 0x40, sdUnderlined = 0x80,
+	        sdCaseForce = 0x100, sdVisible = 0x200, sdChangeable = 0x400} specified;
 	StyleDefinition(const char *definition);
 	bool ParseStyleDefinition(const char *definition);
 	long ForeAsLong() const;
@@ -380,7 +330,17 @@ struct StyleAndWords {
 	bool IsSingleChar() { return words.length() == 1; }
 };
 
-#define SciTE_MARKER_BOOKMARK 1
+class Localization : public PropSetFile {
+	SString missing;
+public:
+	bool read;
+	Localization() : PropSetFile(true), read(false) {
+	}
+	SString Text(const char *s, bool retainIfNotFound=true);
+	void SetMissing(const SString &missing_) {
+		missing = missing_;
+	}
+};
 
 class SciTEBase : public ExtensionAPI {
 protected:
@@ -399,7 +359,8 @@ protected:
 
 	SString findWhat;
 	SString replaceWhat;
-	Window wFindReplace;
+	enum { indicatorMatch = INDIC_CONTAINER };
+	enum { markerBookmark = 1 };
 	Window wFindIncrement;
 	bool replacing;
 	bool havefound;
@@ -416,7 +377,6 @@ protected:
 	ComboMemory memFiles;
 	ComboMemory memDirectory;
 	enum { maxParam = 4 };
-	Window wParameters;
 	SString parameterisedCommand;
 	char abbrevInsert[200];
 
@@ -432,7 +392,7 @@ protected:
 	int characterSet;
 	SString language;
 	int lexLanguage;
-	WordList apis;
+	StringList apis;
 	SString apisFileNames;
 	SString functionDefinition;
 
@@ -446,9 +406,9 @@ protected:
 	StyleAndWords blockEnd;
 	enum { noPPC, ppcStart, ppcMiddle, ppcEnd, ppcDummy };	///< Indicate the kind of preprocessor condition line
 	char preprocessorSymbol;	///< Preprocessor symbol (in C: #)
-	WordList preprocCondStart;	///< List of preprocessor conditional start keywords (in C: if ifdef ifndef)
-	WordList preprocCondMiddle;	///< List of preprocessor conditional middle keywords (in C: else elif)
-	WordList preprocCondEnd;	///< List of preprocessor conditional end keywords (in C: endif)
+	StringList preprocCondStart;	///< List of preprocessor conditional start keywords (in C: if ifdef ifndef)
+	StringList preprocCondMiddle;	///< List of preprocessor conditional middle keywords (in C: else elif)
+	StringList preprocCondEnd;	///< List of preprocessor conditional end keywords (in C: endif)
 
 	Window wSciTE;  ///< Contains wToolBar, wTabBar, wContent, and wStatusBar
 	Window wContent;    ///< Contains wEditor and wOutput
@@ -495,7 +455,6 @@ protected:
 	bool capturedMouse;
 	int previousHeightOutput;
 	bool firstPropertiesRead;
-	bool localisationRead;
 	bool splitVertical;	///< @c true if the split bar between editor and output is vertical.
 	bool bufferedDraw;
 	bool twoPhaseDraw;
@@ -505,6 +464,7 @@ protected:
 	int braceCount;
 
 	bool indentationWSVisible;
+	int indentExamine;
 
 	bool autoCompleteIgnoreCase;
 	bool callTipIgnoreCase;
@@ -538,19 +498,10 @@ protected:
 	bool lineNumbersExpand;
 
 	bool usePalette;
-	bool clearBeforeExecute;
 	bool allowMenuActions;
-	bool isBuilding;
-	bool isBuilt;
-	bool executing;
 	int scrollOutput;
 	bool returnOutputToCommand;
-	enum { commandMax = 2 };
-	int commandCurrent;
-	Job jobQueue[commandMax];
-	bool jobUsesOutputPane;
-	long cancelFlag;
-	bool timeCommands;
+	JobQueue jobQueue;
 
 	bool macrosEnabled;
 	SString currentMacro;
@@ -559,6 +510,7 @@ protected:
 	PropSetFile propsEmbed;
 	PropSetFile propsBase;
 	PropSetFile propsUser;
+	PropSetFile propsDirectory;
 	PropSetFile propsLocal;
 	PropSetFile props;
 
@@ -566,7 +518,7 @@ protected:
 
 	FilePath pathAbbreviations;
 
-	PropSetFile propsUI;
+	Localization localiser;
 
 	PropSetFile propsStatus;	// Not attached to a file but need SetInteger method.
 
@@ -578,8 +530,8 @@ protected:
 	int AddBuffer();
 	void UpdateBuffersCurrent();
 	bool IsBufferAvailable();
-	bool CanMakeRoom(bool maySaveIfDirty=true);
-	void SetDocumentAt(int index, bool updateStack=true);
+	bool CanMakeRoom(bool maySaveIfDirty = true);
+	void SetDocumentAt(int index, bool updateStack = true);
 	Buffer *CurrentBuffer() {
 		return buffers.CurrentBuffer();
 	}
@@ -590,9 +542,17 @@ protected:
 	void PrevInStack();
 	void EndStackedTabbing();
 
+	virtual void TabInsert(int index, char *title) = 0;
+	virtual void TabSelect(int index) = 0;
+	virtual void RemoveAllTabs() = 0;
+	void ShiftTab(int indexFrom, int indexTo);
+	void MoveTabRight();
+	void MoveTabLeft();
+
 	void ReadGlobalPropFile();
 	void ReadAbbrevPropFile();
 	void ReadLocalPropFile();
+	void ReadDirectoryPropFile();
 
 	sptr_t SendEditor(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	sptr_t SendEditorString(unsigned int msg, uptr_t wParam, const char *s);
@@ -600,14 +560,14 @@ protected:
 	sptr_t SendOutputString(unsigned int msg, uptr_t wParam, const char *s);
 	sptr_t SendFocused(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	sptr_t SendPane(int destination, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	sptr_t SendWindow(Window &w, unsigned int msg, uptr_t wParam=0, sptr_t lParam=0);
+	sptr_t SendWindow(Window &w, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	void SendChildren(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	sptr_t SendOutputEx(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0, bool direct = true);
-	SString GetTranslationToAbout(const char * const propname, bool retainIfNotFound=true);
+	SString GetTranslationToAbout(const char * const propname, bool retainIfNotFound = true);
 	int LengthDocument();
 	int GetCaretInLine();
-	void GetLine(char *text, int sizeText, int line=-1);
-	SString GetLine(int line=-1);
+	void GetLine(char *text, int sizeText, int line = -1);
+	SString GetLine(int line = -1);
 	void GetRange(Window &win, int start, int end, char *text);
 	int IsLinePreprocessorCondition(char *line);
 	bool FindMatchingPreprocessorCondition(int &curLine, int direction, int condEnd1, int condEnd2);
@@ -632,6 +592,7 @@ protected:
 	void SetIndentSettings();
 	void SetEol();
 	void New();
+	void RestoreState(const Buffer &buffer);
 	void Close(bool updateUI = true, bool loadingSession = false, bool makingRoomForNew = false);
 	bool IsAbsolutePath(const char *path);
 	bool Exists(const char *dir, const char *path, FilePath *resultPath);
@@ -646,14 +607,14 @@ protected:
 	virtual void SaveSessionDialog() { };
 	void CountLineEnds(int &linesCR, int &linesLF, int &linesCRLF);
 	enum OpenFlags {
-		ofNone=0, 		// Default
-		ofNoSaveIfDirty=1, 	// Suppress check for unsaved changes
-		ofForceLoad=2,	// Reload file even if already in a buffer
-		ofPreserveUndo=4,	// Do not delete undo history
-		ofQuiet=8		// Avoid "Could not open file" message
+	    ofNone = 0, 		// Default
+	    ofNoSaveIfDirty = 1, 	// Suppress check for unsaved changes
+	    ofForceLoad = 2,	// Reload file even if already in a buffer
+	    ofPreserveUndo = 4,	// Do not delete undo history
+	    ofQuiet = 8		// Avoid "Could not open file" message
 	};
 	virtual bool PreOpenCheck(const char *file);
-	bool Open(FilePath file, OpenFlags of=ofNone);
+	bool Open(FilePath file, OpenFlags of = ofNone);
 	bool OpenSelected();
 	void Revert();
 	FilePath SaveName(const char *ext);
@@ -681,6 +642,8 @@ protected:
 	virtual FilePath GetSciteUserHome() = 0;
 	FilePath GetDefaultPropertiesFileName();
 	FilePath GetUserPropertiesFileName();
+	FilePath GetDirectoryPropertiesFileName();
+	FilePath GetLocalPropertiesFileName();
 	FilePath GetAbbrevPropertiesFileName();
 	void OpenProperties(int propsFile);
 	int GetMenuCommandAsInt(SString commandName);
@@ -690,10 +653,11 @@ protected:
 	void SetSelection(int anchor, int currentPos);
 	//	void SelectionExtend(char *sel, int len, char *notselchar);
 	void GetCTag(char *sel, int len);
+	SString GetRange(Window &win, int selStart, int selEnd);
 	virtual SString GetRangeInUIEncoding(Window &win, int selStart, int selEnd);
 	SString GetLine(Window &win, int line);
 	SString RangeExtendAndGrab(Window &wCurrent, int &selStart, int &selEnd,
-		bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
+	        bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	SString SelectionExtend(bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	void FindWordAtCaret(int &start, int &end);
 	bool SelectWordAtCaret();
@@ -703,8 +667,8 @@ protected:
 	void SelectionIntoFind(bool stripEol = true);
 	virtual SString EncodeString(const SString &s);
 	virtual void Find() = 0;
-	virtual int WindowMessageBox(Window &w, const SString &m, int style)=0;
-	virtual void FindMessageBox(const SString &msg, const SString *findItem=0)=0;
+	virtual int WindowMessageBox(Window &w, const SString &m, int style) = 0;
+	virtual void FindMessageBox(const SString &msg, const SString *findItem = 0) = 0;
 	int FindInTarget(const char *findWhat, int lenFind, int startPosition, int endPosition);
 	int FindNext(bool reverseDirection, bool showWarnings = true);
 	virtual void FindIncrement() = 0;
@@ -719,6 +683,7 @@ protected:
 	virtual void GoLineDialog() = 0;
 	virtual bool AbbrevDialog() = 0;
 	virtual void TabSizeDialog() = 0;
+	virtual bool ParametersOpen() = 0;
 	virtual void ParamGrab() = 0;
 	virtual bool ParametersDialog(bool modal) = 0;
 	bool HandleXml(char ch);
@@ -734,6 +699,8 @@ protected:
 	virtual void StopExecute() = 0;
 	void GoMessage(int dir);
 	virtual bool StartCallTip();
+	char *GetNearestWords(const char *wordStart, int searchLen,
+		const char *separators, bool ignoreCase=false, bool exactLen=false);
 	virtual void FillFunctionDefinition(int pos = -1);
 	void ContinueCallTip();
 	virtual void EliminateDuplicateWords(char *words);
@@ -748,6 +715,7 @@ protected:
 	void SetLineIndentation(int line, int indent);
 	int GetLineIndentation(int line);
 	int GetLineIndentPosition(int line);
+	void ConvertIndentation(int tabSize, int useTabs);
 	bool RangeIsAllWhitespace(int start, int end);
 	IndentationStatus GetIndentState(int line);
 	int IndentOfBlock(int line);
@@ -756,31 +724,31 @@ protected:
 	void CharAdded(char ch);
 	void CharAddedOutput(int ch);
 	void SetTextProperties(PropSetFile &ps);
-	virtual void SetFileProperties(PropSet &ps) = 0;
+	virtual void SetFileProperties(PropSetFile &ps) = 0;
 	virtual void UpdateStatusBar(bool bUpdateSlowData);
 	int GetLineLength(int line);
 	int GetCurrentLineNumber();
 	int GetCurrentScrollPosition();
 	virtual void AddCommand(const SString &cmd, const SString &dir,
-		JobSubsystem jobType, const SString &input = "",
-		int flags = 0);
+	        JobSubsystem jobType, const SString &input = "",
+	        int flags = 0);
 	virtual void AboutDialog() = 0;
 	virtual void QuitProgram() = 0;
 	void CloseTab(int tab);
 	void CloseAllBuffers(bool loadingSession = false);
-	int SaveAllBuffers(bool forceQuestion, bool alwaysYes=false);
+	int SaveAllBuffers(bool forceQuestion, bool alwaysYes = false);
 	void SaveTitledBuffers();
 	virtual void CopyAsRTF() {};
 	void SetLineNumberWidth();
-	void MenuCommand(int cmdID, int source=0);
+	void MenuCommand(int cmdID, int source = 0);
 	void FoldChanged(int line, int levelNow, int levelPrev);
 	void FoldChanged(int position);
 	void Expand(int &line, bool doExpand, bool force = false,
-	            int visLevels = 0, int level = -1);
+	        int visLevels = 0, int level = -1);
 	void FoldAll();
 	void ToggleFoldRecursive(int line, int level);
 	void EnsureAllChildrenVisible(int line, int level);
-	void EnsureRangeVisible(int posStart, int posEnd, bool enforcePolicy=true);
+	void EnsureRangeVisible(int posStart, int posEnd, bool enforcePolicy = true);
 	void GotoLineEnsureVisible(int line);
 	bool MarginClick(int position, int modifiers);
 	void NewLineInOutput();
@@ -802,13 +770,14 @@ protected:
 	virtual void SizeSubWindows() = 0;
 
 	virtual void SetMenuItem(int menuNumber, int position, int itemID,
-	                         const char *text, const char *mnemonic = 0) = 0;
+	        const char *text, const char *mnemonic = 0) = 0;
+	virtual void RedrawMenu() {}
 	virtual void DestroyMenuItem(int menuNumber, int itemID) = 0;
 	virtual void CheckAMenuItem(int wIDCheckItem, bool val) = 0;
 	virtual void EnableAMenuItem(int wIDCheckItem, bool val) = 0;
 	virtual void CheckMenusClipboard();
 	virtual void CheckMenus();
-	virtual void AddToPopUp(const char *label, int cmd=0, bool enabled=true)=0;
+	virtual void AddToPopUp(const char *label, int cmd = 0, bool enabled = true) = 0;
 	void ContextMenu(Window wSource, Point pt, Window wCmd);
 
 	void DeleteFileStackMenu();
@@ -825,7 +794,7 @@ protected:
 
 	void RemoveToolsMenu();
 	void SetMenuItemLocalised(int menuNumber, int position, int itemID,
-		const char *text, const char *mnemonic);
+	        const char *text, const char *mnemonic);
 	void SetToolsMenu();
 	JobSubsystem SubsystemType(char c);
 	JobSubsystem SubsystemType(const char *cmd, int item = -1);
@@ -838,10 +807,10 @@ protected:
 	void ImportMenu(int pos);
 	void SetLanguageMenu();
 	void SetPropertiesInitial();
-	SString LocaliseString(const char *s, bool retainIfNotFound=true);
-	SString LocaliseMessage(const char *s, const char *param0=0,
-		const char *param1=0, const char *param2=0);
-	virtual void ReadLocalisation();
+	SString LocaliseMessage(const char *s, const char *param0 = 0,
+	        const char *param1 = 0, const char *param2 = 0);
+	virtual void ReadLocalization();
+	SString GetFileNameProperty(const char *name);
 	virtual void ReadPropertiesInitial();
 	void ReadFontProperties();
 	void SetOverrideLanguage(int cmdID);
@@ -851,7 +820,7 @@ protected:
 	void ForwardPropertyToEditor(const char *key);
 	void DefineMarker(int marker, int markerType, ColourDesired fore, ColourDesired back);
 	void ReadAPI(const SString &fileNameForExtension);
-	SString FindLanguageProperty(const char *pattern, const char *defaultValue="");
+	SString FindLanguageProperty(const char *pattern, const char *defaultValue = "");
 	virtual void ReadProperties();
 	void SetOneStyle(Window &win, int style, const StyleDefinition &sd);
 	void SetStyleFor(Window &win, const char *language);
@@ -879,7 +848,10 @@ protected:
 	virtual bool IsStdinBlocked();
 	void OpenFromStdin(bool UseOutputPane);
 	void OpenFilesFromStdin();
-	enum GrepFlags { grepNone=0, grepWholeWord=1, grepMatchCase=2, grepStdOut=4 };
+	enum GrepFlags {
+	    grepNone = 0, grepWholeWord = 1, grepMatchCase = 2, grepStdOut = 4,
+	    grepDot = 8, grepBinary = 16
+	};
 	void GrepRecursive(GrepFlags gf, FilePath baseDir, const char *searchString, const char *fileTypes);
 	void InternalGrep(GrepFlags gf, const char *directory, const char *files, const char *search);
 	void EnumProperties(const char *action);
@@ -899,6 +871,7 @@ protected:
 	uptr_t GetInstance();
 	void ShutDown();
 	void Perform(const char *actions);
+	void DoMenuCommand(int cmdID);
 
 	// Valid CurrentWord characters
 	bool iswordcharforsel(char ch);
@@ -940,3 +913,4 @@ long ColourOfProperty(PropSet &props, const char *key, ColourDesired colourDefau
 char *Slash(const char *s, bool quoteQuotes);
 unsigned int UnSlash(char *s);
 void WindowSetFocus(Window &w);
+
