@@ -49,6 +49,7 @@
 
 #include "SciTE.h"
 #include "PropSet.h"
+#include "StringList.h"
 #include "Accessor.h"
 #include "WindowAccessor.h"
 #include "KeyWords.h"
@@ -57,24 +58,14 @@
 #include "Extender.h"
 #include "FilePath.h"
 #include "PropSetFile.h"
+#include "Mutex.h"
+#include "JobQueue.h"
 #include "SciTEBase.h"
 
 const char recentFileName[] = "SciTE.recent";
-const char defaultSessionFileName[] = "SciTE.ses";
+const char defaultSessionFileName[] = "SciTE.session";
 
-Job::Job() {
-	Clear();
-}
-
-void Job::Clear() {
-	command = "";
-	directory.Init();
-	jobType = jobCLI;
-	input = "";
-	flags = 0;
-}
-
-BufferList::BufferList() : current(0), stackcurrent(0), buffers(0), size(0), length(0), initialised(false) {}
+BufferList::BufferList() : current(0), stackcurrent(0), stack(0), buffers(0), size(0), length(0), initialised(false) {}
 
 BufferList::~BufferList() {
 	delete []buffers;
@@ -115,7 +106,7 @@ int BufferList::GetDocumentByName(FilePath filename) {
 
 void BufferList::RemoveCurrent() {
 	// Delete and move up to fill gap but ensure doc pointer is saved.
-	int currentDoc = buffers[current].doc;
+	sptr_t currentDoc = buffers[current].doc;
 	for (int i = current;i < length - 1;i++) {
 		buffers[i] = buffers[i + 1];
 	}
@@ -152,7 +143,7 @@ void BufferList::SetCurrent(int index) {
 }
 
 void BufferList::PopStack() {
-	for (int i=0; i<length-1; ++i) {
+	for (int i = 0; i < length - 1; ++i) {
 		int index = stack[i + 1];
 		// adjust the index for items that will move in buffers[]
 		if (index > current)
@@ -176,7 +167,7 @@ int BufferList::StackPrev() {
 void BufferList::MoveToStackTop(int index) {
 	// shift top chunk of stack down into the slot that index occupies
 	bool move = false;
-	for (int i=length-1; i>0; --i) {
+	for (int i = length - 1; i > 0; --i) {
 		if (stack[i] == index)
 			move = true;
 		if (move)
@@ -192,6 +183,30 @@ void BufferList::CommitStackSelection() {
 	stackcurrent = 0;
 }
 
+void BufferList::ShiftTo(int indexFrom, int indexTo) {
+	// shift buffer to new place in buffers array
+	if (indexFrom == indexTo ||
+		indexFrom < 0 || indexFrom >= length ||
+		indexTo < 0 || indexTo >= length) return;
+	int step = (indexFrom > indexTo) ? -1 : 1;
+	Buffer tmp = buffers[indexFrom];
+	int i;
+	for (i = indexFrom; i != indexTo; i += step) {
+		buffers[i] = buffers[i+step];
+	}
+	buffers[indexTo] = tmp;
+	// update stack indexes
+	for (i = 0; i < length; i++) {
+		if (stack[i] == indexFrom) {
+			stack[i] = indexTo;
+		} else if (step == 1) {
+			if (indexFrom < stack[i] && stack[i] <= indexTo) stack[i] -= step;
+		} else {
+			if (indexFrom > stack[i] && stack[i] >= indexTo) stack[i] -= step;
+		}
+	}
+}
+
 sptr_t SciTEBase::GetDocumentAt(int index) {
 	if (index < 0 || index >= buffers.size) {
 		//Platform::DebugPrintf("SciTEBase::GetDocumentAt: Index out of range.\n");
@@ -231,41 +246,9 @@ void SciTEBase::SetDocumentAt(int index, bool updateStack) {
 	Buffer bufferNext = buffers.buffers[buffers.Current()];
 	SetFileName(bufferNext);
 	SendEditor(SCI_SETDOCPOINTER, 0, GetDocumentAt(buffers.Current()));
-	SetWindowName();
-	ReadProperties();
-	if (CurrentBuffer()->unicodeMode != uni8Bit) {
-		// Override the code page if Unicode
-		codePage = SC_CP_UTF8;
-		SendEditor(SCI_SETCODEPAGE, codePage);
-	}
-	isReadOnly = SendEditor(SCI_GETREADONLY);
+	RestoreState(bufferNext);
 
-	// check to see whether there is saved fold state, restore
-	bufferNext.foldState.BeginIteration();
-	// Platform::DebugPrintf("Restoring fold state... (%d states)", count);
-
-	int line = 0;
-	bool folded = false;
-	while (bufferNext.foldState.GetState(&line, &folded)) {
-		bool expanded = SendEditor(SCI_GETFOLDEXPANDED, line);
-		// set line to state folded
-		if (folded && !expanded) {
-			SendEditor(SCI_TOGGLEFOLD, line);
-		}
-		if (!folded && expanded) {
-			SendEditor(SCI_TOGGLEFOLD, line);
-		}
-	}
-
-#if PLAT_WIN
-	// Tab Bar
-	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()), TCM_SETCURSEL, (WPARAM)index, (LPARAM)0);
-#endif
-#if PLAT_GTK
-
-	if (wTabBar.GetID())
-		gtk_notebook_set_page(GTK_NOTEBOOK(wTabBar.GetID()), index);
-#endif
+	TabSelect(index);
 
 	if (lineNumbers && lineNumbersExpand)
 		SetLineNumberWidth();
@@ -288,19 +271,18 @@ void SciTEBase::UpdateBuffersCurrent() {
 		buffers.buffers[currentbuf].selection = GetSelection();
 		buffers.buffers[currentbuf].scrollPosition = GetCurrentScrollPosition();
 
-		// retrieve fold state and store in buffer state info
+		// Retrieve fold state and store in buffer state info
 		int maxLine = SendEditor(SCI_GETLINECOUNT);
 		int foldPoints = 0;
 
 		for (int line = 0; line < maxLine; line++) {
-			int level = SendEditor(SCI_GETFOLDLEVEL, line);
-			if (level & SC_FOLDLEVELHEADERFLAG) {
-
-				foldPoints ++;
+			if ((SendEditor(SCI_GETFOLDLEVEL, line) & SC_FOLDLEVELHEADERFLAG) &&
+				!SendEditor(SCI_GETFOLDEXPANDED, line)) {
+				foldPoints++;
 			}
 		}
 
-		FoldState* f = &buffers.buffers[currentbuf].foldState;
+		FoldState *f = &buffers.buffers[currentbuf].foldState;
 		f->Clear();
 
 		if (foldPoints > 0) {
@@ -309,11 +291,9 @@ void SciTEBase::UpdateBuffersCurrent() {
 			f->Alloc(foldPoints);
 
 			for (int line = 0; line < maxLine; line++) {
-				int level = SendEditor(SCI_GETFOLDLEVEL, line);
-				if (level & SC_FOLDLEVELHEADERFLAG) {
-
-					bool expanded = SendEditor(SCI_GETFOLDEXPANDED, line);
-					f->PushState(line, expanded);
+				if ((SendEditor(SCI_GETFOLDLEVEL, line) & SC_FOLDLEVELHEADERFLAG) &&
+					!SendEditor(SCI_GETFOLDEXPANDED, line)) {
+					f->Append(line);
 				}
 			}
 		}
@@ -340,9 +320,11 @@ bool SciTEBase::CanMakeRoom(bool maySaveIfDirty) {
 }
 
 void SciTEBase::ClearDocument() {
+	SendEditor(SCI_SETREADONLY, 0);
 	SendEditor(SCI_CLEARALL);
 	SendEditor(SCI_EMPTYUNDOBUFFER);
 	SendEditor(SCI_SETSAVEPOINT);
+	SendEditor(SCI_SETREADONLY, isReadOnly);
 }
 
 void SciTEBase::CreateBuffers() {
@@ -365,13 +347,10 @@ void SciTEBase::InitialiseBuffers() {
 		if (buffers.size == 1) {
 			// Single buffer mode, delete the Buffers main menu entry
 			DestroyMenuItem(menuBuffers, 0);
-#if PLAT_WIN
-			// Make previous change visible.
-			::DrawMenuBar(reinterpret_cast<HWND>(wSciTE.GetID()));
-#endif
 			// Destroy command "View Tab Bar" in the menu "View"
 			DestroyMenuItem(menuView, IDM_VIEWTABBAR);
-
+			// Make previous change visible.
+			RedrawMenu();
 		}
 	}
 }
@@ -420,6 +399,14 @@ void SciTEBase::SaveRecentStack() {
 	fclose(recentFile);
 }
 
+static SString SessionPropKey(int bufIndex, const char *bufKey) {
+	SString pKey = "buffer.";
+	pKey += SString(bufIndex + 1);
+	pKey += ".";
+	pKey += bufKey;
+	return pKey;
+}
+
 void SciTEBase::LoadSession(const char *sessionName) {
 	FilePath sessionPathName;
 	if (sessionName[0] == '\0') {
@@ -427,28 +414,64 @@ void SciTEBase::LoadSession(const char *sessionName) {
 	} else {
 		sessionPathName.Set(sessionName);
 	}
-	FILE *sessionFile = sessionPathName.Open(fileRead);
-	if (!sessionFile)
+
+	PropSetFile propsSession;
+	if (!propsSession.Read(sessionPathName, sessionPathName.Directory(), NULL, 0))
 		return;
+
 	// comment next line if you don't want to close all buffers before loading session
 	CloseAllBuffers(true);
-	int curr = -1, pos = 0;
-	char *file, line[MAX_PATH + 128];
+
+	int curr = -1;
 	for (int i = 0; i < bufferMax; i++) {
-		if (!fgets(line, sizeof (line), sessionFile))
-			break;
-		line[strcspn(line, "\r\n")] = '\0';	// Remove \r, \n, or \r\n.
-		if (sscanf(line, "<pos=%i>", &pos) != 1)
-			break;
-		file = strchr(line, '>') + 2;
-		//Platform::DebugPrintf("pos=%i file=:%s:", pos, file);
-		if (pos < 0) {
+		SString propKey = SessionPropKey(i, "path");
+		SString propStr = propsSession.Get(propKey.c_str());
+		if (propStr == "")
+			continue;
+
+		propKey = SessionPropKey(i, "position");
+		int pos = propsSession.GetInt(propKey.c_str());
+
+		AddFileToBuffer(propStr.c_str(), pos - 1);
+
+		propKey = SessionPropKey(i, "current");
+		if (propsSession.GetInt(propKey.c_str()))
 			curr = i;
-			pos = -pos;
+
+		if (props.GetInt("session.bookmarks")) {
+			propKey = SessionPropKey(i, "bookmarks");
+			propStr = propsSession.Get(propKey.c_str());
+			if (propStr.length()) {
+				char *buf = new char[propStr.length() + 1];
+				strcpy(buf, propStr.c_str());
+				char *p = strtok(buf, ",");
+				while (p != NULL) {
+					int line = atoi(p) - 1;
+					SendEditor(SCI_MARKERADD, line, markerBookmark);
+					p = strtok(NULL, ",");
+				}
+				delete []buf;
+			}
+		}
+
+		if (props.GetInt("fold") && !props.GetInt("fold.on.open") &&
+			props.GetInt("session.folds")) {
+			propKey = SessionPropKey(i, "folds");
+			propStr = propsSession.Get(propKey.c_str());
+			if (propStr.length()) {
+				SendEditor(SCI_COLOURISE, 0, -1);
+				char *buf = new char[propStr.length() + 1];
+				strcpy(buf, propStr.c_str());
+				char *p = strtok(buf, ",");
+				while (p != NULL) {
+					int line = atoi(p) - 1;
+					SendEditor(SCI_TOGGLEFOLD, line);
+					p = strtok(NULL, ",");
+				}
+				delete [] buf;
+			}
 		}
-		AddFileToBuffer(file, pos - 1);
 	}
-	fclose(sessionFile);
 
 	FilePath sessionFilePath = FilePath(sessionPathName).AbsolutePath();
 	// add/update SessionPath environment variable
@@ -468,15 +491,58 @@ void SciTEBase::SaveSession(const char *sessionName) {
 	FILE *sessionFile = sessionPathName.Open(fileWrite);
 	if (!sessionFile)
 		return;
+
+	fprintf(sessionFile, "# SciTE session file\n");
 	int curr = buffers.Current();
 	for (int i = 0; i < buffers.length; i++) {
 		if (buffers.buffers[i].IsSet() && !buffers.buffers[i].IsUntitled()) {
-			int pos;
+			SString propKey = SessionPropKey(i, "path");
+			fprintf(sessionFile, "\n%s=%s\n", propKey.c_str(), buffers.buffers[i].AsInternal());
+
 			SetDocumentAt(i);
-			pos = SendEditor(SCI_GETCURRENTPOS) + 1;
-			if (i == curr)
-				pos = -pos;
-			fprintf(sessionFile, "<pos=%i> %s\n", pos, buffers.buffers[i].AsInternal());
+			int pos = SendEditor(SCI_GETCURRENTPOS) + 1;
+			propKey = SessionPropKey(i, "position");
+			fprintf(sessionFile, "%s=%d\n", propKey.c_str(), pos);
+
+			if (i == curr) {
+				propKey = SessionPropKey(i, "current");
+				fprintf(sessionFile, "%s=1\n", propKey.c_str());
+			}
+
+			if (props.GetInt("session.bookmarks")) {
+				int line = -1;
+				bool found = false;
+				while ((line = SendEditor(SCI_MARKERNEXT, line + 1, 1 << markerBookmark)) >= 0) {
+					if (!found) {
+						propKey = SessionPropKey(i, "bookmarks");
+						fprintf(sessionFile, "%s=%d", propKey.c_str(), line + 1);
+						found = true;
+					} else {
+						fprintf(sessionFile, ",%d", line + 1);
+					}
+				}
+				if (found)
+					fprintf(sessionFile, "\n");
+			}
+
+			if (props.GetInt("fold") && props.GetInt("session.folds")) {
+				int maxLine = SendEditor(SCI_GETLINECOUNT);
+				bool found = false;
+				for (int line = 0; line < maxLine; line++) {
+					if ((SendEditor(SCI_GETFOLDLEVEL, line) & SC_FOLDLEVELHEADERFLAG) &&
+						!SendEditor(SCI_GETFOLDEXPANDED, line)) {
+						if (!found) {
+							propKey = SessionPropKey(i, "folds");
+							fprintf(sessionFile, "%s=%d", propKey.c_str(), line + 1);
+						found = true;
+					} else {
+							fprintf(sessionFile, ",%d", line + 1);
+						}
+					}
+				}
+				if (found)
+					fprintf(sessionFile, "\n");
+			}
 		}
 	}
 	fclose(sessionFile);
@@ -496,21 +562,21 @@ void SciTEBase::SetIndentSettings() {
 	// Either set the settings related to the extension or the default ones
 	SString fileNameForExtension = ExtensionFileName();
 	SString useTabsChars = props.GetNewExpand("use.tabs.",
-	                       fileNameForExtension.c_str());
+	        fileNameForExtension.c_str());
 	if (useTabsChars.length() != 0) {
 		SendEditor(SCI_SETUSETABS, useTabsChars.value());
 	} else {
 		SendEditor(SCI_SETUSETABS, useTabs);
 	}
 	SString tabSizeForExt = props.GetNewExpand("tab.size.",
-	                        fileNameForExtension.c_str());
+	        fileNameForExtension.c_str());
 	if (tabSizeForExt.length() != 0) {
 		SendEditor(SCI_SETTABWIDTH, tabSizeForExt.value());
 	} else if (tabSize != 0) {
 		SendEditor(SCI_SETTABWIDTH, tabSize);
 	}
 	SString indentSizeForExt = props.GetNewExpand("indent.size.",
-	                           fileNameForExtension.c_str());
+	        fileNameForExtension.c_str());
 	if (indentSizeForExt.length() != 0) {
 		SendEditor(SCI_SETINDENT, indentSizeForExt.value());
 	} else {
@@ -535,8 +601,8 @@ void SciTEBase::New() {
 
 	if ((buffers.size == 1) && (!buffers.buffers[0].IsUntitled())) {
 		AddFileToStack(buffers.buffers[0],
-		               buffers.buffers[0].selection,
-		               buffers.buffers[0].scrollPosition);
+		        buffers.buffers[0].selection,
+		        buffers.buffers[0].scrollPosition);
 	}
 
 	// If the current buffer is the initial untitled, clean buffer then overwrite it,
@@ -544,11 +610,11 @@ void SciTEBase::New() {
 	if ((buffers.length > 1) ||
 	        (buffers.Current() != 0) ||
 	        (buffers.buffers[0].isDirty) ||
-			(!buffers.buffers[0].IsUntitled())) {
-			if (buffers.size == buffers.length) {
-				Close(false, false, true);
-			}
-			buffers.SetCurrent(buffers.Add());
+	        (!buffers.buffers[0].IsUntitled())) {
+		if (buffers.size == buffers.length) {
+			Close(false, false, true);
+		}
+		buffers.SetCurrent(buffers.Add());
 	}
 
 	sptr_t doc = GetDocumentAt(buffers.Current());
@@ -558,8 +624,8 @@ void SciTEBase::New() {
 	filePath.Set(curDirectory, "");
 	SetFileName(filePath);
 	CurrentBuffer()->isDirty = false;
-	isBuilding = false;
-	isBuilt = false;
+	jobQueue.isBuilding = false;
+	jobQueue.isBuilt = false;
 	isReadOnly = false;	// No sense to create an empty, read-only buffer...
 
 	ClearDocument();
@@ -569,9 +635,29 @@ void SciTEBase::New() {
 		extender->InitBuffer(buffers.Current());
 }
 
+void SciTEBase::RestoreState(const Buffer &buffer) {
+	SetWindowName();
+	ReadProperties();
+	if (CurrentBuffer()->unicodeMode != uni8Bit) {
+		// Override the code page if Unicode
+		codePage = SC_CP_UTF8;
+		SendEditor(SCI_SETCODEPAGE, codePage);
+	}
+	isReadOnly = SendEditor(SCI_GETREADONLY);
+
+	// check to see whether there is saved fold state, restore
+	for (int fold = 0; fold < buffer.foldState.Folds(); fold++) {
+		SendEditor(SCI_TOGGLEFOLD, buffer.foldState.Line(fold));
+	}
+}
+
 void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew) {
 	bool closingLast = false;
 
+	if (extender) {
+		extender->OnClose(filePath.AsFileSystem());
+	}
+
 	if (buffers.size == 1) {
 		// With no buffer list, Close means close from MRU
 		closingLast = !(recentFileStack[0].IsSet());
@@ -594,6 +680,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 		} else {
 			if (extender)
 				extender->RemoveBuffer(buffers.Current());
+			ClearDocument();
 			buffers.RemoveCurrent();
 			if (extender && !makingRoomForNew)
 				extender->ActivateBuffer(buffers.Current());
@@ -604,35 +691,10 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 			SetFileName(bufferNext);
 		SendEditor(SCI_SETDOCPOINTER, 0, GetDocumentAt(buffers.Current()));
 		if (closingLast) {
-			SendEditor(SCI_SETREADONLY, 0);
 			ClearDocument();
 		}
 		if (updateUI) {
-			SetWindowName();
-			ReadProperties();
-			if (CurrentBuffer()->unicodeMode != uni8Bit) {
-				// Override the code page if Unicode
-				codePage = SC_CP_UTF8;
-				SendEditor(SCI_SETCODEPAGE, codePage);
-			}
-			isReadOnly = SendEditor(SCI_GETREADONLY);
-
-			// check to see whether there is saved fold state, restore
-			bufferNext.foldState.BeginIteration();
-
-			int line = 0;
-			bool folded = false;
-			while (bufferNext.foldState.GetState(&line, &folded)) {
-				bool expanded = SendEditor(SCI_GETFOLDEXPANDED, line);
-				// set line to state folded
-				if (folded && !expanded) {
-					SendEditor(SCI_TOGGLEFOLD, line);
-				}
-				if (!folded && expanded) {
-					SendEditor(SCI_TOGGLEFOLD, line);
-				}
-			}
-
+			RestoreState(bufferNext);
 			DisplayAround(bufferNext);
 		}
 	}
@@ -642,7 +704,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 		UpdateStatusBar(true);
 	}
 
-	if (extender && !closingLast) {
+	if (extender && !closingLast && !makingRoomForNew) {
 		extender->OnSwitchFile(filePath.AsFileSystem());
 	}
 
@@ -727,6 +789,32 @@ void SciTEBase::Prev() {
 	CheckReload();
 }
 
+void SciTEBase::ShiftTab(int indexFrom, int indexTo) {
+	buffers.ShiftTo(indexFrom, indexTo);
+	buffers.SetCurrent(indexTo);
+	BuffersMenu();
+
+	TabSelect(indexTo);
+
+	DisplayAround(buffers.buffers[buffers.Current()]);
+}
+
+void SciTEBase::MoveTabRight() {
+	if (buffers.length < 2) return;
+	int indexFrom = buffers.Current();
+	int indexTo = indexFrom + 1;
+	if (indexTo >= buffers.length) indexTo = 0;
+	ShiftTab(indexFrom, indexTo);
+}
+
+void SciTEBase::MoveTabLeft() {
+	if (buffers.length < 2) return;
+	int indexFrom = buffers.Current();
+	int indexTo = indexFrom - 1;
+	if (indexTo < 0) indexTo = buffers.length - 1;
+	ShiftTab(indexFrom, indexTo);
+}
+
 void SciTEBase::NextInStack() {
 	SetDocumentAt(buffers.StackNext(), false);
 	CheckReload();
@@ -744,20 +832,8 @@ void SciTEBase::EndStackedTabbing() {
 void SciTEBase::BuffersMenu() {
 	UpdateBuffersCurrent();
 	DestroyMenuItem(menuBuffers, IDM_BUFFERSEP);
-#if PLAT_WIN
+	RemoveAllTabs();
 
-	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()), TCM_DELETEALLITEMS, (WPARAM)0, (LPARAM)0);
-#endif
-
-#if PLAT_GTK
-
-	if (wTabBar.GetID()) {
-		GtkWidget *tab;
-
-		while ((tab = gtk_notebook_get_nth_page(GTK_NOTEBOOK(wTabBar.GetID()), 0)))
-			gtk_notebook_remove_page(GTK_NOTEBOOK(wTabBar.GetID()), 0);
-	}
-#endif
 	int pos;
 	for (pos = 0; pos < bufferMax; pos++) {
 		DestroyMenuItem(menuBuffers, IDM_BUFFER + pos);
@@ -780,7 +856,7 @@ void SciTEBase::BuffersMenu() {
 #endif
 
 			if (buffers.buffers[pos].IsUntitled()) {
-				SString untitled = LocaliseString("Untitled");
+				SString untitled = localiser.Text("Untitled");
 				strcat(entry, untitled.c_str());
 				strcat(titleTab, untitled.c_str());
 			} else {
@@ -813,35 +889,7 @@ void SciTEBase::BuffersMenu() {
 			}
 
 			SetMenuItem(menuBuffers, menuStart + pos + 1, itemID, entry);
-#if PLAT_WIN
-			// Windows specific !
-			TCITEM tie;
-			tie.mask = TCIF_TEXT | TCIF_IMAGE;
-			tie.iImage = -1;
-
-			tie.pszText = titleTab;
-			::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()), TCM_INSERTITEM, (WPARAM)pos, (LPARAM)&tie);
-			//::SendMessage(wTabBar.GetID(), TCM_SETCURSEL, (WPARAM)pos, (LPARAM)0);
-#endif
-#if PLAT_GTK
-
-			if (wTabBar.GetID()) {
-				GtkWidget *tablabel, *tabcontent;
-
-				tablabel = gtk_label_new(titleTab);
-
-				if (buffers.buffers[pos].IsUntitled())
-					tabcontent = gtk_label_new(LocaliseString("Untitled").c_str());
-				else
-					tabcontent = gtk_label_new(buffers.buffers[pos].AsInternal());
-
-				gtk_widget_show(tablabel);
-				gtk_widget_show(tabcontent);
-
-				gtk_notebook_append_page(GTK_NOTEBOOK(wTabBar.GetID()), tabcontent, tablabel);
-			}
-#endif
-
+			TabInsert(pos, titleTab);
 		}
 	}
 	CheckMenus();
@@ -945,23 +993,15 @@ RecentFile SciTEBase::GetFilePosition() {
 
 void SciTEBase::DisplayAround(const RecentFile &rf) {
 	if ((rf.selection.cpMin != INVALID_POSITION) && (rf.selection.cpMax != INVALID_POSITION)) {
-		// This can produce better file state restoring
-		bool foldOnOpen = props.GetInt("fold.on.open");
-		//~ if (foldOnOpen)
-			//~ FoldAll();
-
 		int lineStart = SendEditor(SCI_LINEFROMPOSITION, rf.selection.cpMin);
 		SendEditor(SCI_ENSUREVISIBLEENFORCEPOLICY, lineStart);
 		int lineEnd = SendEditor(SCI_LINEFROMPOSITION, rf.selection.cpMax);
 		SendEditor(SCI_ENSUREVISIBLEENFORCEPOLICY, lineEnd);
 		SetSelection(rf.selection.cpMax, rf.selection.cpMin);
 
-		// Folding can mess up next scrolling, so will be better without scrolling
-		if (!foldOnOpen) {
-			int curTop = SendEditor(SCI_GETFIRSTVISIBLELINE);
-			int lineTop = SendEditor(SCI_VISIBLEFROMDOCLINE, rf.scrollPosition);
-			SendEditor(SCI_LINESCROLL, 0, lineTop - curTop);
-		}
+		int curTop = SendEditor(SCI_GETFIRSTVISIBLELINE);
+		int lineTop = SendEditor(SCI_VISIBLEFROMDOCLINE, rf.scrollPosition);
+		SendEditor(SCI_LINESCROLL, 0, lineTop - curTop);
 	}
 }
 
@@ -1042,8 +1082,8 @@ void SciTEBase::RemoveToolsMenu() {
 }
 
 void SciTEBase::SetMenuItemLocalised(int menuNumber, int position, int itemID,
-                                     const char *text, const char *mnemonic) {
-	SString localised = LocaliseString(text);
+        const char *text, const char *mnemonic) {
+	SString localised = localiser.Text(text);
 	SetMenuItem(menuNumber, position, itemID, localised.c_str(), mnemonic);
 }
 
@@ -1082,13 +1122,13 @@ void SciTEBase::SetToolsMenu() {
 	if (macrosEnabled) {
 		SetMenuItem(menuTools, menuPos++, IDM_MACRO_SEP, "");
 		SetMenuItemLocalised(menuTools, menuPos++, IDM_MACROLIST,
-		                     "&List Macros...", "Shift+F9");
+		        "&List Macros...", "Shift+F9");
 		SetMenuItemLocalised(menuTools, menuPos++, IDM_MACROPLAY,
-		                     "Run Current &Macro", "F9");
+		        "Run Current &Macro", "F9");
 		SetMenuItemLocalised(menuTools, menuPos++, IDM_MACRORECORD,
-		                     "&Record Macro", "Ctrl+F9");
+		        "&Record Macro", "Ctrl+F9");
 		SetMenuItemLocalised(menuTools, menuPos++, IDM_MACROSTOPRECORD,
-		                     "S&top Recording Macro", "Ctrl+Shift+F9");
+		        "S&top Recording Macro", "Ctrl+Shift+F9");
 	}
 }
 
@@ -1159,56 +1199,56 @@ void SciTEBase::ToolsMenu(int item) {
 				}
 
 				if (0 == strcmp(opt, "subsystem") && colon) {
-					if (colon[0] == '0' || 0==strcmp(colon, "console"))
+					if (colon[0] == '0' || 0 == strcmp(colon, "console"))
 						jobType = jobCLI;
-					else if (colon[0] == '1' || 0==strcmp(colon, "windows"))
+					else if (colon[0] == '1' || 0 == strcmp(colon, "windows"))
 						jobType = jobGUI;
-					else if (colon[0] == '2' || 0==strcmp(colon, "shellexec"))
+					else if (colon[0] == '2' || 0 == strcmp(colon, "shellexec"))
 						jobType = jobShell;
-					else if (colon[0] == '3' || 0==strcmp(colon, "lua") || strcmp(colon, "director"))
+					else if (colon[0] == '3' || 0 == strcmp(colon, "lua") || strcmp(colon, "director"))
 						jobType = jobExtension;
-					else if (colon[0] == '4' || 0==strcmp(colon, "winhelp"))
+					else if (colon[0] == '4' || 0 == strcmp(colon, "winhelp"))
 						jobType = jobHelp;
-					else if (colon[0] == '5' || 0==strcmp(colon, "htmlhelp"))
+					else if (colon[0] == '5' || 0 == strcmp(colon, "htmlhelp"))
 						jobType = jobOtherHelp;
 				}
 
 				if (0 == strcmp(opt, "quiet")) {
-					if (!colon || colon[0] == '1' || 0==strcmp(colon, "yes"))
+					if (!colon || colon[0] == '1' || 0 == strcmp(colon, "yes"))
 						quiet = true;
-					else if (colon[0] == '0' || 0==strcmp(colon, "no"))
+					else if (colon[0] == '0' || 0 == strcmp(colon, "no"))
 						quiet = false;
 				}
 
 				if (0 == strcmp(opt, "savebefore")) {
-					if (!colon || colon[0] == '1' || 0==strcmp(colon, "yes"))
+					if (!colon || colon[0] == '1' || 0 == strcmp(colon, "yes"))
 						saveBefore = 1;
-					else if (colon[0] == '0' || 0==strcmp(colon, "no"))
+					else if (colon[0] == '0' || 0 == strcmp(colon, "no"))
 						saveBefore = 2;
-					else if (0==strcmp(colon, "prompt"))
+					else if (0 == strcmp(colon, "prompt"))
 						saveBefore = 0;
 				}
 
 				if (0 == strcmp(opt, "filter")) {
-					if (!colon || colon[0] == '1' || 0==strcmp(colon, "yes"))
+					if (!colon || colon[0] == '1' || 0 == strcmp(colon, "yes"))
 						filter = true;
-					else if (colon[1] == '0' || 0==strcmp(colon, "no"))
+					else if (colon[1] == '0' || 0 == strcmp(colon, "no"))
 						filter = false;
 				}
 
 				if (0 == strcmp(opt, "replaceselection")) {
-					if (!colon || colon[0] == '1' || 0==strcmp(colon, "yes"))
+					if (!colon || colon[0] == '1' || 0 == strcmp(colon, "yes"))
 						repSel = 1;
-					else if (colon[0] == '0' || 0==strcmp(colon, "no"))
+					else if (colon[0] == '0' || 0 == strcmp(colon, "no"))
 						repSel = 0;
-					else if (0==strcmp(colon, "auto"))
+					else if (0 == strcmp(colon, "auto"))
 						repSel = 2;
 				}
 
 				if (0 == strcmp(opt, "groupundo")) {
-					if (!colon || colon[0] == '1' || 0==strcmp(colon, "yes"))
+					if (!colon || colon[0] == '1' || 0 == strcmp(colon, "yes"))
 						groupUndo = true;
-					else if (colon[0] == '0' || 0==strcmp(colon, "no"))
+					else if (colon[0] == '0' || 0 == strcmp(colon, "no"))
 						groupUndo = false;
 				}
 
@@ -1234,7 +1274,7 @@ void SciTEBase::ToolsMenu(int item) {
 			propName = "command.is.filter.";
 			propName += itemSuffix;
 			if (props.GetWild(propName.c_str(), FileNameExt().AsInternal()).length())
-				filter = (props.GetNewExpand(propName.c_str(), FileNameExt().AsInternal())[0]=='1');
+				filter = (props.GetNewExpand(propName.c_str(), FileNameExt().AsInternal())[0] == '1');
 			if (filter)
 				CurrentBuffer()->fileModTime -= 1;
 
@@ -1256,7 +1296,7 @@ void SciTEBase::ToolsMenu(int item) {
 			propName = "command.quiet.";
 			propName += itemSuffix;
 			if (props.GetWild(propName.c_str(), FileNameExt().AsInternal()).length())
-				quiet = (props.GetNewExpand(propName.c_str(), FileNameExt().AsInternal()).value()==1);
+				quiet = (props.GetNewExpand(propName.c_str(), FileNameExt().AsInternal()).value() == 1);
 			if (quiet)
 				flags |= jobQuiet;
 
@@ -1274,24 +1314,24 @@ void SciTEBase::ToolsMenu(int item) {
 				flags |= jobGroupUndo;
 
 			AddCommand(command, "", jobType, input, flags);
-			if (commandCurrent > 0)
+			if (jobQueue.commandCurrent > 0)
 				Execute();
 		}
 	}
 }
 
 inline bool isdigitchar(int ch) {
-    return (ch >= '0') && (ch <= '9');
+	return (ch >= '0') && (ch <= '9');
 }
 
-int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
+int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 	sourcePath[0] = '\0';
-	column= -1; // default to not detected
+	column = -1; // default to not detected
 	switch (format) {
 	case SCE_ERR_PYTHON: {
 			// Python
-			char *startPath = strchr(cdoc, '\"') + 1;
-			char *endPath = strchr(startPath, '\"');
+			const char *startPath = strchr(cdoc, '\"') + 1;
+			const char *endPath = strchr(startPath, '\"');
 			int length = endPath - startPath;
 			if (length > 0) {
 				strncpy(sourcePath, startPath, length);
@@ -1324,11 +1364,11 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 		}
 	case SCE_ERR_MS: {
 			// Visual *
-			char *start = cdoc;
+			const char *start = cdoc;
 			while (isspacechar(*start)) {
 				start++;
 			}
-			char *endPath = strchr(start, '(');
+			const char *endPath = strchr(start, '(');
 			int length = endPath - start;
 			if ((length > 0) && (length < MAX_PATH)) {
 				strncpy(sourcePath, start, length);
@@ -1339,7 +1379,7 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 		}
 	case SCE_ERR_BORLAND: {
 			// Borland
-			char *space = strchr(cdoc, ' ');
+			const char *space = strchr(cdoc, ' ');
 			if (space) {
 				while (isspacechar(*space)) {
 					space++;
@@ -1351,7 +1391,7 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 					space++;
 				}
 
-				char *space2 = NULL;
+				const char *space2 = NULL;
 
 				if (strlen(space) > 2) {
 					space2 = strchr(space + 2, ':');
@@ -1379,8 +1419,8 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 		}
 	case SCE_ERR_PERL: {
 			// perl
-			char *at = strstr(cdoc, " at ");
-			char *line = strstr(cdoc, " line ");
+			const char *at = strstr(cdoc, " at ");
+			const char *line = strstr(cdoc, " line ");
 			int length = line - (at + 4);
 			if (at && line && length > 0) {
 				strncpy(sourcePath, at + 4, length);
@@ -1392,8 +1432,8 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 		}
 	case SCE_ERR_NET: {
 			// .NET traceback
-			char *in = strstr(cdoc, " in ");
-			char *line = strstr(cdoc, ":line ");
+			const char *in = strstr(cdoc, " in ");
+			const char *line = strstr(cdoc, ":line ");
 			if (in && line && (line > in)) {
 				in += 4;
 				strncpy(sourcePath, in, line - in);
@@ -1403,16 +1443,16 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 			}
 		}
 	case SCE_ERR_LUA: {
-			// Lua error look like: last token read: `result' at line 40 in file `Test.lua'
-			char *idLine = "at line ";
-			char *idFile = "file ";
+			// Lua 4 error looks like: last token read: `result' at line 40 in file `Test.lua'
+			const char *idLine = "at line ";
+			const char *idFile = "file ";
 			size_t lenLine = strlen(idLine);
 			size_t lenFile = strlen(idFile);
-			char *line = strstr(cdoc, idLine);
-			char *file = strstr(cdoc, idFile);
+			const char *line = strstr(cdoc, idLine);
+			const char *file = strstr(cdoc, idFile);
 			if (line && file) {
-				char *fileStart = file + lenFile + 1;
-				char *quote = strstr(fileStart, "'");
+				const char *fileStart = file + lenFile + 1;
+				const char *quote = strstr(fileStart, "'");
 				size_t length = quote - fileStart;
 				if (quote && length > 0) {
 					strncpy(sourcePath, fileStart, length);
@@ -1420,6 +1460,12 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 				}
 				line += lenLine;
 				return atoi(line) - 1;
+			} else {
+				// Lua 5.1 error looks like: lua.exe: test1.lua:3: syntax error
+				// reuse the GCC error parsing code above!
+				const char* colon = strstr(cdoc,": ");
+				if (cdoc)
+					return DecodeMessage(colon + 2,sourcePath,SCE_ERR_GCC,column);
 			}
 			break;
 		}
@@ -1443,12 +1489,12 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 		}
 	case SCE_ERR_PHP: {
 			// PHP error look like: Fatal error: Call to undefined function:  foo() in example.php on line 11
-			char *idLine = " on line ";
-			char *idFile = " in ";
+			const char *idLine = " on line ";
+			const char *idFile = " in ";
 			size_t lenLine = strlen(idLine);
 			size_t lenFile = strlen(idFile);
-			char *line = strstr(cdoc, idLine);
-			char *file = strstr(cdoc, idFile);
+			const char *line = strstr(cdoc, idLine);
+			const char *file = strstr(cdoc, idFile);
 			if (line && file && (line > file)) {
 				file += lenFile;
 				size_t length = line - file;
@@ -1462,12 +1508,12 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 
 	case SCE_ERR_ELF: {
 			// Essential Lahey Fortran error look like: Line 11, file c:\fortran90\codigo\demo.f90
-			char *line = strchr(cdoc, ' ');
+			const char *line = strchr(cdoc, ' ');
 			if (line) {
 				while (isspacechar(*line)) {
 					line++;
 				}
-				char *file = strchr(line, ' ');
+				const char *file = strchr(line, ' ');
 				if (file) {
 					while (isspacechar(*file)) {
 						file++;
@@ -1494,11 +1540,11 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 			 *
 			 * These are trapped by the MS handler, and are identified OK, so no problem...
 			 */
-			char *line = strchr(cdoc, '(');
-			char *file = strchr(line, ':');
+			const char *line = strchr(cdoc, '(');
+			const char *file = strchr(line, ':');
 			if (line && file) {
 				file++;
-				char *endfile = strchr(file, ')');
+				const char *endfile = strchr(file, ')');
 				size_t length = endfile - file;
 				strncpy(sourcePath, file, length);
 				sourcePath[length] = '\0';
@@ -1510,14 +1556,14 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 
 	case SCE_ERR_ABSF: {
 			// Absoft Pro Fortran 90/95 v8.x, v9.x  errors look like: cf90-113 f90fe: ERROR SHF3D, File = shf.f90, Line = 1101, Column = 19
-			char *idFile = " File = ";
-			char *idLine = ", Line = ";
-			char *idColumn = ", Column = ";
+			const char *idFile = " File = ";
+			const char *idLine = ", Line = ";
+			const char *idColumn = ", Column = ";
 			size_t lenFile = strlen(idFile);
 			size_t lenLine = strlen(idLine);
-			char *file = strstr(cdoc, idFile);
-			char *line = strstr(cdoc, idLine);
-			char *column = strstr(cdoc, idColumn);
+			const char *file = strstr(cdoc, idFile);
+			const char *line = strstr(cdoc, idLine);
+			const char *column = strstr(cdoc, idColumn);
 			if (line && file && (line > file)) {
 				file += lenFile;
 				size_t length = line - file;
@@ -1525,7 +1571,6 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 				sourcePath[length] = '\0';
 				line += lenLine;
 				length = column - line;
-				strncpy(line, line, length);
 				return atoi(line) - 1;
 			}
 			break;
@@ -1535,13 +1580,13 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 			/* Intel Fortran Compiler v8.x error/warnings look like:
 			 * fortcom: Error: shf.f90, line 5602: This name does not have ...
 				 */
-			char *idFile = ": Error: ";
-			char *idLine = ", line ";
+			const char *idFile = ": Error: ";
+			const char *idLine = ", line ";
 			size_t lenFile = strlen(idFile);
 			size_t lenLine = strlen(idLine);
-			char *file = strstr(cdoc, idFile);
-			char *line = strstr(cdoc, idLine);
-			char *lineend = strrchr(cdoc, ':');
+			const char *file = strstr(cdoc, idFile);
+			const char *line = strstr(cdoc, idLine);
+			const char *lineend = strrchr(cdoc, ':');
 			if (line && file && (line > file)) {
 				file += lenFile;
 				size_t length = line - file;
@@ -1550,7 +1595,6 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 				line += lenLine;
 				if ((lineend > line)) {
 					length = lineend - line;
-					strncpy(line, line, length);
 					return atoi(line) - 1;
 				}
 			}
@@ -1562,18 +1606,18 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 			 * line 8 column 1 - Error: unexpected </head> in <meta>
 			 * line 41 column 1 - Warning: <table> lacks "summary" attribute
 			 */
-			char *line = strchr(cdoc, ' ');
+			const char *line = strchr(cdoc, ' ');
 			if (line) {
-				char *col = strchr(line+1, ' ');
+				const char *col = strchr(line + 1, ' ');
 				if (col) {
-					*col = '\0';
+					//*col = '\0';
 					int lnr = atoi(line) - 1;
-					col = strchr(col+1, ' ');
+					col = strchr(col + 1, ' ');
 					if (col) {
-						char *endcol = strchr(col+1, ' ');
+						const char *endcol = strchr(col + 1, ' ');
 						if (endcol) {
-							*endcol = '\0';
-							column = atoi(col)-1;
+							//*endcol = '\0';
+							column = atoi(col) - 1;
 							return lnr;
 						}
 					}
@@ -1586,13 +1630,13 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 			/* Java runtime stack trace
 				\tat <methodname>(<filename>:<line>)
 				 */
-			char *startPath = strrchr(cdoc, '(') + 1;
-			char *endPath = strchr(startPath, ':');
+			const char *startPath = strrchr(cdoc, '(') + 1;
+			const char *endPath = strchr(startPath, ':');
 			int length = endPath - startPath;
 			if (length > 0) {
 				strncpy(sourcePath, startPath, length);
 				sourcePath[length] = 0;
-				int sourceNumber = atoi(endPath+1) - 1;
+				int sourceNumber = atoi(endPath + 1) - 1;
 				return sourceNumber;
 			}
 			break;
@@ -1601,8 +1645,8 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 	case SCE_ERR_DIFF_MESSAGE: {
 			// Diff file header, either +++ <filename>\t or --- <filename>\t
 			// Often followed by a position line @@ <linenumber>
-			char *startPath = cdoc + 4;
-			char *endPath = strchr(startPath, '\t');
+			const char *startPath = cdoc + 4;
+			const char *endPath = strchr(startPath, '\t');
 			if (endPath) {
 				int length = endPath - startPath;
 				strncpy(sourcePath, startPath, length);
@@ -1634,28 +1678,23 @@ void SciTEBase::GoMessage(int dir) {
 		//Platform::DebugPrintf("GOMessage %d %d %d of %d linestart = %d\n", selStart, curLine, lookLine, maxLine, startPosLine);
 		char style = acc.StyleAt(startPosLine);
 		if (style != SCE_ERR_DEFAULT &&
-			style != SCE_ERR_CMD &&
-			style != SCE_ERR_DIFF_ADDITION &&
-			style != SCE_ERR_DIFF_CHANGED &&
-			style != SCE_ERR_DIFF_DELETION) {
+		        style != SCE_ERR_CMD &&
+		        style != SCE_ERR_DIFF_ADDITION &&
+		        style != SCE_ERR_DIFF_CHANGED &&
+		        style != SCE_ERR_DIFF_DELETION) {
 			//Platform::DebugPrintf("Marker to %d\n", lookLine);
 			SendOutput(SCI_MARKERDELETEALL, static_cast<uptr_t>(-1));
 			SendOutput(SCI_MARKERDEFINE, 0, SC_MARK_SMALLRECT);
 			SendOutput(SCI_MARKERSETFORE, 0, ColourOfProperty(props,
-			           "error.marker.fore", ColourDesired(0x7f, 0, 0)));
+			        "error.marker.fore", ColourDesired(0x7f, 0, 0)));
 			SendOutput(SCI_MARKERSETBACK, 0, ColourOfProperty(props,
-			           "error.marker.back", ColourDesired(0xff, 0xff, 0)));
+			        "error.marker.back", ColourDesired(0xff, 0xff, 0)));
 			SendOutput(SCI_MARKERADD, lookLine, 0);
 			SendOutput(SCI_SETSEL, startPosLine, startPosLine);
-			char *cdoc = new char[lineLength + 1];
-			if (!cdoc)
-				return;
-			GetRange(wOutput, startPosLine, startPosLine + lineLength, cdoc);
-			SString message(cdoc);
+			SString message = GetRange(wOutput, startPosLine, startPosLine + lineLength);
 			char source[MAX_PATH];
 			int column;
-			int sourceLine = DecodeMessage(cdoc, source, style, column);
-			//Platform::DebugPrintf("<%s> %d %d\n",source, sourceLine, lookLine);
+			int sourceLine = DecodeMessage(message.c_str(), source, style, column);
 			if (sourceLine >= 0) {
 				FilePath sourcePath(source);
 				if (!filePath.Name().SameNameAs(sourcePath)) {
@@ -1680,7 +1719,6 @@ void SciTEBase::GoMessage(int dir) {
 					}
 					if (bExists) {
 						if (!Open(messagePath)) {
-							delete []cdoc;
 							return;
 						}
 						CheckReload();
@@ -1709,18 +1747,18 @@ void SciTEBase::GoMessage(int dir) {
 				SendEditor(SCI_MARKERDELETEALL, 0);
 				SendEditor(SCI_MARKERDEFINE, 0, SC_MARK_CIRCLE);
 				SendEditor(SCI_MARKERSETFORE, 0, ColourOfProperty(props,
-				           "error.marker.fore", ColourDesired(0x7f, 0, 0)));
+				        "error.marker.fore", ColourDesired(0x7f, 0, 0)));
 				SendEditor(SCI_MARKERSETBACK, 0, ColourOfProperty(props,
-				           "error.marker.back", ColourDesired(0xff, 0xff, 0)));
+				        "error.marker.back", ColourDesired(0xff, 0xff, 0)));
 				SendEditor(SCI_MARKERADD, sourceLine, 0);
 				int startSourceLine = SendEditor(SCI_POSITIONFROMLINE, sourceLine, 0);
-				int endSourceline= SendEditor(SCI_POSITIONFROMLINE, sourceLine+1, 0);
+				int endSourceline = SendEditor(SCI_POSITIONFROMLINE, sourceLine + 1, 0);
 				if (column >= 0) {
 					// Get the position in line according to current tab setting
 					startSourceLine = SendEditor(SCI_FINDCOLUMN, sourceLine, column);
 				}
 				EnsureRangeVisible(startSourceLine, startSourceLine);
-				if (props.GetInt("error.select.line")==1) {
+				if (props.GetInt("error.select.line") == 1) {
 					//select whole source source line from column with error
 					SetSelection(endSourceline, startSourceLine);
 				} else {
@@ -1733,7 +1771,6 @@ void SciTEBase::GoMessage(int dir) {
 				UpdateStatusBar(false);
 				WindowSetFocus(wEditor);
 			}
-			delete []cdoc;
 			return;
 		}
 		lookLine += dir;
@@ -1745,3 +1782,4 @@ void SciTEBase::GoMessage(int dir) {
 			return;
 	}
 }
+
