@@ -52,132 +52,17 @@ const char menuAccessIndicator[] = "&";
 
 #include "SciTE.h"
 #include "PropSet.h"
+#include "StringList.h"
 #include "Accessor.h"
 #include "Scintilla.h"
 #include "SciLexer.h"
 #include "Extender.h"
 #include "FilePath.h"
 #include "PropSetFile.h"
+#include "Mutex.h"
+#include "JobQueue.h"
 #include "SciTEBase.h"
-
-PropSetFile::PropSetFile(bool lowerKeys_) : lowerKeys(lowerKeys_) {}
-
-PropSetFile::~PropSetFile() {}
-
-/**
- * Get a line of input. If end of line escaped with '\\' then continue reading.
- */
-static bool GetFullLine(const char *&fpc, int &lenData, char *s, int len) {
-	bool continuation = true;
-	s[0] = '\0';
-	while ((len > 1) && lenData > 0) {
-		char ch = *fpc;
-		fpc++;
-		lenData--;
-		if ((ch == '\r') || (ch == '\n')) {
-			if (!continuation) {
-				if ((lenData > 0) && (ch == '\r') && ((*fpc) == '\n')) {
-					// munch the second half of a crlf
-					fpc++;
-					lenData--;
-				}
-				*s = '\0';
-				return true;
-			}
-		} else if ((ch == '\\') && (lenData > 0) && ((*fpc == '\r') || (*fpc == '\n'))) {
-			continuation = true;
-			if ((lenData > 1) && ((*fpc == '\r') && (*(fpc+1) == '\r') || (*fpc == '\n') && (*(fpc+1) == '\n')))
-				continuation = false;
-			else if ((lenData > 2) && ((*fpc == '\r') && (*(fpc+1) == '\n') && (*(fpc+2) == '\n' || *(fpc+2) == '\r')))
-				continuation = false;
-		} else {
-			continuation = false;
-			*s++ = ch;
-			*s = '\0';
-			len--;
-		}
-	}
-	return false;
-}
-
-static bool IsSpaceOrTab(char ch) {
-	return (ch == ' ') || (ch == '\t');
-}
-
-static bool IsCommentLine(const char *line) {
-	while (IsSpaceOrTab(*line)) ++line;
-	return (*line == '#');
-}
-
-bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, FilePath directoryForImports,
-                           FilePath imports[], int sizeImports) {
-	//UnSlash(lineBuffer);
-	if (!IsSpaceOrTab(lineBuffer[0]))    // If clause ends with first non-indented line
-		ifIsTrue = true;
-	if (isprefix(lineBuffer, "if ")) {
-		const char *expr = lineBuffer + strlen("if") + 1;
-		ifIsTrue = GetInt(expr);
-	} else if (isprefix(lineBuffer, "import ") && directoryForImports.IsSet()) {
-		SString importName(lineBuffer + strlen("import") + 1);
-		importName += ".properties";
-		FilePath importPath(directoryForImports, FilePath(importName.c_str()));
-		if (Read(importPath, directoryForImports, imports, sizeImports)) {
-			if (imports) {
-				for (int i = 0; i < sizeImports; i++) {
-					if (!imports[i].IsSet()) {
-						imports[i] = importPath;
-						break;
-					}
-				}
-			}
-		}
-	} else if (ifIsTrue && !IsCommentLine(lineBuffer)) {
-		Set(lineBuffer);
-	}
-	return ifIsTrue;
-}
-
-void PropSetFile::ReadFromMemory(const char *data, int len, FilePath directoryForImports,
-                                 FilePath imports[], int sizeImports) {
-	const char *pd = data;
-	char lineBuffer[60000];
-	bool ifIsTrue = true;
-	while (len > 0) {
-		GetFullLine(pd, len, lineBuffer, sizeof(lineBuffer));
-		if (lowerKeys) {
-			for (int i=0; lineBuffer[i] && (lineBuffer[i] != '='); i++) {
-				if ((lineBuffer[i] >= 'A') && (lineBuffer[i] <= 'Z')) {
-					lineBuffer[i] = static_cast<char>(lineBuffer[i] - 'A' + 'a');
-				}
-			}
-		}
-		ifIsTrue = ReadLine(lineBuffer, ifIsTrue, directoryForImports, imports, sizeImports);
-	}
-}
-
-bool PropSetFile::Read(FilePath filename, FilePath directoryForImports,
-                       FilePath imports[], int sizeImports) {
-	FILE *rcfile = filename.Open(fileRead);
-	if (rcfile) {
-		char propsData[60000];
-		int lenFile = static_cast<int>(fread(propsData, 1, sizeof(propsData), rcfile));
-		fclose(rcfile);
-		const char *data = propsData;
-		if (memcmp(data, "\xef\xbb\xbf", 3) == 0) {
-			data += 3;
-			lenFile -= 3;
-		}
-		ReadFromMemory(data, lenFile, directoryForImports, imports, sizeImports);
-		return true;
-	}
-	return false;
-}
-
-void PropSetFile::SetInteger(const char *key, sptr_t i) {
-	char tmp[32];
-	sprintf(tmp, "%d", static_cast<int>(i));
-	Set(key, tmp);
-}
+#include "IFaceTable.h"
 
 void SciTEBase::SetImportMenu() {
 	for (int i = 0; i < importMax; i++) {
@@ -187,7 +72,7 @@ void SciTEBase::SetImportMenu() {
 		for (int stackPos = 0; stackPos < importMax; stackPos++) {
 			int itemID = importCmdID + stackPos;
 			if (importFiles[stackPos].IsSet()) {
-				SString entry = LocaliseString("Open");
+				SString entry = localiser.Text("Open");
 				entry += " ";
 				entry += importFiles[stackPos].Name().AsInternal();
 				SetMenuItem(menuOptions, IMPORT_START + stackPos, itemID, entry.c_str());
@@ -211,7 +96,7 @@ void SciTEBase::SetLanguageMenu() {
 	}
 	for (int item = 0; item < languageItems; item++) {
 		int itemID = languageCmdID + item;
-		SString entry = LocaliseString(languageMenu[item].menuItem.c_str());
+		SString entry = localiser.Text(languageMenu[item].menuItem.c_str());
 		if (languageMenu[item].menuKey.length()) {
 #if PLAT_GTK
 			entry += " ";
@@ -226,8 +111,12 @@ void SciTEBase::SetLanguageMenu() {
 	}
 }
 
-const char propFileName[] = "SciTE.properties";
+const char propLocalFileName[] = "SciTE.properties";
+const char propDirectoryFileName[] = "SciTEDirectory.properties";
 
+/**
+Read global and user properties files.
+*/
 void SciTEBase::ReadGlobalPropFile() {
 #ifdef unix
 	extern char **environ;
@@ -258,8 +147,8 @@ void SciTEBase::ReadGlobalPropFile() {
 	FilePath propfileUser = GetUserPropertiesFileName();
 	propsUser.Read(propfileUser, propfileUser.Directory(), importFiles, importMax);
 
-	if (!localisationRead) {
-		ReadLocalisation();
+	if (!localiser.read) {
+		ReadLocalization();
 	}
 }
 
@@ -268,13 +157,35 @@ void SciTEBase::ReadAbbrevPropFile() {
 	propsAbbrev.Read(pathAbbreviations, pathAbbreviations.Directory(), importFiles, importMax);
 }
 
+/**
+Reads the directory properties file depending on the variable
+"properties.directory.enable". Also sets the variable $(SciteDirectoryHome) to the path
+where this property file is found. If it is not found $(SciteDirectoryHome) will
+be set to $(FilePath).
+*/
+void SciTEBase::ReadDirectoryPropFile() {
+	propsDirectory.Clear();
+
+	if (props.GetInt("properties.directory.enable") != 0) {
+		FilePath propfile = GetDirectoryPropertiesFileName();
+		props.Set("SciteDirectoryHome", propfile.Directory().AsFileSystem());
+
+		propsDirectory.Read(propfile, propfile.Directory());
+	}
+}
+
+/**
+Read local and directory properties file.
+*/
 void SciTEBase::ReadLocalPropFile() {
-	FilePath propdir = filePath.Directory();
-	FilePath propfile;
-	propfile.Set(propdir, propFileName);
+	// The directory properties acts like a base local properties file.
+	// Therefore it must be read always before reading the local prop file.
+	ReadDirectoryPropFile();
+
+	FilePath propfile = GetLocalPropertiesFileName();
 
 	propsLocal.Clear();
-	propsLocal.Read(propfile, propdir);
+	propsLocal.Read(propfile, propfile.Directory());
 	//Platform::DebugPrintf("Reading local properties from %s\n", propfile);
 
 	// TODO: Grab these from Platform and update when environment says to
@@ -501,7 +412,9 @@ void SciTEBase::SetStyleFor(Window &win, const char *lang) {
 			char key[200];
 			sprintf(key, "style.%s.%0d", lang, style);
 			SString sval = props.GetExpanded(key);
-			SetOneStyle(win, style, sval.c_str());
+			if (sval.length()) {
+				SetOneStyle(win, style, sval.c_str());
+			}
 		}
 	}
 }
@@ -539,6 +452,8 @@ void SciTEBase::ForwardPropertyToEditor(const char *key) {
 	SString value = props.Get(key);
 	SendEditorString(SCI_SETPROPERTY,
 	                 reinterpret_cast<uptr_t>(key), value.c_str());
+	SendOutputString(SCI_SETPROPERTY,
+	                 reinterpret_cast<uptr_t>(key), value.c_str());
 }
 
 void SciTEBase::DefineMarker(int marker, int markerType, ColourDesired fore, ColourDesired back) {
@@ -634,6 +549,8 @@ static const char *propertiesToForward[] = {
 	"html.tags.case.sensitive",
 	"lexer.caml.magic",
 	"lexer.cpp.allow.dollars",
+	"lexer.d.fold.at.else",
+	"lexer.errorlist.value.separate",
 	"lexer.metapost.comment.process",
 	"lexer.metapost.interface.default",
 	"lexer.sql.backticks.identifier",
@@ -741,6 +658,18 @@ static char *bookmarkBluegem[] = {
 "    1GdddG1    "
 };
 
+SString SciTEBase::GetFileNameProperty(const char *name) {
+	SString namePlusDot = name;
+	namePlusDot.append(".");
+	SString valueForFileName = props.GetNewExpand(namePlusDot.c_str(),
+	        ExtensionFileName().c_str());
+	if (valueForFileName.length() != 0) {
+		return valueForFileName;
+	} else {
+		return props.Get(name);
+	}
+}
+
 void SciTEBase::ReadProperties() {
 	if (extender)
 		extender->Clear();
@@ -821,7 +750,7 @@ void SciTEBase::ReadProperties() {
 
 	wrapStyle = props.GetInt("wrap.style", SC_WRAP_WORD);
 
-	SendEditor(SCI_SETCARETFORE,
+	SendChildren(SCI_SETCARETFORE,
 	           ColourOfProperty(props, "caret.fore", ColourDesired(0, 0, 0)));
 
 	SendEditor(SCI_SETMOUSEDWELLTIME,
@@ -841,8 +770,8 @@ void SciTEBase::ReadProperties() {
 
 	SString findMark = props.Get("find.mark");
 	if (findMark.length()) {
-		SendEditor(SCI_INDICSETFORE, 2, ColourFromString(findMark));
-		SendEditor(SCI_INDICSETSTYLE, 2, INDIC_ROUNDBOX);
+		SendEditor(SCI_INDICSETSTYLE, indicatorMatch, INDIC_ROUNDBOX);
+		SendEditor(SCI_INDICSETFORE, indicatorMatch, ColourFromString(findMark));
 	}
 
 	SString controlCharSymbol = props.Get("control.char.symbol");
@@ -984,8 +913,8 @@ void SciTEBase::ReadProperties() {
 	SendEditor(SCI_SETPRINTMAGNIFICATION, props.GetInt("print.magnification"));
 	SendEditor(SCI_SETPRINTCOLOURMODE, props.GetInt("print.colour.mode"));
 
-	clearBeforeExecute = props.GetInt("clear.before.execute");
-	timeCommands = props.GetInt("time.commands");
+	jobQueue.clearBeforeExecute = props.GetInt("clear.before.execute");
+	jobQueue.timeCommands = props.GetInt("time.commands");
 
 	int blankMarginLeft = props.GetInt("blank.margin.left", 1);
 	int blankMarginRight = props.GetInt("blank.margin.right", 1);
@@ -1010,8 +939,8 @@ void SciTEBase::ReadProperties() {
 	twoPhaseDraw = props.GetInt("two.phase.draw", 1);
 	SendEditor(SCI_SETTWOPHASEDRAW, twoPhaseDraw);
 
-	SendEditor(SCI_SETLAYOUTCACHE, props.GetInt("cache.layout"));
-	SendOutput(SCI_SETLAYOUTCACHE, props.GetInt("output.cache.layout"));
+	SendEditor(SCI_SETLAYOUTCACHE, props.GetInt("cache.layout", SC_CACHE_CARET));
+	SendOutput(SCI_SETLAYOUTCACHE, props.GetInt("output.cache.layout", SC_CACHE_CARET));
 
 	bracesCheck = props.GetInt("braces.check");
 	bracesSloppy = props.GetInt("braces.sloppy");
@@ -1029,6 +958,11 @@ void SciTEBase::ReadProperties() {
 		SendEditorString(SCI_SETWHITESPACECHARS, 0, whitespaceCharacters.c_str());
 	}
 
+	SString viewIndentExamine = GetFileNameProperty("view.indentation.examine");
+	indentExamine = viewIndentExamine.length() ? viewIndentExamine.value() : SC_IV_REAL;
+	SendEditor(SCI_SETINDENTATIONGUIDES, props.GetInt("view.indentation.guides") ?
+		indentExamine : SC_IV_NONE);
+
 	SendEditor(SCI_SETTABINDENTS, props.GetInt("tab.indents", 1));
 	SendEditor(SCI_SETBACKSPACEUNINDENTS, props.GetInt("backspace.unindents", 1));
 
@@ -1173,25 +1107,27 @@ void SciTEBase::ReadProperties() {
 		break;
 	}
 
-	SendEditor(SCI_MARKERSETFORE, SciTE_MARKER_BOOKMARK,
+	SendEditor(SCI_MARKERSETFORE, markerBookmark,
 	           ColourOfProperty(props, "bookmark.fore", ColourDesired(0, 0, 0x7f)));
-	SendEditor(SCI_MARKERSETBACK, SciTE_MARKER_BOOKMARK,
+	SendEditor(SCI_MARKERSETBACK, markerBookmark,
 	           ColourOfProperty(props, "bookmark.back", ColourDesired(0x80, 0xff, 0xff)));
-	SendEditor(SCI_MARKERSETALPHA, SciTE_MARKER_BOOKMARK, props.GetInt("bookmark.alpha", SC_ALPHA_NOALPHA));
+	SendEditor(SCI_MARKERSETALPHA, markerBookmark, props.GetInt("bookmark.alpha", SC_ALPHA_NOALPHA));
 	SString bookMarkXPM = props.Get("bookmark.pixmap");
 	if (bookMarkXPM.length()) {
-		SendEditorString(SCI_MARKERDEFINEPIXMAP, SciTE_MARKER_BOOKMARK,
+		SendEditorString(SCI_MARKERDEFINEPIXMAP, markerBookmark,
 			bookMarkXPM.c_str());
 	} else if (props.Get("bookmark.fore").length()) {
-		SendEditor(SCI_MARKERDEFINE, SciTE_MARKER_BOOKMARK, SC_MARK_CIRCLE);
+		SendEditor(SCI_MARKERDEFINE, markerBookmark, SC_MARK_CIRCLE);
 	} else {
 		// No bookmark.fore setting so display default pixmap.
-		SendEditorString(SCI_MARKERDEFINEPIXMAP, SciTE_MARKER_BOOKMARK,
+		SendEditorString(SCI_MARKERDEFINEPIXMAP, markerBookmark,
 			reinterpret_cast<char *>(bookmarkBluegem));
 	}
 
 	SendEditor(SCI_SETSCROLLWIDTH, props.GetInt("horizontal.scroll.width", 2000));
+	SendEditor(SCI_SETSCROLLWIDTHTRACKING, props.GetInt("horizontal.scroll.width.tracking", 1));
 	SendOutput(SCI_SETSCROLLWIDTH, props.GetInt("output.horizontal.scroll.width", 2000));
+	SendOutput(SCI_SETSCROLLWIDTHTRACKING, props.GetInt("output.horizontal.scroll.width.tracking", 1));
 
 	// Do these last as they force a style refresh
 	SendEditor(SCI_SETHSCROLLBAR, props.GetInt("horizontal.scrollbar", 1));
@@ -1306,13 +1242,13 @@ void SciTEBase::SetPropertiesInitial() {
 	wrapFind = props.GetInt("find.replace.wrap", 1);
 }
 
-SString SciTEBase::LocaliseString(const char *s, bool retainIfNotFound) {
+SString Localization::Text(const char *s, bool retainIfNotFound) {
 	SString translation = s;
 	int ellipseIndicator = translation.remove("...");
 	int accessKeyPresent = translation.remove(menuAccessIndicator);
 	translation.lowercase();
 	translation.substitute("\n", "\\n");
-	translation = propsUI.Get(translation.c_str());
+	translation = Get(translation.c_str());
 	if (translation.length()) {
 		if (ellipseIndicator)
 			translation += "...";
@@ -1334,7 +1270,7 @@ SString SciTEBase::LocaliseString(const char *s, bool retainIfNotFound) {
 		translation.substitute("&", menuAccessIndicator);
 		translation.substitute("\\n", "\n");
 	} else {
-		translation = props.Get("translation.missing");
+		translation = missing;
 	}
 	if ((translation.length() > 0) || !retainIfNotFound) {
 		return translation;
@@ -1343,7 +1279,7 @@ SString SciTEBase::LocaliseString(const char *s, bool retainIfNotFound) {
 }
 
 SString SciTEBase::LocaliseMessage(const char *s, const char *param0, const char *param1, const char *param2) {
-	SString translation = LocaliseString(s);
+	SString translation = localiser.Text(s);
 	if (param0)
 		translation.substitute("^0", param0);
 	if (param1)
@@ -1353,8 +1289,8 @@ SString SciTEBase::LocaliseMessage(const char *s, const char *param0, const char
 	return translation;
 }
 
-void SciTEBase::ReadLocalisation() {
-	propsUI.Clear();
+void SciTEBase::ReadLocalization() {
+	localiser.Clear();
 	const char *title = "locale.properties";
 	SString localeProps = props.GetExpanded(title);
 	if (localeProps.length()) {
@@ -1362,8 +1298,9 @@ void SciTEBase::ReadLocalisation() {
 	}
 	FilePath propdir = GetSciteDefaultHome();
 	FilePath localePath(propdir, title);
-	propsUI.Read(localePath, propdir, importFiles, importMax);
-	localisationRead = true;
+	localiser.Read(localePath, propdir, importFiles, importMax);
+	localiser.SetMissing(props.Get("translation.missing"));
+	localiser.read = true;
 }
 
 void SciTEBase::ReadPropertiesInitial() {
@@ -1381,7 +1318,9 @@ void SciTEBase::ReadPropertiesInitial() {
 		}
 	}
 	ViewWhitespace(props.GetInt("view.whitespace"));
-	SendEditor(SCI_SETINDENTATIONGUIDES, props.GetInt("view.indentation.guides"));
+	SendEditor(SCI_SETINDENTATIONGUIDES, props.GetInt("view.indentation.guides") ?
+		indentExamine : SC_IV_NONE);
+
 	SendEditor(SCI_SETVIEWEOL, props.GetInt("view.eol"));
 	SendEditor(SCI_SETZOOM, props.GetInt("magnification"));
 	SendOutput(SCI_SETZOOM, props.GetInt("output.magnification"));
@@ -1457,183 +1396,71 @@ FilePath SciTEBase::GetUserPropertiesFileName() {
 	return FilePath(GetSciteUserHome(), propUserFileName);
 }
 
+FilePath SciTEBase::GetLocalPropertiesFileName() {
+	return FilePath(filePath.Directory(), propLocalFileName);
+}
+
+FilePath SciTEBase::GetDirectoryPropertiesFileName() {
+	FilePath propfile;
+
+	if (filePath.IsSet()) {
+		propfile.Set(filePath.Directory(), propDirectoryFileName);
+
+		// if this file does not exist try to find the prop file in a parent directory
+		while (!propfile.Directory().IsRoot() && !propfile.Exists()) {
+			propfile.Set(propfile.Directory().Directory(), propDirectoryFileName);
+		}
+
+		// not found -> set it to the initial directory
+		if (!propfile.Exists()) {
+			propfile.Set(filePath.Directory(), propDirectoryFileName);
+		}
+	}
+	return propfile;
+}
+
 void SciTEBase::OpenProperties(int propsFile) {
 	FilePath propfile;
-	if (propsFile == IDM_OPENLOCALPROPERTIES) {
-		FilePath propdir = filePath.Directory();
-		propfile.Set(propdir, propFileName);
+	switch (propsFile) {
+	case IDM_OPENLOCALPROPERTIES:
+		propfile = GetLocalPropertiesFileName();
 		Open(propfile, ofQuiet);
-	} else if (propsFile == IDM_OPENUSERPROPERTIES) {
+		break;
+	case IDM_OPENUSERPROPERTIES:
 		propfile = GetUserPropertiesFileName();
 		Open(propfile, ofQuiet);
-	} else if (propsFile == IDM_OPENABBREVPROPERTIES) {
+		break;
+	case IDM_OPENABBREVPROPERTIES:
 		propfile = pathAbbreviations;
 		Open(propfile, ofQuiet);
-	} else if (propsFile == IDM_OPENGLOBALPROPERTIES) {
+		break;
+	case IDM_OPENGLOBALPROPERTIES:
 		propfile = GetDefaultPropertiesFileName();
 		Open(propfile, ofQuiet);
-	} else if (propsFile == IDM_OPENLUAEXTERNALFILE) {
-		SString extlua = props.GetExpanded("ext.lua.startup.script");
-		if (extlua.length()) {
-			Open(extlua.c_str(), ofQuiet);
+		break;
+	case IDM_OPENLUAEXTERNALFILE: {
+			SString extlua = props.GetExpanded("ext.lua.startup.script");
+			if (extlua.length()) {
+				Open(extlua.c_str(), ofQuiet);
+			}
+			break;
 		}
+	case IDM_OPENDIRECTORYPROPERTIES: {
+			propfile = GetDirectoryPropertiesFileName();
+			bool alreadyExists = propfile.Exists();
+			Open(propfile, ofQuiet);
+			if (!alreadyExists)
+				SaveAsDialog();
+		}
+		break;
 	}
 }
 
 // return the int value of the command name passed in.
 int SciTEBase::GetMenuCommandAsInt(SString commandName) {
-	struct {
-		const char *name;
-		int id;
-	} idmTable[] = {
-//++Autogenerated -- run ../scintilla/src/LexGen.py to regenerate
-//**1:\(\t\t{"\*",\t\*},\n\)
-		{"IDM_MRUFILE",	IDM_MRUFILE},
-		{"IDM_TOOLS",	IDM_TOOLS},
-		{"IDM_BUFFER",	IDM_BUFFER},
-		{"IDM_IMPORT",	IDM_IMPORT},
-		{"IDM_LANGUAGE",	IDM_LANGUAGE},
-		{"IDM_NEW",	IDM_NEW},
-		{"IDM_OPEN",	IDM_OPEN},
-		{"IDM_OPENSELECTED",	IDM_OPENSELECTED},
-		{"IDM_REVERT",	IDM_REVERT},
-		{"IDM_CLOSE",	IDM_CLOSE},
-		{"IDM_SAVE",	IDM_SAVE},
-		{"IDM_SAVEAS",	IDM_SAVEAS},
-		{"IDM_SAVEASHTML",	IDM_SAVEASHTML},
-		{"IDM_SAVEASRTF",	IDM_SAVEASRTF},
-		{"IDM_SAVEASPDF",	IDM_SAVEASPDF},
-		{"IDM_FILER",	IDM_FILER},
-		{"IDM_SAVEASTEX",	IDM_SAVEASTEX},
-		{"IDM_SAVEACOPY",	IDM_SAVEACOPY},
-		{"IDM_SAVEASXML",	IDM_SAVEASXML},
-		{"IDM_MRU_SEP",	IDM_MRU_SEP},
-		{"IDM_PRINTSETUP",	IDM_PRINTSETUP},
-		{"IDM_PRINT",	IDM_PRINT},
-		{"IDM_LOADSESSION",	IDM_LOADSESSION},
-		{"IDM_SAVESESSION",	IDM_SAVESESSION},
-		{"IDM_QUIT",	IDM_QUIT},
-		{"IDM_ENCODING_DEFAULT",	IDM_ENCODING_DEFAULT},
-		{"IDM_ENCODING_UCS2BE",	IDM_ENCODING_UCS2BE},
-		{"IDM_ENCODING_UCS2LE",	IDM_ENCODING_UCS2LE},
-		{"IDM_ENCODING_UTF8",	IDM_ENCODING_UTF8},
-		{"IDM_ENCODING_UCOOKIE",	IDM_ENCODING_UCOOKIE},
-		{"IDM_UNDO",	IDM_UNDO},
-		{"IDM_REDO",	IDM_REDO},
-		{"IDM_CUT",	IDM_CUT},
-		{"IDM_COPY",	IDM_COPY},
-		{"IDM_PASTE",	IDM_PASTE},
-		{"IDM_CLEAR",	IDM_CLEAR},
-		{"IDM_SELECTALL",	IDM_SELECTALL},
-		{"IDM_PASTEANDDOWN",	IDM_PASTEANDDOWN},
-		{"IDM_FIND",	IDM_FIND},
-		{"IDM_FINDNEXT",	IDM_FINDNEXT},
-		{"IDM_FINDNEXTBACK",	IDM_FINDNEXTBACK},
-		{"IDM_FINDNEXTSEL",	IDM_FINDNEXTSEL},
-		{"IDM_FINDNEXTBACKSEL",	IDM_FINDNEXTBACKSEL},
-		{"IDM_FINDINFILES",	IDM_FINDINFILES},
-		{"IDM_REPLACE",	IDM_REPLACE},
-		{"IDM_GOTO",	IDM_GOTO},
-		{"IDM_BOOKMARK_NEXT",	IDM_BOOKMARK_NEXT},
-		{"IDM_BOOKMARK_TOGGLE",	IDM_BOOKMARK_TOGGLE},
-		{"IDM_BOOKMARK_PREV",	IDM_BOOKMARK_PREV},
-		{"IDM_BOOKMARK_CLEARALL",	IDM_BOOKMARK_CLEARALL},
-		{"IDM_BOOKMARK_NEXT_SELECT",	IDM_BOOKMARK_NEXT_SELECT},
-		{"IDM_BOOKMARK_PREV_SELECT",	IDM_BOOKMARK_PREV_SELECT},
-		{"IDM_MATCHBRACE",	IDM_MATCHBRACE},
-		{"IDM_SELECTTOBRACE",	IDM_SELECTTOBRACE},
-		{"IDM_SHOWCALLTIP",	IDM_SHOWCALLTIP},
-		{"IDM_COMPLETE",	IDM_COMPLETE},
-		{"IDM_COMPLETEWORD",	IDM_COMPLETEWORD},
-		{"IDM_EXPAND",	IDM_EXPAND},
-		{"IDM_TOGGLE_FOLDALL",	IDM_TOGGLE_FOLDALL},
-		{"IDM_TOGGLE_FOLDRECURSIVE",	IDM_TOGGLE_FOLDRECURSIVE},
-		{"IDM_EXPAND_ENSURECHILDRENVISIBLE",	IDM_EXPAND_ENSURECHILDRENVISIBLE},
-		{"IDM_UPRCASE",	IDM_UPRCASE},
-		{"IDM_LWRCASE",	IDM_LWRCASE},
-		{"IDM_ABBREV",	IDM_ABBREV},
-		{"IDM_BLOCK_COMMENT",	IDM_BLOCK_COMMENT},
-		{"IDM_STREAM_COMMENT",	IDM_STREAM_COMMENT},
-		{"IDM_COPYASRTF",	IDM_COPYASRTF},
-		{"IDM_BOX_COMMENT",	IDM_BOX_COMMENT},
-		{"IDM_INS_ABBREV",	IDM_INS_ABBREV},
-		{"IDM_JOIN",	IDM_JOIN},
-		{"IDM_SPLIT",	IDM_SPLIT},
-		{"IDM_DUPLICATE",	IDM_DUPLICATE},
-		{"IDM_INCSEARCH",	IDM_INCSEARCH},
-		{"IDM_ENTERSELECTION",	IDM_ENTERSELECTION},
-		{"IDM_PREVMATCHPPC",	IDM_PREVMATCHPPC},
-		{"IDM_SELECTTOPREVMATCHPPC",	IDM_SELECTTOPREVMATCHPPC},
-		{"IDM_NEXTMATCHPPC",	IDM_NEXTMATCHPPC},
-		{"IDM_SELECTTONEXTMATCHPPC",	IDM_SELECTTONEXTMATCHPPC},
-		{"IDM_COMPILE",	IDM_COMPILE},
-		{"IDM_BUILD",	IDM_BUILD},
-		{"IDM_GO",	IDM_GO},
-		{"IDM_STOPEXECUTE",	IDM_STOPEXECUTE},
-		{"IDM_FINISHEDEXECUTE",	IDM_FINISHEDEXECUTE},
-		{"IDM_NEXTMSG",	IDM_NEXTMSG},
-		{"IDM_PREVMSG",	IDM_PREVMSG},
-		{"IDM_MACRO_SEP",	IDM_MACRO_SEP},
-		{"IDM_MACRORECORD",	IDM_MACRORECORD},
-		{"IDM_MACROSTOPRECORD",	IDM_MACROSTOPRECORD},
-		{"IDM_MACROPLAY",	IDM_MACROPLAY},
-		{"IDM_MACROLIST",	IDM_MACROLIST},
-		{"IDM_ACTIVATE",	IDM_ACTIVATE},
-		{"IDM_SRCWIN",	IDM_SRCWIN},
-		{"IDM_RUNWIN",	IDM_RUNWIN},
-		{"IDM_TOOLWIN",	IDM_TOOLWIN},
-		{"IDM_STATUSWIN",	IDM_STATUSWIN},
-		{"IDM_TABWIN",	IDM_TABWIN},
-		{"IDM_SPLITVERTICAL",	IDM_SPLITVERTICAL},
-		{"IDM_VIEWSPACE",	IDM_VIEWSPACE},
-		{"IDM_VIEWEOL",	IDM_VIEWEOL},
-		{"IDM_VIEWGUIDES",	IDM_VIEWGUIDES},
-		{"IDM_SELMARGIN",	IDM_SELMARGIN},
-		{"IDM_FOLDMARGIN",	IDM_FOLDMARGIN},
-		{"IDM_LINENUMBERMARGIN",	IDM_LINENUMBERMARGIN},
-		{"IDM_VIEWTOOLBAR",	IDM_VIEWTOOLBAR},
-		{"IDM_TOGGLEOUTPUT",	IDM_TOGGLEOUTPUT},
-		{"IDM_VIEWTABBAR",	IDM_VIEWTABBAR},
-		{"IDM_VIEWSTATUSBAR",	IDM_VIEWSTATUSBAR},
-		{"IDM_TOGGLEPARAMETERS",	IDM_TOGGLEPARAMETERS},
-		{"IDM_OPENFILESHERE",	IDM_OPENFILESHERE},
-		{"IDM_WRAP",	IDM_WRAP},
-		{"IDM_WRAPOUTPUT",	IDM_WRAPOUTPUT},
-		{"IDM_READONLY",	IDM_READONLY},
-		{"IDM_CLEAROUTPUT",	IDM_CLEAROUTPUT},
-		{"IDM_SWITCHPANE",	IDM_SWITCHPANE},
-		{"IDM_EOL_CRLF",	IDM_EOL_CRLF},
-		{"IDM_EOL_CR",	IDM_EOL_CR},
-		{"IDM_EOL_LF",	IDM_EOL_LF},
-		{"IDM_EOL_CONVERT",	IDM_EOL_CONVERT},
-		{"IDM_TABSIZE",	IDM_TABSIZE},
-		{"IDM_MONOFONT",	IDM_MONOFONT},
-		{"IDM_OPENLOCALPROPERTIES",	IDM_OPENLOCALPROPERTIES},
-		{"IDM_OPENUSERPROPERTIES",	IDM_OPENUSERPROPERTIES},
-		{"IDM_OPENGLOBALPROPERTIES",	IDM_OPENGLOBALPROPERTIES},
-		{"IDM_OPENABBREVPROPERTIES",	IDM_OPENABBREVPROPERTIES},
-		{"IDM_OPENLUAEXTERNALFILE",	IDM_OPENLUAEXTERNALFILE},
-		{"IDM_PREVFILE",	IDM_PREVFILE},
-		{"IDM_NEXTFILE",	IDM_NEXTFILE},
-		{"IDM_CLOSEALL",	IDM_CLOSEALL},
-		{"IDM_SAVEALL",	IDM_SAVEALL},
-		{"IDM_BUFFERSEP",	IDM_BUFFERSEP},
-		{"IDM_PREVFILESTACK",	IDM_PREVFILESTACK},
-		{"IDM_NEXTFILESTACK",	IDM_NEXTFILESTACK},
-		{"IDM_HELP",	IDM_HELP},
-		{"IDM_ABOUT",	IDM_ABOUT},
-		{"IDM_HELP_SCITE",	IDM_HELP_SCITE},
-		{"IDM_ONTOP",	IDM_ONTOP},
-		{"IDM_FULLSCREEN",	IDM_FULLSCREEN},
-
-//--Autogenerated -- end of automatically generated section
-		{0, 0},
-	};
-
-	for (int i=0; idmTable[i].id; i++) {
-		if (commandName == idmTable[i].name) {
-			return idmTable[i].id;
-		}
+	int i = IFaceTable::FindConstant(commandName.c_str());
+	if (i != -1) {
+		return IFaceTable::constants[i].value;
 	}
 	// Otherwise we might have entered a number as command to access a "SCI_" command
 	return commandName.value();
