@@ -97,6 +97,8 @@ long SciTEKeys::ParseKeyCode(const char *mnemonic) {
 				keyval = VK_RETURN;
 			} else if (sKey == "Space") {
 				keyval = VK_SPACE;
+			} else if (sKey == "Tab") {
+				keyval = VK_TAB;
 			} else if (sKey == "Escape") {
 				keyval = VK_ESCAPE;
 			} else if (sKey == "Delete") {
@@ -105,6 +107,10 @@ long SciTEKeys::ParseKeyCode(const char *mnemonic) {
 				keyval = VK_PRIOR;
 			} else if (sKey == "PageDown") {
 				keyval = VK_NEXT;
+			} else if (sKey == "Win") {
+				keyval = VK_LWIN;
+			} else if (sKey == "Menu") {
+				keyval = VK_APPS;
 			}
 		}
 	}
@@ -155,10 +161,12 @@ SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 	propsEmbed.Set("PLAT_WIN", "1");
 	OSVERSIONINFO osv = {sizeof(OSVERSIONINFO), 0, 0, 0, 0, ""};
 	::GetVersionEx(&osv);
+	isWindowsNT = osv.dwPlatformId == VER_PLATFORM_WIN32_NT;
 	if (osv.dwPlatformId == VER_PLATFORM_WIN32_NT)
 		propsEmbed.Set("PLAT_WINNT", "1");
 	else if (osv.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
 		propsEmbed.Set("PLAT_WIN95", "1");
+
 	HRSRC handProps = ::FindResource(hInstance, "Embedded", "Properties");
 	if (handProps) {
 		DWORD size = ::SizeofResource(hInstance, handProps);
@@ -415,6 +423,10 @@ void SciTEWin::Command(WPARAM wParam, LPARAM lParam) {
 		// From accelerator -> goes to focused pane.
 		menuSource = 0;
 	}
+	if (reinterpret_cast<HWND>(lParam) == wToolBar.GetID()) {
+		// From toolbar -> goes to focused pane.
+		menuSource = 0;
+	}
 
 	switch (cmdID) {
 
@@ -434,14 +446,14 @@ void SciTEWin::Command(WPARAM wParam, LPARAM lParam) {
 		break;
 
 	case IDM_FINISHEDEXECUTE: {
-			executing = false;
+			jobQueue.SetExecuting(false);
 			if (needReadProperties)
 				ReadProperties();
 			CheckMenus();
-			for (int icmd = 0; icmd < commandMax; icmd++) {
-				jobQueue[icmd].Clear();
+			for (int icmd = 0; icmd < jobQueue.commandMax; icmd++) {
+				jobQueue.jobQueue[icmd].Clear();
 			}
-			commandCurrent = 0;
+			jobQueue.commandCurrent = 0;
 			CheckReload();
 		}
 		break;
@@ -514,7 +526,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun, bool &seenOutput) {
 	}
 
 	if (jobToRun.jobType == jobGrep) {
-		// jobToRun.command is "(w|~)(c|~)\0files\0text"
+		// jobToRun.command is "(w|~)(c|~)(d|~)(b|~)\0files\0text"
 		const char *grepCmd = jobToRun.command.c_str();
 		GrepFlags gf = grepNone;
 		if (*grepCmd == 'w')
@@ -522,6 +534,12 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun, bool &seenOutput) {
 		grepCmd++;
 		if (*grepCmd == 'c')
 			gf = static_cast<GrepFlags>(gf | grepMatchCase);
+		grepCmd++;
+		if (*grepCmd == 'd')
+			gf = static_cast<GrepFlags>(gf | grepDot);
+		grepCmd++;
+		if (*grepCmd == 'b')
+			gf = static_cast<GrepFlags>(gf | grepBinary);
 		const char *findFiles = grepCmd + 2;
 		const char *findWhat = findFiles + strlen(findFiles) + 1;
 		InternalGrep(gf, jobToRun.directory.AsFileSystem(), findFiles, findWhat);
@@ -643,9 +661,12 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun, bool &seenOutput) {
 				// There is input to transmit to the process.  Do it in small blocks, interleaved
 				// with reads, so that our hRead buffer will not be overrun with results.
 
-				size_t bytesToWrite = jobToRun.input.search("\n", writingPosition) + 1 - writingPosition;
-				if ((bytesToWrite <= 0) || (writingPosition + bytesToWrite >= totalBytesToWrite)) {
+				size_t bytesToWrite;
+				int eol_pos = jobToRun.input.search("\n", writingPosition);
+				if (eol_pos == -1) {
 					bytesToWrite = totalBytesToWrite - writingPosition;
+				} else {
+					bytesToWrite = eol_pos + 1 - writingPosition;
 				}
 				if (bytesToWrite > 250) {
 					bytesToWrite = 250;
@@ -725,7 +746,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun, bool &seenOutput) {
 				}
 			}
 
-			if (::InterlockedExchange(&cancelFlag, 0)) {
+			if (jobQueue.SetCancelFlag(0)) {
 				if (WAIT_OBJECT_0 != ::WaitForSingleObject(pi.hProcess, 500)) {
 					// We should use it only if the GUI process is stuck and
 					// don't answer to a normal termination command.
@@ -746,7 +767,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun, bool &seenOutput) {
 		::GetExitCodeProcess(pi.hProcess, &exitcode);
 		SString sExitMessage(exitcode);
 		sExitMessage.insert(0, ">Exit code: ");
-		if (timeCommands) {
+		if (jobQueue.TimeCommands()) {
 			sExitMessage += "    Time: ";
 			sExitMessage += SString(commandTime.Duration(), 3);
 		}
@@ -810,14 +831,14 @@ void SciTEWin::ProcessExecute() {
 	int originalEnd = SendOutputEx(SCI_GETCURRENTPOS, 0, 0, false);
 	bool seenOutput = false;
 
-	for (int icmd = 0; icmd < commandCurrent && icmd < commandMax && exitcode == 0; icmd++) {
-		exitcode = ExecuteOne(jobQueue[icmd], seenOutput);
-		if (isBuilding) {
+	for (int icmd = 0; icmd < jobQueue.commandCurrent && icmd < jobQueue.commandMax && exitcode == 0; icmd++) {
+		exitcode = ExecuteOne(jobQueue.jobQueue[icmd], seenOutput);
+		if (jobQueue.isBuilding) {
 			// The build command is first command in a sequence so it is only built if
 			// that command succeeds not if a second returns after document is modified.
-			isBuilding = false;
+			jobQueue.isBuilding = false;
 			if (exitcode == 0)
-				isBuilt = true;
+				jobQueue.isBuilt = true;
 		}
 	}
 
@@ -983,7 +1004,7 @@ void SciTEWin::StopExecute() {
 	}
 #endif
 
-	::InterlockedExchange(&cancelFlag, 1L);
+	jobQueue.SetCancelFlag(1);
 }
 
 void SciTEWin::AddCommand(const SString &cmd, const SString &dir, JobSubsystem jobType, const SString &input, int flags) {
@@ -992,7 +1013,7 @@ void SciTEWin::AddCommand(const SString &cmd, const SString &dir, JobSubsystem j
 			SString pCmd = cmd;
 			parameterisedCommand = "";
 			if (pCmd[0] == '*') {
-				pCmd.remove(0);
+				pCmd.remove(0, 1);
 				parameterisedCommand = pCmd;
 				if (!ParametersDialog(true)) {
 					return;
@@ -1405,6 +1426,9 @@ LRESULT SciTEWin::KeyDown(WPARAM wParam) {
 	    (Platform::IsKeyDown(VK_CONTROL) ? SCMOD_CTRL : 0) |
 	    (Platform::IsKeyDown(VK_MENU) ? SCMOD_ALT : 0);
 
+	if (extender && extender->OnKey(wParam, modifiers))
+		return 1l;
+
 	for (int j = 0; j < languageItems; j++) {
 		if (KeyMatch(languageMenu[j].menuKey, wParam, modifiers)) {
 			SciTEBase::MenuCommand(IDM_LANGUAGE + j);
@@ -1456,7 +1480,7 @@ LRESULT SciTEWin::KeyUp(WPARAM wParam) {
 }
 
 void SciTEWin::AddToPopUp(const char *label, int cmd, bool enabled) {
-	SString localised = LocaliseString(label);
+	SString localised = localiser.Text(label);
 	HMENU menu = reinterpret_cast<HMENU>(popup.GetID());
 	if (0 == localised.length())
 		::AppendMenu(menu, MF_SEPARATOR, 0, "");
@@ -1601,6 +1625,10 @@ LRESULT SciTEWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 		QuitProgram();
 		return 0;
 
+	case WM_QUERYENDSESSION:
+		QuitProgram();
+		return 1;
+
 	case WM_DESTROY:
 		break;
 
@@ -1794,14 +1822,6 @@ LRESULT PASCAL SciTEWin::IWndProc(
 		return scite->WndProcI(iMessage, wParam, lParam);
 }
 
-// IsNT() did not work on my machine... :-(
-// I found that Platform_Initialise() was never called
-bool IsWindowsNT() {
-	OSVERSIONINFO osv = {sizeof(OSVERSIONINFO), 0, 0, 0, 0, ""};
-	::GetVersionEx(&osv);
-	return (osv.dwPlatformId == VER_PLATFORM_WIN32_NT);
-}
-
 // from ScintillaWin.cxx
 static UINT CodePageFromCharSet(DWORD characterSet, UINT documentCodePage) {
 	CHARSETINFO ci = { 0, 0, { { 0, 0, 0, 0 }, { 0, 0 } } };
@@ -1825,7 +1845,7 @@ static UINT CodePageFromCharSet(DWORD characterSet, UINT documentCodePage) {
 SString SciTEWin::EncodeString(const SString &s) {
 	//::MessageBox(GetFocus(),SString(s).c_str(),"EncodeString:in",0);
 
-	if (IsWindowsNT()) {
+	if (isWindowsNT) {
 		UINT codePage = SendEditor(SCI_GETCODEPAGE);
 
 		if (codePage != SC_CP_UTF8) {
@@ -1857,9 +1877,7 @@ SString SciTEWin::EncodeString(const SString &s) {
 SString SciTEWin::GetRangeInUIEncoding(Window &win, int selStart, int selEnd) {
 	SString s = SciTEBase::GetRangeInUIEncoding(win, selStart, selEnd);
 
-	//::MessageBox(GetFocus(),s.c_str(),"GetRangeInUIEncoding:in",0);
-
-	if (IsWindowsNT()) {
+	if (isWindowsNT) {
 		UINT codePage = SendEditor(SCI_GETCODEPAGE);
 
 		if (codePage != SC_CP_UTF8) {
@@ -1880,13 +1898,33 @@ SString SciTEWin::GetRangeInUIEncoding(Window &win, int selStart, int selEnd) {
 			delete []pszWide;
 			delete []pszMulti;
 
-			//::MessageBox(GetFocus(),result.c_str(),"GetRangeInUIEncoding:out",0);
 			return result;
 		}
 	}
 	return s;
 }
 
+int SciTEWin::EventLoop() {
+	MSG msg;
+	msg.wParam = 0;
+	bool going = true;
+	while (going) {
+		going = isWindowsNT ? ::GetMessageW(&msg, NULL, 0, 0) : ::GetMessageA(&msg, NULL, 0, 0);
+		if (going) {
+			if (!ModelessHandler(&msg)) {
+				if (::TranslateAccelerator(reinterpret_cast<HWND>(GetID()), GetAcceleratorTable(), &msg) == 0) {
+					::TranslateMessage(&msg);
+					if (isWindowsNT)
+						::DispatchMessageW(&msg);
+					else
+						::DispatchMessageA(&msg);
+				}
+			}
+		}
+	}
+	return msg.wParam;
+}
+
 int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR lpszCmdLine, int) {
 
 #ifdef NO_EXTENSIONS
@@ -1918,23 +1956,11 @@ int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR lpszCmdLine, int) {
 		::MessageBox(NULL, "The Scintilla DLL could not be loaded.  SciTE will now close", "Error loading Scintilla", MB_OK | MB_ICONERROR);
 #endif
 
-	MSG msg;
-	msg.wParam = 0;
+	int result;
 	{
 		SciTEWin MainWind(extender);
 		MainWind.Run(lpszCmdLine);
-		bool going = true;
-		while (going) {
-			going = ::GetMessage(&msg, NULL, 0, 0);
-			if (going) {
-				if (!MainWind.ModelessHandler(&msg)) {
-					if (::TranslateAccelerator(reinterpret_cast<HWND>(MainWind.GetID()), MainWind.GetAcceleratorTable(), &msg) == 0) {
-						::TranslateMessage(&msg);
-						::DispatchMessage(&msg);
-					}
-				}
-			}
-		}
+		result = MainWind.EventLoop();
 	}
 
 #ifdef STATIC_BUILD
@@ -1944,5 +1970,5 @@ int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR lpszCmdLine, int) {
 	::FreeLibrary(hmod);
 #endif
 
-	return msg.wParam;
+	return result;
 }
