@@ -11,7 +11,7 @@
  * Set up properties for FileTime, FileDate, CurrentTime, CurrentDate and FileAttr.
  */
 void SciTEWin::SetFileProperties(
-    PropSet &ps) {			///< Property set to update.
+    PropSetFile &ps) {			///< Property set to update.
 
 	const int TEMP_LEN = 100;
 	char temp[TEMP_LEN];
@@ -72,6 +72,22 @@ void SciTEWin::SetStatusBarText(const char *s) {
 	              SB_SETTEXT, 0, reinterpret_cast<LPARAM>(s));
 }
 
+void SciTEWin::TabInsert(int index, char *title) {
+	TCITEM tie;
+	tie.mask = TCIF_TEXT | TCIF_IMAGE;
+	tie.iImage = -1;
+	tie.pszText = title;
+	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()), TCM_INSERTITEM, (WPARAM)index, (LPARAM)&tie);
+}
+
+void SciTEWin::TabSelect(int index) {
+	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()), TCM_SETCURSEL, (WPARAM)index, (LPARAM)0);
+}
+
+void SciTEWin::RemoveAllTabs() {
+	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()), TCM_DELETEALLITEMS, (WPARAM)0, (LPARAM)0);
+}
+
 /**
  * Manage Windows specific notifications.
  */
@@ -240,7 +256,7 @@ void SciTEWin::Notify(SCNotification *notification) {
 				break;
 			}
 			if (ttext) {
-				SString localised = LocaliseString(ttext);
+				SString localised = localiser.Text(ttext);
 				strcpy(ttt, localised.c_str());
 				pDispInfo->lpszText = ttt;
 			}
@@ -249,7 +265,7 @@ void SciTEWin::Notify(SCNotification *notification) {
 
 	case SCN_CHARADDED:
 		if ((notification->nmhdr.idFrom == IDM_RUNWIN) &&
-		        executing &&
+		        jobQueue.IsExecuting() &&
 		        hWriteSubProcess) {
 			char chToWrite = static_cast<char>(notification->ch);
 			if (chToWrite != '\r') {
@@ -385,10 +401,9 @@ void SciTEWin::SizeSubWindows() {
 void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
                            const char *text, const char *mnemonic) {
 	// On Windows the menu items are modified if they already exist or are created
-	HMENU hmenuBar = ::GetMenu(MainHWND());
-	HMENU hmenu = ::GetSubMenu(hmenuBar, menuNumber);
+	HMENU hmenu = ::GetSubMenu(::GetMenu(MainHWND()), menuNumber);
 	SString sTextMnemonic = text;
-	long keycode;
+	long keycode = 0;
 	if (mnemonic && *mnemonic) {
 		keycode = SciTEKeys::ParseKeyCode(mnemonic);
 		if (keycode) {
@@ -398,8 +413,6 @@ void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
 		// the keycode could be used to make a custom accelerator table
 		// but for now, the menu's item data is used instead for command
 		// tools, and for other menu entries it is just discarded.
-	} else {
-		keycode = 0; //I don't think this is needed in ANSI C++.
 	}
 
 	if (::GetMenuState(hmenu, itemID, MF_BYCOMMAND) == 0xffffffff) {
@@ -422,6 +435,11 @@ void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
 	}
 }
 
+void SciTEWin::RedrawMenu() {
+	// Make previous change visible.
+	::DrawMenuBar(reinterpret_cast<HWND>(wSciTE.GetID()));
+}
+
 void SciTEWin::DestroyMenuItem(int menuNumber, int itemID) {
 	// On Windows menu items are destroyed as they can not be hidden and they can be recreated in any position
 	HMENU hmenuBar = ::GetMenu(MainHWND());
@@ -435,9 +453,9 @@ void SciTEWin::DestroyMenuItem(int menuNumber, int itemID) {
 
 void SciTEWin::CheckAMenuItem(int wIDCheckItem, bool val) {
 	if (val)
-		CheckMenuItem(GetMenu(MainHWND()), wIDCheckItem, MF_CHECKED | MF_BYCOMMAND);
+		CheckMenuItem(::GetMenu(MainHWND()), wIDCheckItem, MF_CHECKED | MF_BYCOMMAND);
 	else
-		CheckMenuItem(GetMenu(MainHWND()), wIDCheckItem, MF_UNCHECKED | MF_BYCOMMAND);
+		CheckMenuItem(::GetMenu(MainHWND()), wIDCheckItem, MF_UNCHECKED | MF_BYCOMMAND);
 }
 
 void EnableButton(HWND wTools, int id, bool enable) {
@@ -449,17 +467,17 @@ void EnableButton(HWND wTools, int id, bool enable) {
 
 void SciTEWin::EnableAMenuItem(int wIDCheckItem, bool val) {
 	if (val)
-		EnableMenuItem(::GetMenu(MainHWND()), wIDCheckItem, MF_ENABLED | MF_BYCOMMAND);
+		::EnableMenuItem(::GetMenu(MainHWND()), wIDCheckItem, MF_ENABLED | MF_BYCOMMAND);
 	else
-		EnableMenuItem(::GetMenu(MainHWND()), wIDCheckItem, MF_DISABLED | MF_GRAYED | MF_BYCOMMAND);
+		::EnableMenuItem(::GetMenu(MainHWND()), wIDCheckItem, MF_DISABLED | MF_GRAYED | MF_BYCOMMAND);
 	::EnableButton(reinterpret_cast<HWND>(wToolBar.GetID()), wIDCheckItem, val);
 }
 
 void SciTEWin::CheckMenus() {
 	SciTEBase::CheckMenus();
-	CheckMenuRadioItem(GetMenu(MainHWND()), IDM_EOL_CRLF, IDM_EOL_LF,
+	CheckMenuRadioItem(::GetMenu(MainHWND()), IDM_EOL_CRLF, IDM_EOL_LF,
 	                   SendEditor(SCI_GETEOLMODE) - SC_EOL_CRLF + IDM_EOL_CRLF, 0);
-	CheckMenuRadioItem(GetMenu(MainHWND()), IDM_ENCODING_DEFAULT, IDM_ENCODING_UCOOKIE,
+	CheckMenuRadioItem(::GetMenu(MainHWND()), IDM_ENCODING_DEFAULT, IDM_ENCODING_UCOOKIE,
 	                   CurrentBuffer()->unicodeMode + IDM_ENCODING_DEFAULT, 0);
 }
 
@@ -537,7 +555,7 @@ void SciTEWin::MakeAccelerator(SString sAccelerator, ACCEL &Accel) {
 //SString SciTEWin::LocaliseAccelerator(const char *pAccelerator, int cmd) {
 SString SciTEWin::LocaliseAccelerator(const char *pAccelerator, int) {
 #ifdef LOCALISE_ACCELERATORS_WORKED
-	SString translation = LocaliseString(pAccelerator, true);
+	SString translation = localiser.Text(pAccelerator, true);
 	int AccelCount = ::CopyAcceleratorTable(hAccTable, NULL, 0);
 	ACCEL *AccelTable = new ACCEL[AccelCount];
 	::CopyAcceleratorTable(hAccTable, AccelTable, AccelCount);
@@ -579,7 +597,7 @@ void SciTEWin::LocaliseMenu(HMENU hmenu) {
 			if (mii.hSubMenu) {
 				LocaliseMenu(mii.hSubMenu);
 			}
-			if (mii.fType == MFT_STRING) {
+			if (mii.fType == MFT_STRING || mii.fType == MFT_RADIOCHECK) {
 				if (mii.dwTypeData) {
 					SString text(mii.dwTypeData);
 					SString accel(mii.dwTypeData);
@@ -591,7 +609,7 @@ void SciTEWin::LocaliseMenu(HMENU hmenu) {
 					} else {
 						accel = "";
 					}
-					text = LocaliseString(text.c_str(), true);
+					text = localiser.Text(text.c_str(), true);
 					if (text.length()) {
 						if (accel != "") {
 							text += "\t";
@@ -630,7 +648,7 @@ void SciTEWin::LocaliseAccelerators() {
 void SciTEWin::LocaliseControl(HWND w) {
 	char wtext[200];
 	if (::GetWindowText(w, wtext, sizeof(wtext))) {
-		SString text = LocaliseString(wtext, false);
+		SString text = localiser.Text(wtext, false);
 		if (text.length())
 			::SetWindowText(w, text.c_str());
 	}
@@ -653,8 +671,6 @@ void SciTEWin::LocaliseDialog(HWND wDialog) {
 #define TB_LOADIMAGES (WM_USER + 50)
 #endif
 
-#define ELEMENTS(a)	(sizeof(a) / sizeof(a[0]))
-
 struct BarButton {
 	int id;
 	int cmd;
