@@ -123,9 +123,11 @@ void SciTEWin::WarnUser(int warnID) {
 	PlayThisSound(sound, atoi(soundDuration), hMM);
 }
 
-bool DialogHandled(WindowID id, MSG *pmsg) {
+bool SciTEWin::DialogHandled(WindowID id, MSG *pmsg) {
 	if (id) {
-		if (::IsDialogMessage(reinterpret_cast<HWND>(id), pmsg))
+		if (isWindowsNT ?
+			::IsDialogMessageW(reinterpret_cast<HWND>(id), pmsg) :
+			::IsDialogMessageA(reinterpret_cast<HWND>(id), pmsg))
 			return true;
 	}
 	return false;
@@ -188,7 +190,7 @@ bool SciTEWin::OpenDialog(FilePath directory, const char *filter) {
 				next += strlen(openFilter.c_str() + next) + 1;
 				openFilter.remove(start, next - start);
 			} else {
-				SString localised = LocaliseString(filterName, false);
+				SString localised = localiser.Text(filterName, false);
 				if (localised.length()) {
 					openFilter.remove(start, strlen(filterName));
 					openFilter.insert(start, localised.c_str());
@@ -200,7 +202,7 @@ bool SciTEWin::OpenDialog(FilePath directory, const char *filter) {
 	}
 
 	if (!openWhat[0]) {
-		strcpy(openWhat, LocaliseString("Custom Filter").c_str());
+		strcpy(openWhat, localiser.Text("Custom Filter").c_str());
 		openWhat[strlen(openWhat) + 1] = '\0';
 	}
 
@@ -219,7 +221,7 @@ bool SciTEWin::OpenDialog(FilePath directory, const char *filter) {
 	ofn.lpstrCustomFilter = openWhat;
 	ofn.nMaxCustFilter = sizeof(openWhat);
 	ofn.nFilterIndex = filterDefault;
-	SString translatedTitle = LocaliseString("Open File");
+	SString translatedTitle = localiser.Text("Open File");
 	ofn.lpstrTitle = translatedTitle.c_str();
 	if (props.GetInt("open.dialog.in.file.directory")) {
 		ofn.lpstrInitialDir = directory.AsFileSystem();
@@ -267,7 +269,7 @@ FilePath SciTEWin::ChooseSaveName(FilePath directory, const char *title, const c
 		ofn.hInstance = hInstance;
 		ofn.lpstrFile = saveName;
 		ofn.nMaxFile = sizeof(saveName);
-		SString translatedTitle = LocaliseString(title);
+		SString translatedTitle = localiser.Text(title);
 		ofn.lpstrTitle = translatedTitle.c_str();
 		ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
 		ofn.lpstrFilter = filter;
@@ -356,8 +358,8 @@ void SciTEWin::LoadSessionDialog() {
 	ofn.hInstance = hInstance;
 	ofn.lpstrFile = openName;
 	ofn.nMaxFile = sizeof(openName);
-	ofn.lpstrFilter = "Session (.ses)\0*.ses\0";
-	SString translatedTitle = LocaliseString("Load Session");
+	ofn.lpstrFilter = "Session (.session)\0*.session\0";
+	SString translatedTitle = localiser.Text("Load Session");
 	ofn.lpstrTitle = translatedTitle.c_str();
 	ofn.Flags = OFN_HIDEREADONLY;
 	if (::GetOpenFileName(&ofn))
@@ -366,18 +368,19 @@ void SciTEWin::LoadSessionDialog() {
 
 void SciTEWin::SaveSessionDialog() {
 	char saveName[MAX_PATH] = "\0";
-	strcpy(saveName, "SciTE.ses");
+	strcpy(saveName, "SciTE.session");
 	OPENFILENAME ofn = {
 			       sizeof(OPENFILENAME), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 			   };
 	ofn.hwndOwner = MainHWND();
 	ofn.hInstance = hInstance;
+	ofn.lpstrDefExt = "session";
 	ofn.lpstrFile = saveName;
 	ofn.nMaxFile = sizeof(saveName);
-	SString translatedTitle = LocaliseString("Save Current Session");
+	SString translatedTitle = localiser.Text("Save Current Session");
 	ofn.lpstrTitle = translatedTitle.c_str();
 	ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
-	ofn.lpstrFilter = "Session (.ses)\0*.ses\0";
+	ofn.lpstrFilter = "Session (.session)\0*.session\0";
 	if (::GetSaveFileName(&ofn)) {
 		SaveSession(saveName);
 	}
@@ -603,7 +606,7 @@ void SciTEWin::Print(
 	// Print each page
 	int pageNum = 1;
 	bool printPage;
-	PropSet propsPrint;
+	PropSetFile propsPrint;
 	propsPrint.superPS = &props;
 	SetFileProperties(propsPrint);
 
@@ -722,8 +725,6 @@ void SciTEWin::PrintSetup() {
 	hDevNames = pdlg.hDevNames;
 }
 
-extern bool IsWindowsNT();
-
 // This is a reasonable buffer size for dialog box text conversions
 #define CTL_TEXT_BUF 512
 
@@ -829,7 +830,7 @@ public:
 
 };
 
-static void FillComboFromProps(HWND combo, PropSet &props) {
+static void FillComboFromProps(HWND combo, PropSetFile &props) {
 	char *key;
 	char *val;
 	if (props.GetFirst(&key, &val)) {
@@ -1054,28 +1055,15 @@ BOOL SciTEWin::IncrementFindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		PRectangle aNewRect = aTBRect;
 		aNewRect.top = aNewRect.bottom - (aRect.bottom - aRect.top);
 		aNewRect.right = aNewRect.left + aRect.right - aRect.left;
-		//~ RegisterHotKey(hDlg,1,0,VK_F3);
-		//~ RegisterHotKey(hDlg,2,MOD_SHIFT,VK_F3);
-
 		wFindIncrement.SetPosition(aNewRect);
 
-
 		return TRUE;
 	}
 
 	case WM_SETFOCUS:
 		return 0;
 
-	case WM_HOTKEY:
-		if (wParam == 1)
-			FindNext(false,false);
-		if (wParam == 2)
-			FindNext(true,false);
-		break;
-
 	case WM_CLOSE:
-		//~ UnregisterHotKey(hDlg,1);
-		//~ UnregisterHotKey(hDlg,2);
 		::SendMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
 		break;
 
@@ -1122,8 +1110,10 @@ BOOL CALLBACK SciTEWin::FindIncrementDlg(HWND hDlg, UINT message, WPARAM wParam,
 }
 
 void SciTEWin::FindIncrement() {
-	if (wFindIncrement.Created())
+	if (wFindIncrement.Created()) {
+		wFindIncrement.Destroy();
 		return;
+	}
 
 	memset(&fr, 0, sizeof(fr));
 	fr.lStructSize = sizeof(fr);
@@ -1137,20 +1127,20 @@ void SciTEWin::FindIncrement() {
 	fr.wFindWhatLen = static_cast<WORD>(findWhat.length() + 1);
 
 	replacing = false;
-	//DoDialog(hInstance,
-	//		 MAKEINTRESOURCE(IDD_FIND2),
-	//		 MainHWND(),
-	//		 reinterpret_cast<DLGPROC>(FindIncrementDlg));
-	if (IsWindowsNT()) {
-		::DialogBoxParamW(hInstance, (LPCWSTR)MAKEINTRESOURCE(IDD_FIND2),
-		                MainHWND(), reinterpret_cast<DLGPROC>(FindIncrementDlg),
-		                reinterpret_cast<LPARAM>(this));
+	if (isWindowsNT) {
+		::CreateDialogParamW(hInstance,
+		                                    (LPCWSTR)MAKEINTRESOURCE(IDD_FIND2),
+		                                    MainHWND(),
+		                                    reinterpret_cast<DLGPROC>(FindIncrementDlg),
+		                                    reinterpret_cast<LPARAM>(this));
 	} else {
-		::DialogBoxParamA(hInstance, MAKEINTRESOURCE(IDD_FIND2),
-		                MainHWND(), reinterpret_cast<DLGPROC>(FindIncrementDlg),
-		                reinterpret_cast<LPARAM>(this));
+		::CreateDialogParamA(hInstance,
+		                                    MAKEINTRESOURCE(IDD_FIND2),
+		                                    MainHWND(),
+		                                    reinterpret_cast<DLGPROC>(FindIncrementDlg),
+		                                    reinterpret_cast<LPARAM>(this));
 	}
-	WindowSetFocus(wEditor);
+	wFindIncrement.Show();
 }
 
 bool SciTEWin::FindReplaceAdvanced() {
@@ -1175,7 +1165,7 @@ void SciTEWin::Find() {
 	fr.wFindWhatLen = static_cast<WORD>(findWhat.length() + 1);
 	int dialog_id = FindReplaceAdvanced() ? IDD_FIND_ADV : IDD_FIND;
 
-	if (IsWindowsNT()) {
+	if (isWindowsNT) {
 		wFindReplace = ::CreateDialogParamW(hInstance,
 		                                    (LPCWSTR)MAKEINTRESOURCE(dialog_id),
 		                                    MainHWND(),
@@ -1216,11 +1206,13 @@ void SciTEWin::PerformGrep() {
 	SString findCommand = props.GetNewExpand("find.command");
 	if (findCommand == "") {
 		// Call InternalGrep in a new thread
-		// searchParams is "(w|~)(c|~)\0files\0text"
-		// A "w" indicates whole word and a "c" indicates case sensitive
+		// searchParams is "(w|~)(c|~)(d|~)(b|~)\0files\0text"
+		// A "w" indicates whole word, "c" case sensitive, "d" dot directories, "b" binary files
 		SString searchParams;
 		searchParams.append(wholeWord ? "w" : "~");
 		searchParams.append(matchCase ? "c" : "~");
+		searchParams.append(props.GetInt("find.in.dot") ? "d" : "~");
+		searchParams.append(props.GetInt("find.in.binary") ? "b" : "~");
 		searchParams.append("\0", 1);
 		searchParams.append(props.Get("find.files").c_str());
 		searchParams.append("\0", 1);
@@ -1231,7 +1223,7 @@ void SciTEWin::PerformGrep() {
 			   props.Get("find.directory"),
 			   jobCLI, findInput, flags);
 	}
-	if (commandCurrent > 0) {
+	if (jobQueue.commandCurrent > 0) {
 		Execute();
 	}
 }
@@ -1324,7 +1316,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				info.pidlRoot = NULL;
 				char szDisplayName[MAX_PATH];
 				info.pszDisplayName = szDisplayName;
-				SString title = LocaliseString("Select a folder to search from");
+				SString title = localiser.Text("Select a folder to search from");
 				info.lpszTitle = title.c_str();
 				info.ulFlags = 0;
 				info.lpfn = BrowseCallbackProc;
@@ -1389,7 +1381,7 @@ void SciTEWin::Replace() {
 	fr.wReplaceWithLen = static_cast<WORD>(replaceWhat.length() + 1);
 	int dialog_id = (!props.GetInt("find.replace.advanced") ? IDD_REPLACE : IDD_REPLACE_ADV);
 
-	if (IsWindowsNT()) {
+	if (isWindowsNT) {
 		wFindReplace = ::CreateDialogParamW(hInstance,
 		                                    (LPCWSTR)MAKEINTRESOURCE(dialog_id),
 		                                    MainHWND(),
@@ -1561,7 +1553,8 @@ BOOL SciTEWin::TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		if (ControlIDOfCommand(wParam) == IDCANCEL) {
 			::EndDialog(hDlg, IDCANCEL);
 			return FALSE;
-		} else if (ControlIDOfCommand(wParam) == IDOK) {
+		} else if ((ControlIDOfCommand(wParam) == IDCONVERT) ||
+			(ControlIDOfCommand(wParam) == IDOK)) {
 			BOOL bOK;
 			int tabSize = static_cast<int>(::GetDlgItemInt(hDlg, IDTABSIZE, &bOK, FALSE));
 			if (tabSize > 0)
@@ -1571,7 +1564,10 @@ BOOL SciTEWin::TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				SendEditor(SCI_SETINDENT, indentSize);
 			bool useTabs = static_cast<bool>(::IsDlgButtonChecked(hDlg, IDUSETABS));
 			SendEditor(SCI_SETUSETABS, useTabs);
-			::EndDialog(hDlg, IDOK);
+			if (ControlIDOfCommand(wParam) == IDCONVERT) {
+				ConvertIndentation(tabSize, useTabs);
+			}
+			::EndDialog(hDlg, ControlIDOfCommand(wParam));
 			return TRUE;
 		}
 	}
@@ -1588,6 +1584,10 @@ void SciTEWin::TabSizeDialog() {
 	WindowSetFocus(wEditor);
 }
 
+bool SciTEWin::ParametersOpen() {
+	return wParameters.Created();
+}
+
 void SciTEWin::ParamGrab() {
 	if (wParameters.Created()) {
 		HWND hDlg = reinterpret_cast<HWND>(wParameters.GetID());
@@ -1689,9 +1689,9 @@ void SciTEWin::FindMessageBox(const SString &msg, const SString *findItem) {
 		SString msgBuf = LocaliseMessage(msg.c_str());
 		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf, MB_OK | MB_ICONWARNING);
 	} else {
-		if (IsWindowsNT()) {
+		if (isWindowsNT) {
 
-			SString sFormat = LocaliseString(msg.c_str());
+			SString sFormat = localiser.Text(msg.c_str());
 			SString sPart1 = sFormat.substr(0, sFormat.search("^0", 0));
 			SString sPart2 = sFormat.substr(sFormat.search("^0", 0));
 			sPart2 = sPart2.substr(2); // skip initial ^0
